@@ -31,6 +31,29 @@ NEUTRON_NET=ironic_grenade
 
 set -o xtrace
 
+function wait_for_ironic_resources {
+    local i
+    local nodes_count
+    nodes_count=$(openstack baremetal node list -f value -c "Provisioning State" | wc -l)
+    echo_summary "Waiting 5 minutes for Ironic resources become available again"
+    for i in $(seq 1 30); do
+        if openstack baremetal node list -f value -c "Provisioning State" | grep -qi failed; then
+            die $LINENO "One of nodes is in failed state."
+        fi
+        if [[ $(openstack baremetal node list -f value -c "Provisioning State" | grep -ci available) == $nodes_count ]]; then
+            return 0
+        fi
+        sleep 10
+    done
+    openstack baremetal node list
+    die $LINENO "Timed out waiting for Ironic nodes are available again."
+}
+
+total_cpus=$(( $IRONIC_VM_SPECS_CPU * $IRONIC_VM_COUNT ))
+
+if [[ "${HOST_TOPOLOGY}" == "multinode" ]]; then
+    total_cpus=$(( 2 * $total_cpus ))
+fi
 
 function early_create {
     # We need these steps only in case of flat-network
@@ -93,7 +116,15 @@ function create {
 }
 
 function verify {
-    :
+    local side="$1"
+
+    if [[ "$side" = "post-upgrade" ]]; then
+        nodes=$(openstack --os-baremetal-api-version 1.9 baremetal node list --provision-state active -f value -c UUID)
+        # Trigger nova flavor migration code for active instances.
+        for node_id in $nodes; do
+            openstack --os-baremetal-api-version 1.21 baremetal node set $node_id --resource-class baremetal
+        done
+    fi
 }
 
 function verify_noapi {
@@ -125,23 +156,25 @@ function destroy {
 # Dispatcher
 case $1 in
     "early_create")
+        wait_for_ironic_resources
+        wait_for_nova_resources "vcpus" $total_cpus
         early_create
         ;;
     "create")
         create
         ;;
     "verify_noapi")
-        verify_noapi
+        # NOTE(vdrok): our implementation of verify_noapi is a noop, but
+        # grenade always passes the upgrade side (pre-upgrade or post-upgrade)
+        # as an argument to it. Pass all the arguments grenade passes further.
+        verify_noapi "${@:2}"
         ;;
     "verify")
-        verify
+        # NOTE(vdrok): pass all the arguments grenade passes further.
+        verify "${@:2}"
         ;;
     "destroy")
         destroy
         ;;
-    "force_destroy")
-        set +o errexit
-        destroy
-        ;;
 esac
 
