@@ -35,6 +35,7 @@ from ironic.api.controllers.v1 import portgroup
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api.controllers.v1 import versions
+from ironic.api.controllers.v1 import volume
 from ironic.api import expose
 from ironic.common import exception
 from ironic.common.i18n import _
@@ -152,6 +153,9 @@ def hide_fields_in_newer_versions(obj):
         for field in api_utils.V31_FIELDS:
             setattr(obj, field, wsme.Unset)
 
+    if not api_utils.allow_storage_interface():
+        obj.storage_interface = wsme.Unset
+
 
 def update_state_in_older_versions(obj):
     """Change provision state names for API backwards compatibility.
@@ -813,6 +817,9 @@ class Node(base.APIBase):
     portgroups = wsme.wsattr([link.Link], readonly=True)
     """Links to the collection of portgroups on this node"""
 
+    volume = wsme.wsattr([link.Link], readonly=True)
+    """Links to endpoint for retrieving volume resources on this node"""
+
     states = wsme.wsattr([link.Link], readonly=True)
     """Links to endpoint for retrieving and setting node states"""
 
@@ -840,6 +847,9 @@ class Node(base.APIBase):
     raid_interface = wsme.wsattr(wtypes.text)
     """The raid interface to be used for this node"""
 
+    storage_interface = wsme.wsattr(wtypes.text)
+    """The storage interface to be used for this node"""
+
     vendor_interface = wsme.wsattr(wtypes.text)
     """The vendor interface to be used for this node"""
 
@@ -869,7 +879,7 @@ class Node(base.APIBase):
 
     @staticmethod
     def _convert_with_links(node, url, fields=None, show_states_links=True,
-                            show_portgroups=True):
+                            show_portgroups=True, show_volume=True):
         # NOTE(lucasagomes): Since we are able to return a specified set of
         # fields the "uuid" can be unset, so we need to save it in another
         # variable to use when building the links
@@ -897,6 +907,14 @@ class Node(base.APIBase):
                                         node_uuid + "/portgroups",
                                         bookmark=True)]
 
+            if show_volume:
+                node.volume = [
+                    link.Link.make_link('self', url, 'nodes',
+                                        node_uuid + "/volume"),
+                    link.Link.make_link('bookmark', url, 'nodes',
+                                        node_uuid + "/volume",
+                                        bookmark=True)]
+
         # NOTE(lucasagomes): The numeric ID should not be exposed to
         #                    the user, it's internal only.
         node.chassis_id = wtypes.Unset
@@ -951,10 +969,12 @@ class Node(base.APIBase):
         show_states_links = (
             api_utils.allow_links_node_states_and_driver_properties())
         show_portgroups = api_utils.allow_portgroups_subcontrollers()
+        show_volume = api_utils.allow_volume()
         return cls._convert_with_links(node, pecan.request.public_url,
                                        fields=fields,
                                        show_states_links=show_states_links,
-                                       show_portgroups=show_portgroups)
+                                       show_portgroups=show_portgroups,
+                                       show_volume=show_volume)
 
     @classmethod
     def sample(cls, expand=True):
@@ -981,7 +1001,8 @@ class Node(base.APIBase):
                      boot_interface=None, console_interface=None,
                      deploy_interface=None, inspect_interface=None,
                      management_interface=None, power_interface=None,
-                     raid_interface=None, vendor_interface=None)
+                     raid_interface=None, vendor_interface=None,
+                     storage_interface=None)
         # NOTE(matty_dubs): The chassis_uuid getter() is based on the
         # _chassis_uuid variable:
         sample._chassis_uuid = 'edcad704-b2da-41d5-96d9-afd580ecfa12'
@@ -1183,7 +1204,7 @@ class NodeVIFController(rest.RestController):
     def post(self, vif):
         """Attach a VIF to this node
 
-        :param vif_info: a dictionary of information about a VIF.
+        :param vif: a dictionary of information about a VIF.
             It must have an 'id' key, whose value is a unique identifier
             for that VIF.
         """
@@ -1247,6 +1268,7 @@ class NodesController(rest.RestController):
         'ports': port.PortsController,
         'portgroups': portgroup.PortgroupsController,
         'vifs': NodeVIFController,
+        'volume': volume.VolumeController,
     }
 
     @pecan.expose()
@@ -1442,6 +1464,7 @@ class NodesController(rest.RestController):
 
         api_utils.check_allow_specify_fields(fields)
         api_utils.check_allowed_fields(fields)
+        api_utils.check_allowed_fields([sort_key])
         api_utils.check_for_invalid_state_and_allow_filter(provision_state)
         api_utils.check_allow_specify_driver(driver)
         api_utils.check_allow_specify_resource_class(resource_class)
@@ -1495,6 +1518,7 @@ class NodesController(rest.RestController):
         api_utils.check_for_invalid_state_and_allow_filter(provision_state)
         api_utils.check_allow_specify_driver(driver)
         api_utils.check_allow_specify_resource_class(resource_class)
+        api_utils.check_allowed_fields([sort_key])
         # /detail should only work against collections
         parent = pecan.request.path.split('/')[:-1][-1]
         if parent != "nodes":
@@ -1585,6 +1609,10 @@ class NodesController(rest.RestController):
                 if getattr(node, field) is not wsme.Unset:
                     raise exception.NotAcceptable()
 
+        if (not api_utils.allow_storage_interface() and
+                node.storage_interface is not wtypes.Unset):
+            raise exception.NotAcceptable()
+
         # NOTE(deva): get_topic_for checks if node.driver is in the hash ring
         #             and raises NoValidHost if it is not.
         #             We need to ensure that node has a UUID before it can
@@ -1649,6 +1677,10 @@ class NodesController(rest.RestController):
                 if api_utils.get_patch_values(patch, '/%s' % field):
                     raise exception.NotAcceptable()
 
+        s_interface = api_utils.get_patch_values(patch, '/storage_interface')
+        if s_interface and not api_utils.allow_storage_interface():
+            raise exception.NotAcceptable()
+
         rpc_node = api_utils.get_rpc_node(node_ident)
 
         remove_inst_uuid_patch = [{'op': 'remove', 'path': '/instance_uuid'}]
