@@ -54,6 +54,9 @@ def hide_fields_in_newer_versions(obj):
     # if requested version is < 1.24, hide portgroup_uuid field
     if not api_utils.allow_portgroups_subcontrollers():
         obj.portgroup_uuid = wsme.Unset
+    # if requested version is < 1.34, hide physical_network field.
+    if not api_utils.allow_port_physical_network():
+        obj.physical_network = wsme.Unset
 
 
 class Port(base.APIBase):
@@ -145,6 +148,9 @@ class Port(base.APIBase):
     local_link_connection = types.locallinkconnectiontype
     """The port binding profile for the port"""
 
+    physical_network = wtypes.StringType(max_length=64)
+    """The name of the physical network to which this port is connected."""
+
     links = wsme.wsattr([link.Link], readonly=True)
     """A list containing a self link and associated port links"""
 
@@ -224,7 +230,8 @@ class Port(base.APIBase):
                      pxe_enabled=True,
                      local_link_connection={
                          'switch_info': 'host', 'port_id': 'Gig0/1',
-                         'switch_id': 'aa:bb:cc:dd:ee:ff'})
+                         'switch_id': 'aa:bb:cc:dd:ee:ff'},
+                     physical_network='physnet1')
         # NOTE(lucasagomes): node_uuid getter() method look at the
         # _node_uuid variable
         sample._node_uuid = '7ae81bb3-dec3-4289-8d6c-da80bd8001ae'
@@ -354,6 +361,27 @@ class PortsController(rest.RestController):
         except exception.PortNotFound:
             return []
 
+    def _check_allowed_port_fields(self, fields):
+        """Check if fetching a particular field of a port is allowed.
+
+        Check if the required version is being requested for fields
+        that are only allowed to be fetched in a particular API version.
+
+        :param fields: list or set of fields to check
+        :raises: NotAcceptable if a field is not allowed
+        """
+        if fields is None:
+            return
+        if (not api_utils.allow_port_advanced_net_fields() and
+                set(fields).intersection(self.advanced_net_fields)):
+            raise exception.NotAcceptable()
+        if ('portgroup_uuid' in fields and not
+                api_utils.allow_portgroups_subcontrollers()):
+            raise exception.NotAcceptable()
+        if ('physical_network' in fields and not
+                api_utils.allow_port_physical_network()):
+            raise exception.NotAcceptable()
+
     @METRICS.timer('PortsController.get_all')
     @expose.expose(PortCollection, types.uuid_or_name, types.uuid,
                    types.macaddress, types.uuid, int, wtypes.text,
@@ -389,14 +417,8 @@ class PortsController(rest.RestController):
         policy.authorize('baremetal:port:get', cdict, cdict)
 
         api_utils.check_allow_specify_fields(fields)
-        if fields:
-            if (not api_utils.allow_port_advanced_net_fields() and
-                    set(fields).intersection(self.advanced_net_fields)):
-                raise exception.NotAcceptable()
-            if ('portgroup_uuid' in fields and not
-                    api_utils.allow_portgroups_subcontrollers()):
-                    raise exception.NotAcceptable()
-
+        self._check_allowed_port_fields(fields)
+        self._check_allowed_port_fields([sort_key])
         if portgroup and not api_utils.allow_portgroups_subcontrollers():
             raise exception.NotAcceptable()
 
@@ -446,6 +468,7 @@ class PortsController(rest.RestController):
         cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:port:get', cdict, cdict)
 
+        self._check_allowed_port_fields([sort_key])
         if portgroup and not api_utils.allow_portgroups_subcontrollers():
             raise exception.NotAcceptable()
 
@@ -484,6 +507,7 @@ class PortsController(rest.RestController):
             raise exception.OperationNotPermitted()
 
         api_utils.check_allow_specify_fields(fields)
+        self._check_allowed_port_fields(fields)
 
         rpc_port = objects.Port.get_by_uuid(pecan.request.context, port_uuid)
         return Port.convert_with_links(rpc_port, fields=fields)
@@ -504,17 +528,24 @@ class PortsController(rest.RestController):
             raise exception.OperationNotPermitted()
 
         pdict = port.as_dict()
-        if (not api_utils.allow_port_advanced_net_fields() and
-                set(pdict).intersection(self.advanced_net_fields)):
-            raise exception.NotAcceptable()
-        if (not api_utils.allow_portgroups_subcontrollers() and
-            'portgroup_uuid' in pdict):
+        self._check_allowed_port_fields(pdict)
+
+        create_remotely = pecan.request.rpcapi.can_send_create_port()
+        if (not create_remotely and pdict.get('portgroup_uuid')):
+            # NOTE(mgoddard): In RPC API v1.41, port creation was moved to the
+            # conductor service to facilitate validation of the physical
+            # network field of ports in portgroups. During a rolling upgrade,
+            # the RPCAPI will reject the create_port method, so we need to
+            # create the port locally. If the port is a member of a portgroup,
+            # we are unable to perform the validation and must reject the
+            # request.
             raise exception.NotAcceptable()
 
         extra = pdict.get('extra')
         vif = extra.get('vif_port_id') if extra else None
         if vif:
             common_utils.warn_about_deprecated_extra_vif_port_id()
+
         if (pdict.get('portgroup_uuid') and
                 (pdict.get('pxe_enabled') or vif)):
             rpc_pg = objects.Portgroup.get_by_uuid(context,
@@ -531,15 +562,27 @@ class PortsController(rest.RestController):
         if not pdict.get('uuid'):
             pdict['uuid'] = uuidutils.generate_uuid()
 
-        new_port = objects.Port(context, **pdict)
+        rpc_port = objects.Port(context, **pdict)
+        rpc_node = objects.Node.get_by_id(context, rpc_port.node_id)
 
         notify_extra = {'node_uuid': port.node_uuid,
                         'portgroup_uuid': port.portgroup_uuid}
-        notify.emit_start_notification(context, new_port, 'create',
+        notify.emit_start_notification(context, rpc_port, 'create',
                                        **notify_extra)
-        with notify.handle_error_notification(context, new_port, 'create',
+        with notify.handle_error_notification(context, rpc_port, 'create',
                                               **notify_extra):
-            new_port.create()
+            # NOTE(mgoddard): In RPC API v1.41, port creation was moved to the
+            # conductor service to facilitate validation of the physical
+            # network field of ports in portgroups. During a rolling upgrade,
+            # the RPCAPI will reject the create_port method, so we need to
+            # create the port locally.
+            if create_remotely:
+                topic = pecan.request.rpcapi.get_topic_for(rpc_node)
+                new_port = pecan.request.rpcapi.create_port(context, rpc_port,
+                                                            topic)
+            else:
+                rpc_port.create()
+                new_port = rpc_port
         notify.emit_end_notification(context, new_port, 'create',
                                      **notify_extra)
         # Set the HTTP Location Header
@@ -564,17 +607,13 @@ class PortsController(rest.RestController):
             raise exception.OperationNotPermitted()
 
         fields_to_check = set()
-        for field in self.advanced_net_fields + ['portgroup_uuid']:
+        for field in (self.advanced_net_fields +
+                      ['portgroup_uuid', 'physical_network']):
             field_path = '/%s' % field
             if (api_utils.get_patch_values(patch, field_path) or
                     api_utils.is_path_removed(patch, field_path)):
                 fields_to_check.add(field)
-        if (fields_to_check.intersection(self.advanced_net_fields) and
-                not api_utils.allow_port_advanced_net_fields()):
-            raise exception.NotAcceptable()
-        if ('portgroup_uuid' in fields_to_check and
-                not api_utils.allow_portgroups_subcontrollers()):
-            raise exception.NotAcceptable()
+        self._check_allowed_port_fields(fields_to_check)
 
         rpc_port = objects.Port.get_by_uuid(context, port_uuid)
         try:
