@@ -27,7 +27,7 @@ from oslo_serialization import jsonutils
 import six
 from six.moves import http_client
 
-from ironic.common.i18n import _, _LE
+from ironic.common.i18n import _
 from ironic.conf import CONF
 
 LOG = logging.getLogger(__name__)
@@ -62,16 +62,16 @@ def _ensure_exception_kwargs_serializable(exc_class_name, kwargs):
                     {'serializer_type': msg, 'e_contents': e,
                      'e_type': e.__class__.__name__})
     if exceptions:
-        LOG.error(
-            _LE("One or more arguments passed to the %(exc_class)s "
-                "constructor as kwargs can not be serialized. The serialized "
-                "arguments: %(serialized)s. These unserialized kwargs were "
-                "dropped because of the exceptions encountered during their "
-                "serialization:\n%(errors)s"),
-            dict(errors=';\n'.join("%s: %s" % (k, '; '.join(v))
-                                   for k, v in exceptions.items()),
-                 exc_class=exc_class_name, serialized=serializable_kwargs)
-        )
+        LOG.error("One or more arguments passed to the %(exc_class)s "
+                  "constructor as kwargs can not be serialized. The "
+                  "serialized arguments: %(serialized)s. These "
+                  "unserialized kwargs were dropped because of the "
+                  "exceptions encountered during their "
+                  "serialization:\n%(errors)s",
+                  dict(errors=';\n'.join("%s: %s" % (k, '; '.join(v))
+                                         for k, v in exceptions.items()),
+                       exc_class=exc_class_name,
+                       serialized=serializable_kwargs))
         # We might be able to actually put the following keys' values into
         # format string, but there is no guarantee, drop it just in case.
         for k in exceptions:
@@ -114,8 +114,8 @@ class IronicException(Exception):
                 # kwargs doesn't match a variable in self._msg_fmt
                 # log the issue and the kwargs
                 prs = ', '.join('%s: %s' % pair for pair in kwargs.items())
-                LOG.exception(_LE('Exception in string format operation '
-                                  '(arguments %s)'), prs)
+                LOG.exception('Exception in string format operation '
+                              '(arguments %s)', prs)
                 if CONF.fatal_exception_format_errors:
                     raise e
                 else:
@@ -481,21 +481,13 @@ class IPMIFailure(IronicException):
     _msg_fmt = _("IPMI call failed: %(cmd)s.")
 
 
-class SSHConnectFailed(IronicException):
-    _msg_fmt = _("Failed to establish SSH connection to host %(host)s.")
-
-
-class SSHCommandFailed(IronicException):
-    _msg_fmt = _("Failed to execute command via SSH: %(cmd)s.")
-
-
 class UnsupportedDriverExtension(Invalid):
     _msg_fmt = _('Driver %(driver)s does not support %(extension)s '
                  '(disabled or not implemented).')
 
 
 class GlanceConnectionFailed(IronicException):
-    _msg_fmt = _("Connection to glance host %(host)s:%(port)s failed: "
+    _msg_fmt = _("Connection to glance host %(endpoint)s failed: "
                  "%(reason)s")
 
 
@@ -738,3 +730,26 @@ class NotificationSchemaKeyError(IronicException):
 class NotificationPayloadError(IronicException):
     _msg_fmt = _("Payload not populated when trying to send notification "
                  "\"%(class_name)s\"")
+
+
+class StorageError(IronicException):
+    _msg_fmt = _("Storage operation failure.")
+
+
+class RedfishError(IronicException):
+    _msg_fmt = _("Redfish exception occurred. Error: %(error)s")
+
+
+class RedfishConnectionError(RedfishError):
+    _msg_fmt = _("Redfish connection failed for node %(node)s: %(error)s")
+
+
+class PortgroupPhysnetInconsistent(IronicException):
+    _msg_fmt = _("Port group %(portgroup)s has member ports with inconsistent "
+                 "physical networks (%(physical_networks)s). All ports in a "
+                 "port group must have the same physical network.")
+
+
+class VifInvalidForAttach(Conflict):
+    _msg_fmt = _("Unable to attach VIF %(vif)s to node %(node)s. Reason: "
+                 "%(reason)s")
