@@ -27,6 +27,7 @@ import six.moves.urllib.parse as urlparse
 
 from ironic.common import exception
 from ironic.common import image_service
+from ironic.common import keystone
 
 CONF = cfg.CONF
 
@@ -34,18 +35,6 @@ _GLANCE_API_SERVER = None
 """ iterator that cycles (indefinitely) over glance API servers. """
 
 
-def generate_glance_url():
-    """Generate the URL to glance."""
-    return "%s://%s:%d" % (CONF.glance.glance_protocol,
-                           CONF.glance.glance_host,
-                           CONF.glance.glance_port)
-
-
-def generate_image_url(image_ref):
-    """Generate an image URL from an image_ref."""
-    return "%s/images/%s" % (generate_glance_url(), image_ref)
-
-
 def _extract_attributes(image):
     IMAGE_ATTRIBUTES = ['size', 'disk_format', 'owner',
                         'container_format', 'checksum', 'id',
@@ -89,31 +78,18 @@ def _convert_timestamps_to_datetimes(image_meta):
 _CONVERT_PROPS = ('block_device_mapping', 'mappings')
 
 
-def _convert(metadata, method):
+def _convert(metadata):
     metadata = copy.deepcopy(metadata)
     properties = metadata.get('properties')
     if properties:
         for attr in _CONVERT_PROPS:
             if attr in properties:
                 prop = properties[attr]
-                if method == 'from':
-                    if isinstance(prop, six.string_types):
-                        properties[attr] = jsonutils.loads(prop)
-                if method == 'to':
-                    if not isinstance(prop, six.string_types):
-                        properties[attr] = jsonutils.dumps(prop)
+                if isinstance(prop, six.string_types):
+                    properties[attr] = jsonutils.loads(prop)
     return metadata
 
 
-def _remove_read_only(image_meta):
-    IMAGE_ATTRIBUTES = ['status', 'updated_at', 'created_at', 'deleted_at']
-    output = copy.deepcopy(image_meta)
-    for attr in IMAGE_ATTRIBUTES:
-        if attr in output:
-            del output[attr]
-    return output
-
-
 def _get_api_server_iterator():
     """Return iterator over shuffled API servers.
 
@@ -123,24 +99,29 @@ def _get_api_server_iterator():
 
     If CONF.glance.glance_api_servers isn't set, we fall back to using this
     as the server: CONF.glance.glance_host:CONF.glance.glance_port.
+    If CONF.glance.glance_host is also not set, fetch the endpoint from the
+    service catalog.
 
     :returns: iterator that cycles (indefinitely) over shuffled glance API
-              servers. The iterator returns tuples of (host, port, use_ssl).
+              servers.
     """
     api_servers = []
 
-    configured_servers = (CONF.glance.glance_api_servers or
-                          ['%s:%s' % (CONF.glance.glance_host,
-                                      CONF.glance.glance_port)])
-    for api_server in configured_servers:
-        if '//' not in api_server:
-            api_server = '%s://%s' % (CONF.glance.glance_protocol, api_server)
-        url = urlparse.urlparse(api_server)
-        port = url.port or 80
-        host = url.netloc.split(':', 1)[0]
-        use_ssl = (url.scheme == 'https')
-        api_servers.append((host, port, use_ssl))
-    random.shuffle(api_servers)
+    if not CONF.glance.glance_api_servers and not CONF.glance.glance_host:
+        session = keystone.get_session('glance',
+                                       auth=keystone.get_auth('glance'))
+        api_servers = [keystone.get_service_url(session, service_type='image',
+                                                endpoint_type='public')]
+    else:
+        configured_servers = (CONF.glance.glance_api_servers or
+                              ['%s:%s' % (CONF.glance.glance_host,
+                                          CONF.glance.glance_port)])
+        for api_server in configured_servers:
+            if '//' not in api_server:
+                api_server = '%s://%s' % (CONF.glance.glance_protocol,
+                                          api_server)
+            api_servers.append(api_server)
+        random.shuffle(api_servers)
     return itertools.cycle(api_servers)
 
 
@@ -158,61 +139,39 @@ def _get_api_server():
 
 
 def parse_image_ref(image_href):
-    """Parse an image href into composite parts.
+    """Parse an image href.
 
     :param image_href: href of an image
-    :returns: a tuple of the form (image_id, host, port, use_ssl)
+    :returns: a tuple (image ID, glance URL, whether to use SSL)
 
-    :raises ValueError
+    :raises ValueError: when input image href is invalid
     """
     if '/' not in six.text_type(image_href):
-        image_id = image_href
-        (glance_host, glance_port, use_ssl) = _get_api_server()
-        return (image_id, glance_host, glance_port, use_ssl)
+        endpoint = _get_api_server()
+        return (image_href, endpoint, endpoint.startswith('https'))
     else:
         try:
             url = urlparse.urlparse(image_href)
             if url.scheme == 'glance':
-                (glance_host, glance_port, use_ssl) = _get_api_server()
+                endpoint = _get_api_server()
                 image_id = image_href.split('/')[-1]
+                return (image_id, endpoint, endpoint.startswith('https'))
             else:
                 glance_port = url.port or 80
                 glance_host = url.netloc.split(':', 1)[0]
                 image_id = url.path.split('/')[-1]
                 use_ssl = (url.scheme == 'https')
-            return (image_id, glance_host, glance_port, use_ssl)
+                endpoint = '%s://%s:%s' % (url.scheme, glance_host,
+                                           glance_port)
+                return (image_id, endpoint, use_ssl)
         except ValueError:
             raise exception.InvalidImageRef(image_href=image_href)
 
 
-def extract_query_params(params, version):
-    _params = {}
-    accepted_params = ('filters', 'marker', 'limit',
-                       'sort_key', 'sort_dir')
-    for param in accepted_params:
-        if params.get(param):
-            _params[param] = params.get(param)
-    # ensure filters is a dict
-    _params.setdefault('filters', {})
-
-    # NOTE(vish): don't filter out private images
-    # NOTE(ghe): in v2, not passing any visibility doesn't filter prvate images
-    if version == 1:
-        _params['filters'].setdefault('is_public', 'none')
-
-    return _params
-
-
-def translate_to_glance(image_meta):
-    image_meta = _convert(image_meta, 'to')
-    image_meta = _remove_read_only(image_meta)
-    return image_meta
-
-
 def translate_from_glance(image):
     image_meta = _extract_attributes(image)
     image_meta = _convert_timestamps_to_datetimes(image_meta)
-    image_meta = _convert(image_meta, 'from')
+    image_meta = _convert(image_meta)
     return image_meta
 
 
