@@ -49,8 +49,17 @@ def _ensure_config_dirs_exist(node_uuid):
 
     """
     root_dir = get_root_dir()
-    fileutils.ensure_tree(os.path.join(root_dir, node_uuid))
-    fileutils.ensure_tree(os.path.join(root_dir, PXE_CFG_DIR_NAME))
+    node_dir = os.path.join(root_dir, node_uuid)
+    pxe_dir = os.path.join(root_dir, PXE_CFG_DIR_NAME)
+    # NOTE: We should only change the permissions if the folder
+    # does not exist. i.e. if defined, an operator could have
+    # already created it and placed specific ACLs upon the folder
+    # which may not recurse downward.
+    for directory in (node_dir, pxe_dir):
+        if not os.path.isdir(directory):
+            fileutils.ensure_tree(directory)
+            if CONF.pxe.dir_permission:
+                os.chmod(directory, CONF.pxe.dir_permission)
 
 
 def _link_mac_pxe_configs(task):
@@ -129,6 +138,8 @@ def _get_pxe_ip_address_path(ip_address, hex_form):
     :returns: the path to the config file.
 
     """
+    # NOTE(TheJulia): Remove elilo support after the deprecation
+    # period, in the Queens release.
     # elilo bootloader needs hex based config file name.
     if hex_form:
         ip = ip_address.split('.')
@@ -207,6 +218,9 @@ def create_pxe_config(task, pxe_options, template=None):
         hex_form = False
         pxe_config_root_tag = '(( ROOT ))'
         pxe_config_disk_ident = '(( DISK_IDENTIFIER ))'
+        LOG.warning("The requested config appears to support elilo. "
+                    "Support for elilo has been deprecated and will be "
+                    "removed in the Queens release of OpenStack.")
     else:
         # TODO(stendulker): We should use '(' ')' as the delimiters for all our
         # config files so that we do not need special handling for each of the
@@ -227,6 +241,22 @@ def create_pxe_config(task, pxe_options, template=None):
         _link_mac_pxe_configs(task)
 
 
+def create_ipxe_boot_script():
+    """Render the iPXE boot script into the HTTP root directory"""
+    boot_script = utils.render_template(
+        CONF.pxe.ipxe_boot_script,
+        {'ipxe_for_mac_uri': PXE_CFG_DIR_NAME + '/'})
+    bootfile_path = os.path.join(
+        CONF.deploy.http_root,
+        os.path.basename(CONF.pxe.ipxe_boot_script))
+    # NOTE(pas-ha) to prevent unneeded writes,
+    # only write to file if its content is different from required,
+    # which should be rather rare
+    if (not os.path.isfile(bootfile_path) or
+            not utils.file_has_content(bootfile_path, boot_script)):
+        utils.write_to_file(bootfile_path, boot_script)
+
+
 def clean_up_pxe_config(task):
     """Clean up the TFTP environment for the task's node.
 
@@ -248,6 +278,8 @@ def clean_up_pxe_config(task):
                 # Get xx.xx.xx.xx based grub config file
                 ip_address_path = _get_pxe_ip_address_path(port_ip_address,
                                                            False)
+                # NOTE(TheJulia): Remove elilo support after the deprecation
+                # period, in the Queens release.
                 # Get 0AOAOAOA based elilo config file
                 hex_ip_path = _get_pxe_ip_address_path(port_ip_address,
                                                        True)
@@ -279,7 +311,7 @@ def dhcp_options_for_instance(task):
     if CONF.pxe.ipxe_enabled:
         script_name = os.path.basename(CONF.pxe.ipxe_boot_script)
         ipxe_script_url = '/'.join([CONF.deploy.http_url, script_name])
-        dhcp_provider_name = dhcp_factory.CONF.dhcp.dhcp_provider
+        dhcp_provider_name = CONF.dhcp.dhcp_provider
         # if the request comes from dumb firmware send them the iPXE
         # boot image.
         if dhcp_provider_name == 'neutron':
