@@ -32,12 +32,11 @@ from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import netutils
 from oslo_utils import timeutils
-import paramiko
 import pytz
 import six
 
 from ironic.common import exception
-from ironic.common.i18n import _, _LE, _LW
+from ironic.common.i18n import _
 from ironic.conf import CONF
 
 LOG = logging.getLogger(__name__)
@@ -79,46 +78,6 @@ def execute(*cmd, **kwargs):
     return result
 
 
-def ssh_connect(connection):
-    """Method to connect to a remote system using ssh protocol.
-
-    :param connection: a dict of connection parameters.
-    :returns: paramiko.SSHClient -- an active ssh connection.
-    :raises: SSHConnectFailed
-
-    """
-    try:
-        ssh = paramiko.SSHClient()
-        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        key_contents = connection.get('key_contents')
-        if key_contents:
-            data = six.StringIO(key_contents)
-            if "BEGIN RSA PRIVATE" in key_contents:
-                pkey = paramiko.RSAKey.from_private_key(data)
-            elif "BEGIN DSA PRIVATE" in key_contents:
-                pkey = paramiko.DSSKey.from_private_key(data)
-            else:
-                # Can't include the key contents - secure material.
-                raise ValueError(_("Invalid private key"))
-        else:
-            pkey = None
-        ssh.connect(connection.get('host'),
-                    username=connection.get('username'),
-                    password=connection.get('password'),
-                    port=connection.get('port', 22),
-                    pkey=pkey,
-                    key_filename=connection.get('key_filename'),
-                    timeout=connection.get('timeout', 10))
-
-        # send TCP keepalive packets every 20 seconds
-        ssh.get_transport().set_keepalive(20)
-    except Exception as e:
-        LOG.debug("SSH connect failed: %s", e)
-        raise exception.SSHConnectFailed(host=connection.get('host'))
-
-    return ssh
-
-
 def is_valid_datapath_id(datapath_id):
     """Verify the format of an OpenFlow datapath_id.
 
@@ -281,8 +240,13 @@ def hash_file(file_like_object, hash_algo='md5'):
     :returns: a condensed digest of the bytes of contents.
     """
     checksum = _get_hash_object(hash_algo)
-    for chunk in iter(lambda: file_like_object.read(32768), b''):
-        checksum.update(chunk)
+    while True:
+        chunk = file_like_object.read(32768)
+        if not chunk:
+            break
+        encoded_chunk = (chunk.encode(encoding='utf-8')
+                         if isinstance(chunk, six.string_types) else chunk)
+        checksum.update(encoded_chunk)
     return checksum.hexdigest()
 
 
@@ -298,7 +262,9 @@ def file_has_content(path, content, hash_algo='md5'):
     with open(path, 'rb') as existing:
         file_hash_hex = hash_file(existing, hash_algo=hash_algo)
     ref_hash = _get_hash_object(hash_algo)
-    ref_hash.update(content)
+    encoded_content = (content.encode(encoding='utf-8')
+                       if isinstance(content, six.string_types) else content)
+    ref_hash.update(encoded_content)
     return file_hash_hex == ref_hash.hexdigest()
 
 
@@ -312,7 +278,7 @@ def tempdir(**kwargs):
         try:
             shutil.rmtree(tmpdir)
         except OSError as e:
-            LOG.error(_LE('Could not remove tmpdir: %s'), e)
+            LOG.error('Could not remove tmpdir: %s', e)
 
 
 def rmtree_without_raise(path):
@@ -320,7 +286,7 @@ def rmtree_without_raise(path):
         if os.path.isdir(path):
             shutil.rmtree(path)
     except OSError as e:
-        LOG.warning(_LW("Failed to remove dir %(path)s, error: %(e)s"),
+        LOG.warning("Failed to remove dir %(path)s, error: %(e)s",
                     {'path': path, 'e': e})
 
 
@@ -336,10 +302,9 @@ def create_link_without_raise(source, link):
         if e.errno == errno.EEXIST:
             return
         else:
-            LOG.warning(
-                _LW("Failed to create symlink from %(source)s to %(link)s"
-                    ", error: %(e)s"),
-                {'source': source, 'link': link, 'e': e})
+            LOG.warning("Failed to create symlink from "
+                        "%(source)s to %(link)s, error: %(e)s",
+                        {'source': source, 'link': link, 'e': e})
 
 
 def safe_rstrip(value, chars=None):
@@ -351,9 +316,9 @@ def safe_rstrip(value, chars=None):
 
     """
     if not isinstance(value, six.string_types):
-        LOG.warning(_LW("Failed to remove trailing character. Returning "
-                        "original object. Supplied object is not a string: "
-                        "%s,"), value)
+        LOG.warning("Failed to remove trailing character. Returning "
+                    "original object. Supplied object is not a string: "
+                    "%s,", value)
         return value
 
     return value.rstrip(chars) or value
@@ -544,7 +509,7 @@ def warn_about_deprecated_extra_vif_port_id():
     global warn_deprecated_extra_vif_port_id
     if not warn_deprecated_extra_vif_port_id:
         warn_deprecated_extra_vif_port_id = True
-        LOG.warning(_LW("Attaching VIF to a port/portgroup via "
-                        "extra['vif_port_id'] is deprecated and will not "
-                        "be supported in Pike release. API endpoint "
-                        "v1/nodes/<node>/vifs should be used instead."))
+        LOG.warning("Attaching VIF to a port/portgroup via "
+                    "extra['vif_port_id'] is deprecated and will not "
+                    "be supported in Pike release. API endpoint "
+                    "v1/nodes/<node>/vifs should be used instead.")
