@@ -89,11 +89,14 @@ class ConductorAPI(object):
     |    1.38 - Added vif_attach, vif_detach, vif_list
     |    1.39 - Added timeout optional parameter to change_node_power_state
     |    1.40 - Added inject_nmi
+    |    1.41 - Added create_port
 
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
-    RPC_API_VERSION = '1.40'
+    # NOTE(pas-ha): This also must be in sync with
+    #               ironic.common.release_mappings.RELEASE_MAPPING['master']
+    RPC_API_VERSION = '1.41'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
@@ -150,6 +153,10 @@ class ConductorAPI(object):
         host = random.choice(list(ring.nodes))
         return self.topic + "." + host
 
+    def can_send_create_port(self):
+        """Return whether the RPCAPI supports the create_port method."""
+        return self.client.can_send_version("1.41")
+
     def create_node(self, context, node_obj, topic=None):
         """Synchronously, have a conductor validate and create a node.
 
@@ -425,6 +432,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: NodeAssociated if the node contains an instance
             associated with it.
@@ -467,6 +475,21 @@ class ConductorAPI(object):
         return cctxt.call(context, 'set_console_mode', node_id=node_id,
                           enabled=enabled)
 
+    def create_port(self, context, port_obj, topic=None):
+        """Synchronously, have a conductor validate and create a port.
+
+        Create the port's information in the database and return a port object.
+        The conductor will lock related node and trigger specific driver
+        actions if they are needed.
+
+        :param context: request context.
+        :param port_obj: a created (but not saved) port object.
+        :param topic: RPC topic. Defaults to self.topic.
+        :returns: created port object.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.41')
+        return cctxt.call(context, 'create_port', port_obj=port_obj)
+
     def update_port(self, context, port_obj, topic=None):
         """Synchronously, have a conductor update the port's information.
 
@@ -543,6 +566,7 @@ class ConductorAPI(object):
                        :mod:`ironic.common.boot_devices`.
         :param persistent: Whether to set next-boot, or make the change
                            permanent. Default: False.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
@@ -561,6 +585,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
@@ -586,6 +611,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management or management.inject_nmi.
@@ -604,6 +630,7 @@ class ConductorAPI(object):
 
         :param context: request context.
         :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
         :raises: NodeLocked if node is locked by another conductor.
         :raises: UnsupportedDriverExtension if the node's driver doesn't
                  support management.
