@@ -14,11 +14,13 @@
 
 from oslo_config import cfg
 from oslo_log import log
+from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import reflection
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import network
 from ironic.common import states
 from ironic.conductor import notification_utils as notify_utils
 from ironic.conductor import task_manager
@@ -67,6 +69,37 @@ def node_set_boot_device(task, device, persistent=False):
                                                    persistent=persistent)
 
 
+def node_wait_for_power_state(task, new_state, timeout=None):
+    """Wait for node to be in new power state.
+
+    :param task: a TaskManager instance.
+    :param new_state: the desired new power state, one of the power states
+        in :mod:`ironic.common.states`.
+    :param timeout: number of seconds to wait before giving up. If not
+        specified, uses the conductor.power_state_change_timeout config value.
+    :raises: PowerStateFailure if timed out
+    """
+    retry_timeout = (timeout or CONF.conductor.power_state_change_timeout)
+
+    def _wait():
+        status = task.driver.power.get_power_state(task)
+        if status == new_state:
+            raise loopingcall.LoopingCallDone(retvalue=status)
+        # NOTE(sambetts): Return False to trigger BackOffLoopingCall to start
+        # backing off.
+        return False
+
+    try:
+        timer = loopingcall.BackOffLoopingCall(_wait)
+        return timer.start(initial_delay=1, timeout=retry_timeout).wait()
+    except loopingcall.LoopingCallTimeOut:
+        LOG.error('Timed out after %(retry_timeout)s secs waiting for power '
+                  '%(state)s on node %(node_id)s.',
+                  {'retry_timeout': retry_timeout,
+                   'state': new_state, 'node_id': task.node.uuid})
+        raise exception.PowerStateFailure(pstate=new_state)
+
+
 @task_manager.require_exclusive_lock
 def node_power_action(task, new_state, timeout=None):
     """Change power state or reset for a node.
@@ -79,6 +112,8 @@ def node_power_action(task, new_state, timeout=None):
       power state. ``None`` indicates to use default timeout.
     :raises: InvalidParameterValue when the wrong state is specified
              or the wrong driver info is specified.
+    :raises: StorageError when a failure occurs updating the node's
+             storage interface upon setting power on.
     :raises: other exceptions by the node's power driver if something
              wrong occurred during the power action.
 
@@ -154,6 +189,10 @@ def node_power_action(task, new_state, timeout=None):
 
     # take power action
     try:
+        if (target_state == states.POWER_ON and
+                node.provision_state == states.ACTIVE):
+            task.driver.storage.attach_volumes(task)
+
         if new_state != states.REBOOT:
             if ('timeout' in reflection.get_signature(
                     task.driver.power.set_power_state).parameters):
@@ -168,7 +207,11 @@ def node_power_action(task, new_state, timeout=None):
                     "doesn't support 'timeout' parameter.",
                     {'driver_name': node.driver})
                 task.driver.power.set_power_state(task, new_state)
+
         else:
+            # TODO(TheJulia): We likely ought to consider toggling
+            # volume attachments, although we have no mechanism to
+            # really verify what cinder has connector wise.
             if ('timeout' in reflection.get_signature(
                     task.driver.power.reboot).parameters):
                 task.driver.power.reboot(task, timeout=timeout)
@@ -203,6 +246,16 @@ def node_power_action(task, new_state, timeout=None):
                  {'node': node.uuid,
                   'target_state': target_state,
                   'new_state': new_state})
+        # NOTE(TheJulia): Similarly to power-on, when we power-off
+        # a node, we should detach any volume attachments.
+        if (target_state == states.POWER_OFF and
+                node.provision_state == states.ACTIVE):
+            try:
+                task.driver.storage.detach_volumes(task)
+            except exception.StorageError as e:
+                LOG.warning("Volume detachment for node %(node)s "
+                            "failed. Error: %(error)s",
+                            {'node': node.uuid, 'error': e})
 
 
 @task_manager.require_exclusive_lock
@@ -505,3 +558,65 @@ def _validate_user_clean_steps(task, user_steps):
     if errors:
         raise exception.InvalidParameterValue('; '.join(errors))
     return result
+
+
+@task_manager.require_exclusive_lock
+def validate_port_physnet(task, port_obj):
+    """Validate the consistency of physical networks of ports in a portgroup.
+
+    Validate the consistency of a port's physical network with other ports in
+    the same portgroup.  All ports in a portgroup should have the same value
+    (which may be None) for their physical_network field.
+
+    During creation or update of a port in a portgroup we apply the
+    following validation criteria:
+
+    - If the portgroup has existing ports with different physical networks, we
+      raise PortgroupPhysnetInconsistent. This shouldn't ever happen.
+    - If the port has a physical network that is inconsistent with other
+      ports in the portgroup, we raise exception.Conflict.
+
+    If a port's physical network is None, this indicates that ironic's VIF
+    attachment mapping algorithm should operate in a legacy (physical
+    network unaware) mode for this port or portgroup. This allows existing
+    ironic nodes to continue to function after an upgrade to a release
+    including physical network support.
+
+    :param task: a TaskManager instance
+    :param port_obj: a port object to be validated.
+    :raises: Conflict if the port is a member of a portgroup which is on a
+             different physical network.
+    :raises: PortgroupPhysnetInconsistent if the port's portgroup has
+             ports which are not all assigned the same physical network.
+    """
+    if 'portgroup_id' not in port_obj or not port_obj.portgroup_id:
+        return
+
+    delta = port_obj.obj_what_changed()
+    # We can skip this step if the port's portgroup membership or physical
+    # network assignment is not being changed (during creation these will
+    # appear changed).
+    if not (delta & {'portgroup_id', 'physical_network'}):
+        return
+
+    # Determine the current physical network of the portgroup.
+    pg_physnets = network.get_physnets_by_portgroup_id(task,
+                                                       port_obj.portgroup_id,
+                                                       exclude_port=port_obj)
+
+    if not pg_physnets:
+        return
+
+    # Check that the port has the same physical network as any existing
+    # member ports.
+    pg_physnet = pg_physnets.pop()
+    port_physnet = (port_obj.physical_network
+                    if 'physical_network' in port_obj else None)
+    if port_physnet != pg_physnet:
+        portgroup = network.get_portgroup_by_id(task, port_obj.portgroup_id)
+        msg = _("Port with physical network %(physnet)s cannot become a "
+                "member of port group %(portgroup)s which has ports in "
+                "physical network %(pg_physnet)s.")
+        raise exception.Conflict(
+            msg % {'portgroup': portgroup.uuid, 'physnet': port_physnet,
+                   'pg_physnet': pg_physnet})
