@@ -18,6 +18,7 @@ from oslo_config import cfg
 from ironic.conf import agent
 from ironic.conf import api
 from ironic.conf import audit
+from ironic.conf import cinder
 from ironic.conf import cisco
 from ironic.conf import conductor
 from ironic.conf import console
@@ -38,9 +39,9 @@ from ironic.conf import metrics_statsd
 from ironic.conf import neutron
 from ironic.conf import oneview
 from ironic.conf import pxe
+from ironic.conf import redfish
 from ironic.conf import service_catalog
 from ironic.conf import snmp
-from ironic.conf import ssh
 from ironic.conf import swift
 
 CONF = cfg.CONF
@@ -48,6 +49,7 @@ CONF = cfg.CONF
 agent.register_opts(CONF)
 api.register_opts(CONF)
 audit.register_opts(CONF)
+cinder.register_opts(CONF)
 cisco.register_opts(CONF)
 conductor.register_opts(CONF)
 console.register_opts(CONF)
@@ -68,7 +70,7 @@ metrics_statsd.register_opts(CONF)
 neutron.register_opts(CONF)
 oneview.register_opts(CONF)
 pxe.register_opts(CONF)
+redfish.register_opts(CONF)
 service_catalog.register_opts(CONF)
 snmp.register_opts(CONF)
-ssh.register_opts(CONF)
 swift.register_opts(CONF)
