@@ -28,8 +28,8 @@ opts = [
     cfg.IntOpt('heartbeat_interval',
                default=10,
                help=_('Seconds between conductor heart beats.')),
-    cfg.StrOpt('api_url',
-               regex='^http(s?):\/\/.+',
+    cfg.URIOpt('api_url',
+               schemes=('http', 'https'),
                help=_('URL of Ironic API service. If not set ironic can '
                       'get the current value from the keystone service '
                       'catalog. If set, the value must start with either '
@@ -107,13 +107,11 @@ opts = [
                       'conductor will check for nodes that it should '
                       '"take over". Set it to a negative value to disable '
                       'the check entirely.')),
-    cfg.BoolOpt('configdrive_use_swift',
-                default=False,
-                help=_('Whether to upload the config drive to Swift.')),
     cfg.StrOpt('configdrive_swift_container',
                default='ironic_configdrive_container',
                help=_('Name of the Swift container to store config drive '
-                      'data. Used when configdrive_use_swift is True.')),
+                      'data. Used when configdrive_use_object_store is '
+                      'True.')),
     cfg.IntOpt('inspect_timeout',
                default=1800,
                help=_('Timeout (seconds) for waiting for node inspection. '
@@ -147,6 +145,12 @@ opts = [
                min=1,
                help=_('Timeout (in seconds) of soft reboot and soft power '
                       'off operation. This value always has to be positive.')),
+    cfg.IntOpt('power_state_change_timeout',
+               min=2, default=30,
+               help=_('Number of seconds to wait for power operations to '
+                      'complete, i.e., so that a baremetal node is in the '
+                      'desired power state. If timed out, the power operation '
+                      'is considered a failure.')),
 ]
 
 
