@@ -12,6 +12,8 @@
 
 import itertools
 
+from oslo_log import log
+
 import ironic.conf
 
 _default_opt_lists = [
@@ -35,6 +37,7 @@ _opts = [
     ('api', ironic.conf.api.opts),
     ('audit', ironic.conf.audit.opts),
     ('cimc', ironic.conf.cisco.cimc_opts),
+    ('cinder', ironic.conf.cinder.list_opts()),
     ('cisco_ucs', ironic.conf.cisco.ucsm_opts),
     ('conductor', ironic.conf.conductor.opts),
     ('console', ironic.conf.console.opts),
@@ -56,7 +59,6 @@ _opts = [
     ('pxe', ironic.conf.pxe.opts),
     ('service_catalog', ironic.conf.service_catalog.list_opts()),
     ('snmp', ironic.conf.snmp.opts),
-    ('ssh', ironic.conf.ssh.opts),
     ('swift', ironic.conf.swift.list_opts()),
 ]
 
@@ -77,3 +79,27 @@ def list_opts():
     :returns: a list of (group, options) tuples
     """
     return _opts
+
+
+def update_opt_defaults():
+    log.set_defaults(
+        default_log_levels=[
+            'amqp=WARNING',
+            'amqplib=WARNING',
+            'qpid.messaging=INFO',
+            # TODO(therve): when bug #1685148 is fixed in oslo.messaging, we
+            # should be able to remove one of those 2 lines.
+            'oslo_messaging=INFO',
+            'oslo.messaging=INFO',
+            'sqlalchemy=WARNING',
+            'stevedore=INFO',
+            'eventlet.wsgi.server=INFO',
+            'iso8601=WARNING',
+            'requests=WARNING',
+            'neutronclient=WARNING',
+            'glanceclient=WARNING',
+            'urllib3.connectionpool=WARNING',
+            'keystonemiddleware.auth_token=INFO',
+            'keystoneauth.session=INFO',
+        ]
+    )
