@@ -823,6 +823,23 @@ class Connection(object):
         :raises: InvalidParameterValue if sort_key does not exist.
         """
 
+    @abc.abstractmethod
+    def get_volume_targets_by_volume_id(self, volume_id, limit=None,
+                                        marker=None, sort_key=None,
+                                        sort_dir=None):
+        """List all the volume targets for a given volume id.
+
+        :param volume_id: The UUID of the volume.
+        :param limit: Maximum number of volume targets to return.
+        :param marker: the last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted
+        :param sort_dir: direction in which results should be sorted
+                         (asc, desc)
+        :returns: A list of volume targets.
+        :raises: InvalidParameterValue if sort_key does not exist.
+        """
+
     @abc.abstractmethod
     def create_volume_target(self, target_info):
         """Create a new volume target.
@@ -867,3 +884,34 @@ class Connection(object):
         :raises: VolumeTargetNotFound if a volume target with the specified
                  ident does not exist.
         """
+
+    @abc.abstractmethod
+    def check_versions(self):
+        """Checks the whole database for incompatible objects.
+
+        This scans all the tables in search of objects that are not supported;
+        i.e., those that are not specified in
+        `ironic.common.release_mappings.RELEASE_MAPPING`.
+
+        :returns: A Boolean. True if all the objects have supported versions;
+                  False otherwise.
+        """
+
+    @abc.abstractmethod
+    def backfill_version_column(self, max_count):
+        """Backfill the version column with Ocata versions.
+
+        The version column was added to all the resource tables in this Pike
+        release (via 'ironic-dbsync upgrade'). After upgrading (from Ocata to
+        Pike), the 'ironic-dbsync online_data_migrations' command will invoke
+        this method to populate (backfill) the version columns. The version
+        used will be the object version from the pinning set in config (i.e.
+        prior to this column being added).
+
+        :param max_count: The maximum number of objects to migrate. Must be
+                          >= 0. If zero, all the objects will be migrated.
+        :returns: A 2-tuple, 1. the total number of objects that need to be
+                  migrated (at the beginning of this call) and 2. the number
+                  of migrated objects.
+        """
+        # TODO(rloo) Delete this in Queens cycle.
