@@ -18,6 +18,8 @@
 SQLAlchemy models for baremetal data.
 """
 
+from os import path
+
 from oslo_db import options as db_options
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import types as db_types
@@ -28,10 +30,10 @@ from sqlalchemy import schema, String, Text
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 
-from ironic.common import paths
 from ironic.conf import CONF
 
-_DEFAULT_SQL_CONNECTION = 'sqlite:///' + paths.state_path_def('ironic.sqlite')
+_DEFAULT_SQL_CONNECTION = 'sqlite:///' + path.join('$state_path',
+                                                   'ironic.sqlite')
 
 
 db_options.set_defaults(CONF, connection=_DEFAULT_SQL_CONNECTION)
@@ -50,6 +52,8 @@ class IronicBase(models.TimestampMixin,
 
     metadata = None
 
+    version = Column(String(15), nullable=True)
+
     def as_dict(self):
         d = {}
         for c in self.__table__.columns:
@@ -193,6 +197,7 @@ class Port(Base):
     portgroup_id = Column(Integer, ForeignKey('portgroups.id'), nullable=True)
     pxe_enabled = Column(Boolean, default=True)
     internal_info = Column(db_types.JsonEncodedDict)
+    physical_network = Column(String(64), nullable=True)
 
 
 class Portgroup(Base):
