@@ -20,14 +20,51 @@ from oslo_utils import importutils
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.drivers import base
+from ironic.drivers import generic
 from ironic.drivers.modules import agent
 from ironic.drivers.modules.ilo import boot
 from ironic.drivers.modules.ilo import console
-from ironic.drivers.modules.ilo import deploy
 from ironic.drivers.modules.ilo import inspect
 from ironic.drivers.modules.ilo import management
 from ironic.drivers.modules.ilo import power
 from ironic.drivers.modules.ilo import vendor
+from ironic.drivers.modules import inspector
+from ironic.drivers.modules import iscsi_deploy
+from ironic.drivers.modules import noop
+
+
+class IloHardware(generic.GenericHardware):
+    """iLO hardware type.
+
+    iLO hardware type is targeted for iLO 4 based Proliant Gen8
+    and Gen9 servers.
+    """
+
+    @property
+    def supported_boot_interfaces(self):
+        """List of supported boot interfaces."""
+        return [boot.IloVirtualMediaBoot, boot.IloPXEBoot]
+
+    @property
+    def supported_console_interfaces(self):
+        """List of supported console interfaces."""
+        return [console.IloConsoleInterface, noop.NoConsole]
+
+    @property
+    def supported_inspect_interfaces(self):
+        """List of supported inspect interfaces."""
+        return [inspect.IloInspect, inspector.Inspector,
+                noop.NoInspect]
+
+    @property
+    def supported_management_interfaces(self):
+        """List of supported management interfaces."""
+        return [management.IloManagement]
+
+    @property
+    def supported_power_interfaces(self):
+        """List of supported power interfaces."""
+        return [power.IloPower]
 
 
 class IloVirtualMediaIscsiDriver(base.BaseDriver):
@@ -48,7 +85,7 @@ class IloVirtualMediaIscsiDriver(base.BaseDriver):
 
         self.power = power.IloPower()
         self.boot = boot.IloVirtualMediaBoot()
-        self.deploy = deploy.IloVirtualMediaIscsiDeploy()
+        self.deploy = iscsi_deploy.ISCSIDeploy()
         self.console = console.IloConsoleInterface()
         self.management = management.IloManagement()
         self.vendor = vendor.VendorPassthru()
@@ -74,7 +111,7 @@ class IloVirtualMediaAgentDriver(base.BaseDriver):
 
         self.power = power.IloPower()
         self.boot = boot.IloVirtualMediaBoot()
-        self.deploy = deploy.IloVirtualMediaAgentDeploy()
+        self.deploy = agent.AgentDeploy()
         self.console = console.IloConsoleInterface()
         self.management = management.IloManagement()
         self.inspect = inspect.IloInspect()
