@@ -32,7 +32,7 @@ import six
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
-from ironic.common.i18n import _, _LE, _LW
+from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import keystone
 from ironic.common import states
@@ -68,28 +68,6 @@ SUPPORTED_CAPABILITIES = {
 DISK_LAYOUT_PARAMS = ('root_gb', 'swap_mb', 'ephemeral_gb')
 
 
-def warn_about_unsafe_shred_parameters():
-    iterations = CONF.deploy.shred_random_overwrite_iterations
-    overwrite_with_zeros = CONF.deploy.shred_final_overwrite_with_zeros
-    if iterations == 0 and overwrite_with_zeros is False:
-        LOG.warning(_LW('With shred_random_overwrite_iterations set to 0 and '
-                        'shred_final_overwrite_with_zeros set to False, disks '
-                        'may NOT be shredded at all, unless they support ATA '
-                        'Secure Erase. This is a possible SECURITY ISSUE!'))
-
-
-def warn_about_missing_default_boot_option():
-    if not CONF.deploy.default_boot_option:
-        LOG.warning(_LW('The default value of default_boot_option '
-                        'configuration will change eventually from '
-                        '"netboot" to "local". It is recommended to set '
-                        'an explicit value for it during the transition '
-                        'period'))
-
-
-warn_about_unsafe_shred_parameters()
-warn_about_missing_default_boot_option()
-
 # All functions are called from deploy() directly or indirectly.
 # They are split for stub-out.
 
@@ -99,7 +77,9 @@ _IRONIC_SESSION = None
 def _get_ironic_session():
     global _IRONIC_SESSION
     if not _IRONIC_SESSION:
-        _IRONIC_SESSION = keystone.get_session('service_catalog')
+        auth = keystone.get_auth('service_catalog')
+        _IRONIC_SESSION = keystone.get_session('service_catalog',
+                                               auth=auth)
     return _IRONIC_SESSION
 
 
@@ -170,16 +150,15 @@ def login_iscsi(portal_address, portal_port, target_iqn):
             processutils.ProcessExecutionError) as e:
         with excutils.save_and_reraise_exception():
             error_occurred = True
-            LOG.error(_LE("Failed to login to an iSCSI target due to %s"),
-                      e)
+            LOG.error("Failed to login to an iSCSI target due to %s", e)
     finally:
         if error_occurred:
             try:
                 logout_iscsi(portal_address, portal_port, target_iqn)
                 delete_iscsi(portal_address, portal_port, target_iqn)
             except processutils.ProcessExecutionError as e:
-                LOG.warning(_LW("An error occurred when trying to cleanup "
-                                "failed ISCSI session error %s"), e)
+                LOG.warning("An error occurred when trying to cleanup "
+                            "failed ISCSI session error %s", e)
 
 
 def check_file_system_for_iscsi_device(portal_address,
@@ -290,11 +269,13 @@ def _replace_root_uuid(path, root_uuid):
 
 
 def _replace_boot_line(path, boot_mode, is_whole_disk_image,
-                       trusted_boot=False):
+                       trusted_boot=False, iscsi_boot=False):
     if is_whole_disk_image:
         boot_disk_type = 'boot_whole_disk'
     elif trusted_boot:
         boot_disk_type = 'trusted_boot'
+    elif iscsi_boot:
+        boot_disk_type = 'boot_iscsi'
     else:
         boot_disk_type = 'boot_partition'
 
@@ -315,7 +296,8 @@ def _replace_disk_identifier(path, disk_identifier):
 
 
 def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
-                      is_whole_disk_image, trusted_boot=False):
+                      is_whole_disk_image, trusted_boot=False,
+                      iscsi_boot=False):
     """Switch a pxe config from deployment mode to service mode.
 
     :param path: path to the pxe config file in tftpboot.
@@ -326,13 +308,15 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
     :param trusted_boot: if boot with trusted_boot or not. The usage of
         is_whole_disk_image and trusted_boot are mutually exclusive. You can
         have one or neither, but not both.
+    :param iscsi_boot: if boot is from an iSCSI volume or not.
     """
     if not is_whole_disk_image:
         _replace_root_uuid(path, root_uuid_or_disk_id)
     else:
         _replace_disk_identifier(path, root_uuid_or_disk_id)
 
-    _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot)
+    _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot,
+                       iscsi_boot)
 
 
 def get_dev(address, port, iqn, lun):
@@ -407,8 +391,7 @@ def deploy_disk_image(address, port, iqn, lun,
     :param iqn: The iSCSI qualified name.
     :param lun: The iSCSI logical unit number.
     :param image_path: Path for the instance's disk image.
-    :param node_uuid: node's uuid. Used for logging. Currently not in use
-        by this function but could be used in the future.
+    :param node_uuid: node's uuid.
     :param configdrive: Optional. Base64 encoded Gzipped configdrive content
                         or configdrive HTTP URL.
     :returns: a dictionary containing the key 'disk identifier' to identify
@@ -417,12 +400,13 @@ def deploy_disk_image(address, port, iqn, lun,
     with _iscsi_setup_and_handle_errors(address, port, iqn,
                                         lun) as dev:
         disk_utils.populate_image(image_path, dev)
-        disk_identifier = disk_utils.get_disk_identifier(dev)
 
         if configdrive:
             disk_utils.create_config_drive_partition(node_uuid, dev,
                                                      configdrive)
 
+        disk_identifier = disk_utils.get_disk_identifier(dev)
+
     return {'disk identifier': disk_identifier}
 
 
@@ -445,13 +429,13 @@ def _iscsi_setup_and_handle_errors(address, port, iqn, lun):
         yield dev
     except processutils.ProcessExecutionError as err:
         with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Deploy to address %s failed."), address)
-            LOG.error(_LE("Command: %s"), err.cmd)
-            LOG.error(_LE("StdOut: %r"), err.stdout)
-            LOG.error(_LE("StdErr: %r"), err.stderr)
+            LOG.error("Deploy to address %s failed.", address)
+            LOG.error("Command: %s", err.cmd)
+            LOG.error("StdOut: %r", err.stdout)
+            LOG.error("StdErr: %r", err.stderr)
     except exception.InstanceDeployFailure as e:
         with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Deploy to address %s failed."), address)
+            LOG.error("Deploy to address %s failed.", address)
             LOG.error(e)
     finally:
         logout_iscsi(address, port, iqn)
@@ -526,8 +510,8 @@ def set_failed_state(task, msg, collect_logs=True):
     try:
         task.process_event('fail')
     except exception.InvalidState:
-        msg2 = (_LE('Internal error. Node %(node)s in provision state '
-                    '"%(state)s" could not transition to a failed state.')
+        msg2 = ('Internal error. Node %(node)s in provision state '
+                '"%(state)s" could not transition to a failed state.'
                 % {'node': node.uuid, 'state': node.provision_state})
         LOG.exception(msg2)
 
@@ -535,10 +519,10 @@ def set_failed_state(task, msg, collect_logs=True):
         try:
             manager_utils.node_power_action(task, states.POWER_OFF)
         except Exception:
-            msg2 = (_LE('Node %s failed to power off while handling deploy '
-                        'failure. This may be a serious condition. Node '
-                        'should be removed from Ironic or put in maintenance '
-                        'mode until the problem is resolved.') % node.uuid)
+            msg2 = ('Node %s failed to power off while handling deploy '
+                    'failure. This may be a serious condition. Node '
+                    'should be removed from Ironic or put in maintenance '
+                    'mode until the problem is resolved.' % node.uuid)
             LOG.exception(msg2)
     # NOTE(deva): node_power_action() erases node.last_error
     #             so we need to set it here.
@@ -706,12 +690,12 @@ def try_set_boot_device(task, device, persistent=True):
         manager_utils.node_set_boot_device(task, device,
                                            persistent=persistent)
     except exception.IPMIFailure:
-        if get_boot_mode_for_deploy(task.node) == 'uefi':
-            LOG.warning(_LW("ipmitool is unable to set boot device while "
+        with excutils.save_and_reraise_exception() as ctxt:
+            if get_boot_mode_for_deploy(task.node) == 'uefi':
+                ctxt.reraise = False
+                LOG.warning("ipmitool is unable to set boot device while "
                             "the node %s is in UEFI boot mode. Please set "
-                            "the boot device manually."), task.node.uuid)
-        else:
-            raise
+                            "the boot device manually.", task.node.uuid)
 
 
 def is_secure_boot_requested(node):
@@ -1187,11 +1171,11 @@ def build_instance_info_for_deploy(task):
             image_service.HttpImageService().validate_href(url, secret)
         except exception.ImageRefValidationFailed as e:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Agent deploy supports only HTTP(S) URLs as "
-                              "instance_info['image_source'] or swift "
-                              "temporary URL. Either the specified URL is not "
-                              "a valid HTTP(S) URL or is not reachable "
-                              "for node %(node)s. Error: %(msg)s"),
+                LOG.error("Agent deploy supports only HTTP(S) URLs as "
+                          "instance_info['image_source'] or swift "
+                          "temporary URL. Either the specified URL is not "
+                          "a valid HTTP(S) URL or is not reachable "
+                          "for node %(node)s. Error: %(msg)s",
                           {'node': node.uuid, 'msg': e})
     node = task.node
     instance_info = node.instance_info
@@ -1227,3 +1211,114 @@ def build_instance_info_for_deploy(task):
     else:
         instance_info['image_type'] = 'whole-disk-image'
     return instance_info
+
+
+def check_interface_capability(interface, capability):
+    """Evaluate interface to determine if capability is present.
+
+    :param interface: The interface object to check.
+    :param capability: The value representing the capability that
+                       the caller wishes to check if present.
+
+    :returns: True if capability found, otherwise False.
+    """
+    return capability in getattr(interface, 'capabilities', [])
+
+
+def get_remote_boot_volume(task):
+    """Identify a boot volume from any configured volumes.
+
+    :returns: None or the volume target representing the volume.
+    """
+    targets = task.volume_targets
+    for volume in targets:
+        if volume['boot_index'] == 0:
+            return volume
+
+
+def populate_storage_driver_internal_info(task):
+    """Set node driver_internal_info for boot from volume parameters.
+
+    :param task: a TaskManager object containing the node.
+    :raises StorageError when a node has an iSCSI or FibreChannel boot volume
+        defined but is not capable to support it.
+    """
+    node = task.node
+    boot_volume = get_remote_boot_volume(task)
+    if not boot_volume:
+        return
+    vol_type = str(boot_volume.volume_type).lower()
+    node_caps = driver_utils.capabilities_to_dict(
+        node.properties.get('capabilities'))
+    if vol_type == 'iscsi' and 'iscsi_boot' not in node_caps:
+        # TODO(TheJulia): In order to support the FCoE and HBA boot cases,
+        # some additional logic will be needed here to ensure we align.
+        # The deployment, in theory, should never reach this point
+        # if the interfaces all validated, but we shouldn't use that
+        # as the only guard against bad configurations.
+        raise exception.StorageError(_('Node %(node)s has an iSCSI boot '
+                                       'volume defined and no iSCSI boot '
+                                       'support available.') %
+                                     {'node': node.uuid})
+    if vol_type == 'fibre_channel' and 'fibre_channel_boot' not in node_caps:
+        raise exception.StorageError(_('Node %(node)s has a Fibre Channel '
+                                       'boot volume defined and no Fibre '
+                                       'Channel boot support available.') %
+                                     {'node': node.uuid})
+    boot_capability = ("%s_volume_boot" % vol_type)
+    deploy_capability = ("%s_volume_deploy" % vol_type)
+    vol_uuid = boot_volume['uuid']
+    driver_internal_info = node.driver_internal_info
+    if check_interface_capability(task.driver.boot, boot_capability):
+        driver_internal_info['boot_from_volume'] = vol_uuid
+    # NOTE(TheJulia): This would be a convenient place to check
+    # if we need to know about deploying the volume.
+    if (check_interface_capability(task.driver.deploy, deploy_capability) and
+            task.driver.storage.should_write_image(task)):
+        driver_internal_info['boot_from_volume_deploy'] = vol_uuid
+        # NOTE(TheJulia): This is also a useful place to include a
+        # root device hint since we should/might/be able to obtain
+        # and supply that information to IPA if it needs to write
+        # the image to the volume.
+    node.driver_internal_info = driver_internal_info
+    node.save()
+
+
+def tear_down_storage_configuration(task):
+    """Clean up storage configuration.
+
+    Remove entries from driver_internal_info for storage and
+    deletes the volume targets from the database. This is done
+    to ensure a clean state for the next boot of the machine.
+    """
+
+    # TODO(mjturek): TheJulia mentioned that this should
+    # possibly be configurable for the standalone case. However,
+    # this is dangerous if IPA is not handling the cleaning.
+    for volume in task.volume_targets:
+        volume.destroy()
+        LOG.info('Successfully deleted volume target %(target)s. '
+                 'The node associated with the target was %(node)s.',
+                 {'target': volume.uuid, 'node': task.node.uuid})
+
+    node = task.node
+    driver_internal_info = node.driver_internal_info
+    driver_internal_info.pop('boot_from_volume', None)
+    driver_internal_info.pop('boot_from_volume_deploy', None)
+    node.driver_internal_info = driver_internal_info
+    node.save()
+
+
+def is_iscsi_boot(task):
+    """Return true if booting from an iscsi volume."""
+    node = task.node
+    volume = node.driver_internal_info.get('boot_from_volume')
+    if volume:
+        try:
+            boot_volume = objects.VolumeTarget.get_by_uuid(
+                task.context, volume)
+            if boot_volume.volume_type == 'iscsi':
+                return True
+        except exception.VolumeTargetNotFound:
+            return False
+    return False
