@@ -43,17 +43,33 @@ class FakePower(base.PowerInterface):
     def get_power_state(self, task):
         return task.node.power_state
 
-    def set_power_state(self, task, power_state):
+    def set_power_state(self, task, power_state, timeout=None):
         if power_state not in [states.POWER_ON, states.POWER_OFF]:
             raise exception.InvalidParameterValue(
-                _("set_power_state called with an invalid power"
+                _("set_power_state called with an invalid power "
                   "state: %s.") % power_state)
         task.node.power_state = power_state
 
-    def reboot(self, task):
+    def reboot(self, task, timeout=None):
         pass
 
 
+class FakeSoftPower(FakePower):
+    """Example implementation of a simple soft power operations."""
+
+    def set_power_state(self, task, power_state, timeout=None):
+        if power_state not in [states.POWER_ON, states.POWER_OFF,
+                               states.SOFT_REBOOT, states.SOFT_POWER_OFF]:
+            raise exception.InvalidParameterValue(
+                _("set_power_state called with an invalid power "
+                  "state: %s.") % power_state)
+        task.node.power_state = power_state
+
+    def get_supported_power_states(self, task):
+        return [states.POWER_ON, states.POWER_OFF, states.REBOOT,
+                states.SOFT_REBOOT, states.SOFT_POWER_OFF]
+
+
 class FakeBoot(base.BootInterface):
     """Example implementation of a simple boot interface."""
 
@@ -63,7 +79,7 @@ class FakeBoot(base.BootInterface):
     def validate(self, task):
         pass
 
-    def prepare_ramdisk(self, task):
+    def prepare_ramdisk(self, task, ramdisk_params):
         pass
 
     def clean_up_ramdisk(self, task):
@@ -225,3 +241,22 @@ class FakeRAID(base.RAIDInterface):
 
     def delete_configuration(self, task):
         pass
+
+
+class FakeStorage(base.StorageInterface):
+    """Example implementation of simple storage Interface."""
+
+    def validate(self, task):
+        pass
+
+    def get_properties(self):
+        return {}
+
+    def attach_volumes(self, task):
+        pass
+
+    def detach_volumes(self, task):
+        pass
+
+    def should_write_image(self, task):
+        return True
