@@ -18,6 +18,7 @@ Boot Interface for iLO drivers and its supporting methods.
 import os
 import tempfile
 
+from ironic_lib import metrics_utils
 from ironic_lib import utils as ironic_utils
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -28,8 +29,6 @@ from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
-from ironic.common.i18n import _LE
-from ironic.common.i18n import _LW
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import states
@@ -38,9 +37,12 @@ from ironic.conductor import utils as manager_utils
 from ironic.drivers import base
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.ilo import common as ilo_common
+from ironic.drivers.modules import pxe
 
 LOG = logging.getLogger(__name__)
 
+METRICS = metrics_utils.get_metrics_logger(__name__)
+
 CONF = cfg.CONF
 
 REQUIRED_PROPERTIES = {
@@ -128,11 +130,11 @@ def _get_boot_iso(task, root_uuid):
                 image_service.HttpImageService().validate_href(boot_iso)
             except exception.ImageRefValidationFailed:
                 with excutils.save_and_reraise_exception():
-                    LOG.error(_LE("Virtual media deploy accepts only Glance "
-                                  "images or HTTP(S) URLs as "
-                                  "instance_info['ilo_boot_iso']. Either %s "
-                                  "is not a valid HTTP(S) URL or is "
-                                  "not reachable."), boot_iso)
+                    LOG.error("Virtual media deploy accepts only Glance "
+                              "images or HTTP(S) URLs as "
+                              "instance_info['ilo_boot_iso']. Either %s "
+                              "is not a valid HTTP(S) URL or is "
+                              "not reachable.", boot_iso)
 
         return task.node.instance_info['ilo_boot_iso']
 
@@ -156,8 +158,8 @@ def _get_boot_iso(task, root_uuid):
         return boot_iso_uuid
 
     if not kernel_href or not ramdisk_href:
-        LOG.error(_LE("Unable to find kernel or ramdisk for "
-                      "image %(image)s to generate boot ISO for %(node)s"),
+        LOG.error("Unable to find kernel or ramdisk for "
+                  "image %(image)s to generate boot ISO for %(node)s",
                   {'image': image_href, 'node': task.node.uuid})
         return
 
@@ -216,8 +218,8 @@ def _clean_up_boot_iso_for_instance(node):
         try:
             swift_api.delete_object(container, boot_iso_object_name)
         except exception.SwiftOperationError as e:
-            LOG.exception(_LE("Failed to clean up boot ISO for node "
-                              "%(node)s. Error: %(error)s."),
+            LOG.exception("Failed to clean up boot ISO for node "
+                          "%(node)s. Error: %(error)s.",
                           {'node': node.uuid, 'error': e})
     elif CONF.ilo.use_web_server_for_images:
         result = urlparse.urlparse(ilo_boot_iso)
@@ -247,11 +249,144 @@ def _parse_deploy_info(node):
     return info
 
 
+def _validate_driver_info(task):
+    """Validate the prerequisites for virtual media based boot.
+
+    This method validates whether the 'driver_info' property of the
+    supplied node contains the required information for this driver.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :raises: InvalidParameterValue if any parameters are incorrect
+    :raises: MissingParameterValue if some mandatory information
+        is missing on the node
+    """
+    node = task.node
+    ilo_common.parse_driver_info(node)
+    if 'ilo_deploy_iso' not in node.driver_info:
+        raise exception.MissingParameterValue(_(
+            "Missing 'ilo_deploy_iso' parameter in node's 'driver_info'."))
+    deploy_iso = node.driver_info['ilo_deploy_iso']
+    if not service_utils.is_glance_image(deploy_iso):
+        try:
+            image_service.HttpImageService().validate_href(deploy_iso)
+        except exception.ImageRefValidationFailed:
+            raise exception.InvalidParameterValue(_(
+                "Virtual media boot accepts only Glance images or "
+                "HTTP(S) as driver_info['ilo_deploy_iso']. Either '%s' "
+                "is not a glance UUID or not a valid HTTP(S) URL or "
+                "the given URL is not reachable.") % deploy_iso)
+
+
+def _validate_instance_image_info(task):
+    """Validate instance image information for the task's node.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :raises: InvalidParameterValue, if some information is invalid.
+    :raises: MissingParameterValue if 'kernel_id' and 'ramdisk_id' are
+        missing in the Glance image or 'kernel' and 'ramdisk' not provided
+        in instance_info for non-Glance image.
+    """
+
+    node = task.node
+
+    d_info = _parse_deploy_info(node)
+
+    if node.driver_internal_info.get('is_whole_disk_image'):
+        props = []
+    elif service_utils.is_glance_image(d_info['image_source']):
+        props = ['kernel_id', 'ramdisk_id']
+    else:
+        props = ['kernel', 'ramdisk']
+    deploy_utils.validate_image_properties(task.context, d_info, props)
+
+
+def _disable_secure_boot(task):
+    """Disables secure boot on node, if secure boot is enabled on node.
+
+    This method checks if secure boot is enabled on node. If enabled, it
+    disables same and returns True.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :returns: It returns True, if secure boot was successfully disabled on
+              the node.
+              It returns False, if secure boot on node is in disabled state
+              or if secure boot feature is not supported by the node.
+    :raises: IloOperationError, if some operation on iLO failed.
+    """
+    cur_sec_state = False
+    try:
+        cur_sec_state = ilo_common.get_secure_boot_mode(task)
+    except exception.IloOperationNotSupported:
+        LOG.debug('Secure boot mode is not supported for node %s',
+                  task.node.uuid)
+        return False
+
+    if cur_sec_state:
+        LOG.debug('Disabling secure boot for node %s', task.node.uuid)
+        ilo_common.set_secure_boot_mode(task, False)
+        return True
+    return False
+
+
+def prepare_node_for_deploy(task):
+    """Common preparatory steps for all iLO drivers.
+
+    This method performs common preparatory steps required for all drivers.
+    1. Power off node
+    2. Disables secure boot, if it is in enabled state.
+    3. Updates boot_mode capability to 'uefi' if secure boot is requested.
+    4. Changes boot mode of the node if secure boot is disabled currently.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :raises: IloOperationError, if some operation on iLO failed.
+    """
+    manager_utils.node_power_action(task, states.POWER_OFF)
+
+    # Boot mode can be changed only if secure boot is in disabled state.
+    # secure boot and boot mode cannot be changed together.
+    change_boot_mode = True
+
+    # Disable secure boot on the node if it is in enabled state.
+    if _disable_secure_boot(task):
+        change_boot_mode = False
+
+    if change_boot_mode:
+        ilo_common.update_boot_mode(task)
+    else:
+        # Need to update boot mode that will be used during deploy, if one is
+        # not provided.
+        # Since secure boot was disabled, we are in 'uefi' boot mode.
+        if deploy_utils.get_boot_mode_for_deploy(task.node) is None:
+            instance_info = task.node.instance_info
+            instance_info['deploy_boot_mode'] = 'uefi'
+            task.node.instance_info = instance_info
+            task.node.save()
+
+
+def disable_secure_boot_if_supported(task):
+    """Disables secure boot on node, does not throw if its not supported.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :raises: IloOperationError, if some operation on iLO failed.
+    """
+    try:
+        ilo_common.update_secure_boot_mode(task, False)
+    # We need to handle IloOperationNotSupported exception so that if
+    # the user has incorrectly specified the Node capability
+    # 'secure_boot' to a node that does not have that capability and
+    # attempted deploy. Handling this exception here, will help the
+    # user to tear down such a Node.
+    except exception.IloOperationNotSupported:
+        LOG.warning('Secure boot mode is not supported for node %s',
+                    task.node.uuid)
+
+
 class IloVirtualMediaBoot(base.BootInterface):
 
     def get_properties(self):
         return COMMON_PROPERTIES
 
+    @METRICS.timer('IloVirtualMediaBoot.validate')
     def validate(self, task):
         """Validate the deployment information for the task's node.
 
@@ -262,18 +397,10 @@ class IloVirtualMediaBoot(base.BootInterface):
             in instance_info for non-Glance image.
         """
 
-        node = task.node
-
-        d_info = _parse_deploy_info(node)
-
-        if node.driver_internal_info.get('is_whole_disk_image'):
-            props = []
-        elif service_utils.is_glance_image(d_info['image_source']):
-            props = ['kernel_id', 'ramdisk_id']
-        else:
-            props = ['kernel', 'ramdisk']
-        deploy_utils.validate_image_properties(task.context, d_info, props)
+        _validate_instance_image_info(task)
+        _validate_driver_info(task)
 
+    @METRICS.timer('IloVirtualMediaBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of deploy ramdisk using virtual media.
 
@@ -295,12 +422,20 @@ class IloVirtualMediaBoot(base.BootInterface):
 
         node = task.node
         # NOTE(TheJulia): If this method is being called by something
-        # aside from a deployment, such as conductor takeover, we should
-        # treat this as a no-op and move on otherwise we would modify
+        # aside from deployment and clean, such as conductor takeover, we
+        # should treat this as a no-op and move on otherwise we would modify
         # the state of the node due to virtual media operations.
-        if node.provision_state != states.DEPLOYING:
+        if (node.provision_state != states.DEPLOYING and
+                node.provision_state != states.CLEANING):
             return
 
+        # Powering off the Node before initiating boot for node cleaning.
+        # If node is in system POST, setting boot device fails.
+        manager_utils.node_power_action(task, states.POWER_OFF)
+
+        if task.node.provision_state == states.DEPLOYING:
+            prepare_node_for_deploy(task)
+
         # Clear ilo_boot_iso if it's a glance image to force recreate
         # another one again (or use existing one in glance).
         # This is mainly for rebuild scenario.
@@ -312,7 +447,7 @@ class IloVirtualMediaBoot(base.BootInterface):
             node.save()
 
         # Eject all virtual media devices, as we are going to use them
-        # during deploy.
+        # during boot.
         ilo_common.eject_vmedia_devices(task)
 
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
@@ -321,6 +456,7 @@ class IloVirtualMediaBoot(base.BootInterface):
 
         ilo_common.setup_vmedia(task, deploy_iso, ramdisk_params)
 
+    @METRICS.timer('IloVirtualMediaBoot.prepare_instance')
     def prepare_instance(self, task):
         """Prepares the boot of instance.
 
@@ -354,9 +490,14 @@ class IloVirtualMediaBoot(base.BootInterface):
             if root_uuid_or_disk_id:
                 self._configure_vmedia_boot(task, root_uuid_or_disk_id)
             else:
-                LOG.warning(_LW("The UUID for the root partition could not "
-                                "be found for node %s"), node.uuid)
+                LOG.warning("The UUID for the root partition could not "
+                            "be found for node %s", node.uuid)
+        # Set boot mode
+        ilo_common.update_boot_mode(task)
+        # Need to enable secure boot, if being requested
+        ilo_common.update_secure_boot_mode(task, True)
 
+    @METRICS.timer('IloVirtualMediaBoot.clean_up_instance')
     def clean_up_instance(self, task):
         """Cleans up the boot of instance.
 
@@ -368,6 +509,10 @@ class IloVirtualMediaBoot(base.BootInterface):
         :raises: IloOperationError, if some operation on iLO failed.
         """
 
+        LOG.debug("Cleaning up the instance.")
+        manager_utils.node_power_action(task, states.POWER_OFF)
+        disable_secure_boot_if_supported(task)
+
         _clean_up_boot_iso_for_instance(task.node)
 
         driver_internal_info = task.node.driver_internal_info
@@ -378,6 +523,7 @@ class IloVirtualMediaBoot(base.BootInterface):
 
         ilo_common.cleanup_vmedia_boot(task)
 
+    @METRICS.timer('IloVirtualMediaBoot.clean_up_ramdisk')
     def clean_up_ramdisk(self, task):
         """Cleans up the boot of ironic ramdisk.
 
@@ -403,7 +549,7 @@ class IloVirtualMediaBoot(base.BootInterface):
         node = task.node
         boot_iso = _get_boot_iso(task, root_uuid)
         if not boot_iso:
-            LOG.error(_LE("Cannot get boot ISO for node %s"), node.uuid)
+            LOG.error("Cannot get boot ISO for node %s", node.uuid)
             return
 
         # Upon deploy complete, some distros cloud images reboot the system as
@@ -418,3 +564,70 @@ class IloVirtualMediaBoot(base.BootInterface):
         i_info['ilo_boot_iso'] = boot_iso
         node.instance_info = i_info
         node.save()
+
+
+class IloPXEBoot(pxe.PXEBoot):
+
+    @METRICS.timer('IloPXEBoot.prepare_ramdisk')
+    def prepare_ramdisk(self, task, ramdisk_params):
+        """Prepares the boot of Ironic ramdisk using PXE.
+
+        This method prepares the boot of the deploy ramdisk after
+        reading relevant information from the node's driver_info and
+        instance_info.
+
+        :param task: a task from TaskManager.
+        :param ramdisk_params: the parameters to be passed to the ramdisk.
+        :returns: None
+        :raises: MissingParameterValue, if some information is missing in
+            node's driver_info or instance_info.
+        :raises: InvalidParameterValue, if some information provided is
+            invalid.
+        :raises: IronicException, if some power or set boot boot device
+            operation failed on the node.
+        :raises: IloOperationError, if some operation on iLO failed.
+        """
+
+        if task.node.provision_state == states.DEPLOYING:
+            prepare_node_for_deploy(task)
+
+        super(IloPXEBoot, self).prepare_ramdisk(task, ramdisk_params)
+
+    @METRICS.timer('IloPXEBoot.prepare_instance')
+    def prepare_instance(self, task):
+        """Prepares the boot of instance.
+
+        This method prepares the boot of the instance after reading
+        relevant information from the node's instance_info. In case of netboot,
+        it updates the dhcp entries and switches the PXE config. In case of
+        localboot, it cleans up the PXE config.
+
+        :param task: a task from TaskManager.
+        :returns: None
+        :raises: IloOperationError, if some operation on iLO failed.
+        """
+
+        # Set boot mode
+        ilo_common.update_boot_mode(task)
+        # Need to enable secure boot, if being requested
+        ilo_common.update_secure_boot_mode(task, True)
+
+        super(IloPXEBoot, self).prepare_instance(task)
+
+    @METRICS.timer('IloPXEBoot.clean_up_instance')
+    def clean_up_instance(self, task):
+        """Cleans up the boot of instance.
+
+        This method cleans up the PXE environment that was setup for booting
+        the instance. It unlinks the instance kernel/ramdisk in the node's
+        directory in tftproot and removes it's PXE config.
+
+        :param task: a task from TaskManager.
+        :returns: None
+        :raises: IloOperationError, if some operation on iLO failed.
+        """
+
+        manager_utils.node_power_action(task, states.POWER_OFF)
+        disable_secure_boot_if_supported(task)
+
+        super(IloPXEBoot, self).clean_up_instance(task)
