@@ -25,7 +25,7 @@ from six.moves.urllib import parse
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
-from ironic.common.i18n import _, _LE
+from ironic.common.i18n import _
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import task_manager
@@ -88,17 +88,10 @@ def _save_disk_layout(node, i_info):
 def check_image_size(task):
     """Check if the requested image is larger than the root partition size.
 
-    Does nothing for whole-disk images.
-
     :param task: a TaskManager instance containing the node to act on.
     :raises: InstanceDeployFailure if size of the image is greater than root
         partition.
     """
-    if task.node.driver_internal_info['is_whole_disk_image']:
-        # The root partition is already created and populated, no use
-        # validating its size
-        return
-
     i_info = deploy_utils.parse_instance_info(task.node)
     image_path = _get_image_file_path(task.node.uuid)
     image_mb = disk_utils.get_image_mb(image_path)
@@ -257,14 +250,13 @@ def continue_deploy(task, **kwargs):
             uuid_dict_returned = deploy_utils.deploy_partition_image(**params)
     except exception.IronicException as e:
         with excutils.save_and_reraise_exception():
-            LOG.error(_LE('Deploy of instance %(instance)s on node %(node)s '
-                          'failed: %(error)s'),
-                      {'instance': node.instance_uuid, 'node': node.uuid,
-                       'error': e})
+            LOG.error('Deploy of instance %(instance)s on node %(node)s '
+                      'failed: %(error)s', {'instance': node.instance_uuid,
+                                            'node': node.uuid, 'error': e})
             _fail_deploy(task, e, raise_exception=False)
     except Exception as e:
-        LOG.exception(_LE('Deploy of instance %(instance)s on node %(node)s '
-                          'failed with exception'),
+        LOG.exception('Deploy of instance %(instance)s on node %(node)s '
+                      'failed with exception',
                       {'instance': node.instance_uuid, 'node': node.uuid})
         _fail_deploy(task, e)
 
@@ -432,6 +424,14 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
         # Check the boot_mode, boot_option and disk_label capabilities values.
         deploy_utils.validate_capabilities(node)
 
+        # Edit early if we are not writing a volume as the validate
+        # tasks evaluate root device hints.
+        if not task.driver.storage.should_write_image(task):
+            LOG.debug('Skipping complete deployment interface validation '
+                      'for node %s as it is set to boot from a remote '
+                      'volume.', node.uuid)
+            return
+
         # TODO(rameshg87): iscsi_ilo driver uses this method. Remove
         # and copy-paste it's contents here once iscsi_ilo deploy driver
         # broken down into separate boot and deploy implementations.
@@ -451,12 +451,24 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
         :returns: deploy state DEPLOYWAIT.
         """
         node = task.node
-        cache_instance_image(task.context, node)
-        check_image_size(task)
+        if task.driver.storage.should_write_image(task):
+            cache_instance_image(task.context, node)
+            check_image_size(task)
+            manager_utils.node_power_action(task, states.REBOOT)
 
-        manager_utils.node_power_action(task, states.REBOOT)
-
-        return states.DEPLOYWAIT
+            return states.DEPLOYWAIT
+        else:
+            # TODO(TheJulia): At some point, we should de-dupe this code
+            # as it is nearly identical to the agent deploy interface.
+            # This is not being done now as it is expected to be
+            # refactored in the near future.
+            manager_utils.node_power_action(task, states.POWER_OFF)
+            task.driver.network.remove_provisioning_network(task)
+            task.driver.network.configure_tenant_networks(task)
+            manager_utils.node_power_action(task, states.POWER_ON)
+            task.process_event('done')
+            LOG.info('Deployment to node %s done', node.uuid)
+            return states.DEPLOYDONE
 
     @METRICS.timer('ISCSIDeploy.tear_down')
     @task_manager.require_exclusive_lock
@@ -471,10 +483,13 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
         :raises: NetworkError if the cleaning ports cannot be removed.
         :raises: InvalidParameterValue when the wrong state is specified
              or the wrong driver info is specified.
+        :raises: StorageError when volume detachment fails.
         :raises: other exceptions by the node's power driver if something
              wrong occurred during the power action.
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
+        task.driver.storage.detach_volumes(task)
+        deploy_utils.tear_down_storage_configuration(task)
         task.driver.network.unconfigure_tenant_networks(task)
         return states.DELETED
 
@@ -491,12 +506,15 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
         :raises: NetworkError: if the previous cleaning ports cannot be removed
             or if new cleaning ports cannot be created.
         :raises: InvalidParameterValue when the wrong power state is specified
-             or the wrong driver info is specified for power management.
+            or the wrong driver info is specified for power management.
+        :raises: StorageError If the storage driver is unable to attach the
+            configured volumes.
         :raises: other exceptions by the node's power driver if something
-             wrong occurred during the power action.
+            wrong occurred during the power action.
         :raises: any boot interface's prepare_ramdisk exceptions.
         """
         node = task.node
+        deploy_utils.populate_storage_driver_internal_info(task)
         if node.provision_state in [states.ACTIVE, states.ADOPTING]:
             task.driver.boot.prepare_instance(task)
         else:
@@ -506,8 +524,10 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
                 manager_utils.node_power_action(task, states.POWER_OFF)
                 # NOTE(vdrok): in case of rebuild, we have tenant network
                 # already configured, unbind tenant ports if present
-                task.driver.network.unconfigure_tenant_networks(task)
-                task.driver.network.add_provisioning_network(task)
+                if task.driver.storage.should_write_image(task):
+                    task.driver.network.unconfigure_tenant_networks(task)
+                    task.driver.network.add_provisioning_network(task)
+                task.driver.storage.attach_volumes(task)
 
             deploy_opts = deploy_utils.build_agent_options(node)
             task.driver.boot.prepare_ramdisk(task, deploy_opts)
