@@ -21,7 +21,8 @@ from oslo_log import log
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
-from ironic.common.i18n import _, _LW
+from ironic.common.i18n import _
+from ironic.common import network
 from ironic.common import neutron
 from ironic.common import states
 from ironic.common import utils
@@ -54,7 +55,35 @@ def _vif_attached(port_like_obj, vif_id):
     return attached_vif_id is not None
 
 
-def _get_free_portgroups_and_ports(task, vif_id):
+def _is_port_physnet_allowed(port, physnets):
+    """Check whether a port's physical network is allowed for a VIF.
+
+    Supports VIFs on networks with no physical network configuration by
+    allowing all ports regardless of their physical network. This will be the
+    case when the port is not a neutron port because we're in standalone mode
+    or not using neutron.
+
+    Allows ports with physical_network=None to ensure backwards compatibility
+    and provide support for simple deployments with no physical network
+    configuration in ironic.
+
+    When the physnets set is not empty and the port's physical_network field is
+    not None, the port's physical_network field must be present in the physnets
+    set.
+
+    :param port: A Port object to check.
+    :param physnets: Set of physical networks on which the VIF may be
+        attached. This is governed by the segments of the VIF's network. An
+        empty set indicates that the ports' physical networks should be
+        ignored.
+    :returns: True if the port's physical network is allowed, False otherwise.
+    """
+    return (not physnets or
+            port.physical_network is None or
+            port.physical_network in physnets)
+
+
+def _get_free_portgroups_and_ports(task, vif_id, physnets):
     """Get free portgroups and ports.
 
     It only returns ports or portgroups that can be used for attachment of
@@ -62,13 +91,18 @@ def _get_free_portgroups_and_ports(task, vif_id):
 
     :param task: a TaskManager instance.
     :param vif_id: Name or UUID of a VIF.
-    :returns: tuple of: list of free portgroups, list of free ports.
+    :param physnets: Set of physical networks on which the VIF may be
+        attached. This is governed by the segments of the VIF's network. An
+        empty set indicates that the ports' physical networks should be
+        ignored.
+    :returns: list of free ports and portgroups.
     :raises: VifAlreadyAttached, if vif_id is attached to any of the
         node's ports or portgroups.
     """
 
-    # This list contains ports selected as candidates for attachment
-    free_ports = []
+    # This list contains ports and portgroups selected as candidates for
+    # attachment.
+    free_port_like_objs = []
     # This is a mapping of portgroup id to collection of its free ports
     ports_by_portgroup = collections.defaultdict(list)
     # This set contains IDs of portgroups that should be ignored, as they have
@@ -84,58 +118,88 @@ def _get_free_portgroups_and_ports(task, vif_id):
             # added in this set is not a problem
             non_usable_portgroups.add(p.portgroup_id)
             continue
+        if not _is_port_physnet_allowed(p, physnets):
+            continue
         if p.portgroup_id is None:
             # ports without portgroup_id are always considered candidates
-            free_ports.append(p)
+            free_port_like_objs.append(p)
         else:
             ports_by_portgroup[p.portgroup_id].append(p)
 
-    # This list contains portgroups selected as candidates for attachment
-    free_portgroups = []
-
     for pg in task.portgroups:
         if _vif_attached(pg, vif_id):
             continue
         if pg.id in non_usable_portgroups:
             # This portgroup has vifs attached to its ports, consider its
             # ports instead to avoid collisions
-            free_ports.extend(ports_by_portgroup[pg.id])
+            free_port_like_objs.extend(ports_by_portgroup[pg.id])
         # Also ignore empty portgroups
         elif ports_by_portgroup[pg.id]:
-            free_portgroups.append(pg)
+            free_port_like_objs.append(pg)
 
-    return free_portgroups, free_ports
+    return free_port_like_objs
 
 
-def get_free_port_like_object(task, vif_id):
+def get_free_port_like_object(task, vif_id, physnets):
     """Find free port-like object (portgroup or port) VIF will be attached to.
 
-    Ensures that VIF is not already attached to this node. It will return the
-    first free port group. If there are no free port groups, then the first
-    available port (pxe_enabled preferably) is used.
+    Ensures that the VIF is not already attached to this node.  When selecting
+    a port or portgroup to attach the virtual interface to, the following
+    ordered criteria are applied:
+
+    * Require ports or portgroups to have a physical network that is either
+      None or one of the VIF's allowed physical networks.
+    * Prefer ports or portgroups with a physical network field which is not
+      None.
+    * Prefer portgroups to ports.
+    * Prefer ports with PXE enabled.
 
     :param task: a TaskManager instance.
     :param vif_id: Name or UUID of a VIF.
+    :param physnets: Set of physical networks on which the VIF may be
+        attached. This is governed by the segments of the VIF's network. An
+        empty set indicates that the ports' physical networks should be
+        ignored.
     :raises: VifAlreadyAttached, if VIF is already attached to the node.
     :raises: NoFreePhysicalPorts, if there is no port-like object VIF can be
         attached to.
+    :raises: PortgroupPhysnetInconsistent if one of the node's portgroups
+             has ports which are not all assigned the same physical network.
     :returns: port-like object VIF will be attached to.
     """
+    free_port_like_objs = _get_free_portgroups_and_ports(task, vif_id,
+                                                         physnets)
 
-    free_portgroups, free_ports = _get_free_portgroups_and_ports(task, vif_id)
-
-    if free_portgroups:
-        # portgroups are higher priority
-        return free_portgroups[0]
-
-    if not free_ports:
+    if not free_port_like_objs:
         raise exception.NoFreePhysicalPorts(vif=vif_id)
 
-    # Sort ports by pxe_enabled to ensure we always bind pxe_enabled ports
-    # first
-    sorted_free_ports = sorted(free_ports, key=lambda p: p.pxe_enabled,
-                               reverse=True)
-    return sorted_free_ports[0]
+    def sort_key(port_like_obj):
+        """Key function for sorting a combined list of ports and portgroups.
+
+        We key the port-like objects using the following precedence:
+
+        1. Prefer objects with a physical network field which is in the
+           physnets set.
+        2. Prefer portgroups to ports.
+        3. Prefer ports with PXE enabled.
+
+        :param port_like_obj: The port or portgroup to key.
+        :returns: A key value for sorting the object.
+        """
+        is_pg = isinstance(port_like_obj, objects.Portgroup)
+        if is_pg:
+            pg_physnets = network.get_physnets_by_portgroup_id(
+                task, port_like_obj.id)
+            pg_physnet = pg_physnets.pop()
+            physnet_matches = pg_physnet in physnets
+            pxe_enabled = True
+        else:
+            physnet_matches = port_like_obj.physical_network in physnets
+            pxe_enabled = port_like_obj.pxe_enabled
+        return (physnet_matches, is_pg, pxe_enabled)
+
+    sorted_free_plos = sorted(free_port_like_objs, key=sort_key, reverse=True)
+    return sorted_free_plos[0]
 
 
 def plug_port_to_tenant_network(task, port_like_obj, client=None):
@@ -150,6 +214,7 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
 
     node = task.node
     local_link_info = []
+    local_group_info = None
     client_id_opt = None
 
     vif_id = (
@@ -172,6 +237,8 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
                     if p.portgroup_id == port_like_obj.id]
         for port in pg_ports:
             local_link_info.append(port.local_link_connection)
+        local_group_info = neutron.get_local_group_information(
+            task, port_like_obj)
     else:
         # We iterate only on ports or portgroups, no need to check
         # that it is a port
@@ -187,11 +254,13 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
         'port': {
             'binding:vnic_type': 'baremetal',
             'binding:host_id': node.uuid,
-            'binding:profile': {
-                'local_link_information': local_link_info,
-            },
         }
     }
+    binding_profile = {'local_link_information': local_link_info}
+    if local_group_info:
+        binding_profile['local_group_information'] = local_group_info
+    body['port']['binding:profile'] = binding_profile
+
     if client_id_opt:
         body['port']['extra_dhcp_opts'] = [client_id_opt]
 
@@ -211,6 +280,107 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
 
 
 class VIFPortIDMixin(object):
+    """VIF port ID mixin class for non-neutron network interfaces.
+
+    Mixin class that provides VIF-related network interface methods for
+    non-neutron network interfaces. There are no effects due to VIF
+    attach/detach that are external to ironic.
+
+    NOTE: This does not yet support the full set of VIF methods, as it does
+    not provide vif_attach, vif_detach, port_changed, or portgroup_changed.
+    """
+
+    @staticmethod
+    def _save_vif_to_port_like_obj(port_like_obj, vif_id):
+        """Save the ID of a VIF to a port or portgroup.
+
+        :param port_like_obj: port-like object to save to.
+        :param vif_id: VIF ID to save.
+        """
+        int_info = port_like_obj.internal_info
+        int_info[TENANT_VIF_KEY] = vif_id
+        port_like_obj.internal_info = int_info
+        port_like_obj.save()
+
+    @staticmethod
+    def _clear_vif_from_port_like_obj(port_like_obj):
+        """Clear the VIF ID field from a port or portgroup.
+
+        :param port_like_obj: port-like object to clear from.
+        """
+        int_info = port_like_obj.internal_info
+        extra = port_like_obj.extra
+        int_info.pop(TENANT_VIF_KEY, None)
+        extra.pop('vif_port_id', None)
+        port_like_obj.extra = extra
+        port_like_obj.internal_info = int_info
+        port_like_obj.save()
+
+    def _get_port_like_obj_by_vif_id(self, task, vif_id):
+        """Lookup a port or portgroup by its attached VIF ID.
+
+        :param task: A TaskManager instance.
+        :param vif_id: ID of the attached VIF.
+        :returns: A Port or Portgroup object to which the VIF is attached.
+        :raises: VifNotAttached if the VIF is not attached.
+        """
+        for port_like_obj in task.portgroups + task.ports:
+            vif_port_id = self._get_vif_id_by_port_like_obj(port_like_obj)
+            if vif_port_id == vif_id:
+                return port_like_obj
+        raise exception.VifNotAttached(vif=vif_id, node=task.node.uuid)
+
+    @staticmethod
+    def _get_vif_id_by_port_like_obj(port_like_obj):
+        """Lookup the VIF attached to a port or portgroup.
+
+        :param port_like_obj: A port or portgroup to check.
+        :returns: The ID of the attached VIF, or None.
+        """
+        # FIXME(sambetts) Remove this when we no longer support a nova
+        # driver that uses port.extra
+        return (port_like_obj.internal_info.get(TENANT_VIF_KEY) or
+                port_like_obj.extra.get('vif_port_id'))
+
+    def vif_list(self, task):
+        """List attached VIF IDs for a node
+
+        :param task: A TaskManager instance.
+        :returns: List of VIF dictionaries, each dictionary will have an 'id'
+            entry with the ID of the VIF.
+        """
+        vifs = []
+        for port_like_obj in task.ports + task.portgroups:
+            vif_id = self._get_vif_id_by_port_like_obj(port_like_obj)
+            if vif_id:
+                vifs.append({'id': vif_id})
+        return vifs
+
+    def get_current_vif(self, task, p_obj):
+        """Returns the currently used VIF associated with port or portgroup
+
+        We are booting the node only in one network at a time, and presence of
+        cleaning_vif_port_id means we're doing cleaning, of
+        provisioning_vif_port_id - provisioning.
+        Otherwise it's a tenant network
+
+        :param task: A TaskManager instance.
+        :param p_obj: Ironic port or portgroup object.
+        :returns: VIF ID associated with p_obj or None.
+        """
+
+        return (p_obj.internal_info.get('cleaning_vif_port_id') or
+                p_obj.internal_info.get('provisioning_vif_port_id') or
+                self._get_vif_id_by_port_like_obj(p_obj) or None)
+
+
+class NeutronVIFPortIDMixin(VIFPortIDMixin):
+    """VIF port ID mixin class for neutron network interfaces.
+
+    Mixin class that provides VIF-related network interface methods for neutron
+    network interfaces. On VIF attach/detach, the associated neutron port will
+    be updated.
+    """
 
     def port_changed(self, task, port_obj):
         """Handle any actions required when a port changes
@@ -227,8 +397,7 @@ class VIFPortIDMixin(object):
         if port_obj.portgroup_id:
             portgroup_obj = [pg for pg in task.portgroups if
                              pg.id == port_obj.portgroup_id][0]
-        vif = (port_obj.internal_info.get(TENANT_VIF_KEY) or
-               port_obj.extra.get('vif_port_id'))
+        vif = self._get_vif_id_by_port_like_obj(port_obj)
         if 'address' in port_obj.obj_what_changed():
             if vif:
                 neutron.update_port_address(vif, port_obj.address)
@@ -254,10 +423,10 @@ class VIFPortIDMixin(object):
                 # Log warning if there is no VIF and an instance
                 # is associated with the node.
                 elif node.instance_uuid:
-                    LOG.warning(_LW(
+                    LOG.warning(
                         "No VIF found for instance %(instance)s "
                         "port %(port)s when attempting to update port "
-                        "client-id."),
+                        "client-id.",
                         {'port': port_uuid,
                          'instance': node.instance_uuid})
 
@@ -294,8 +463,7 @@ class VIFPortIDMixin(object):
         # Do not touch neutron port if we removed address on portgroup.
         if ('address' in portgroup_obj.obj_what_changed() and
                 portgroup_obj.address):
-            pg_vif = (portgroup_obj.internal_info.get(TENANT_VIF_KEY) or
-                      portgroup_obj.extra.get('vif_port_id'))
+            pg_vif = self._get_vif_id_by_port_like_obj(portgroup_obj)
             if pg_vif:
                 neutron.update_port_address(pg_vif, portgroup_obj.address)
 
@@ -313,8 +481,7 @@ class VIFPortIDMixin(object):
                 ports = [p for p in task.ports if
                          p.portgroup_id == portgroup_obj.id]
                 for p in ports:
-                    vif = p.internal_info.get(
-                        TENANT_VIF_KEY, p.extra.get('vif_port_id'))
+                    vif = self._get_vif_id_by_port_like_obj(p)
                     reason = []
                     if p.pxe_enabled:
                         reason.append("'pxe_enabled' is set to True")
@@ -329,39 +496,67 @@ class VIFPortIDMixin(object):
                                'reason': ', '.join(reason)})
                         raise exception.Conflict(msg)
 
-    def vif_list(self, task):
-        """List attached VIF IDs for a node
-
-        :param task: A TaskManager instance.
-        :returns: List of VIF dictionaries, each dictionary will have an 'id'
-            entry with the ID of the VIF.
-        """
-        vifs = []
-        for port_like_obj in task.ports + task.portgroups:
-            vif_id = port_like_obj.internal_info.get(
-                TENANT_VIF_KEY, port_like_obj.extra.get('vif_port_id'))
-            if vif_id:
-                vifs.append({'id': vif_id})
-        return vifs
-
     def vif_attach(self, task, vif_info):
         """Attach a virtual network interface to a node
 
-        Attach a virtual interface to a node. It will use the first free port
-        group. If there are no free port groups, then the first available port
-        (pxe_enabled preferably) is used.
+        Attach a virtual interface to a node.  When selecting a port or
+        portgroup to attach the virtual interface to, the following ordered
+        criteria are applied:
+
+        * Require ports or portgroups to have a physical network that is either
+          None or one of the VIF's allowed physical networks.
+        * Prefer ports or portgroups with a physical network field which is not
+          None.
+        * Prefer portgroups to ports.
+        * Prefer ports with PXE enabled.
 
         :param task: A TaskManager instance.
         :param vif_info: a dictionary of information about a VIF.
              It must have an 'id' key, whose value is a unique
              identifier for that VIF.
         :raises: NetworkError, VifAlreadyAttached, NoFreePhysicalPorts
+        :raises: PortgroupPhysnetInconsistent if one of the node's portgroups
+                 has ports which are not all assigned the same physical
+                 network.
         """
         vif_id = vif_info['id']
-
-        port_like_obj = get_free_port_like_object(task, vif_id)
-
         client = neutron.get_client()
+
+        # Determine whether any of the node's ports have a physical network. If
+        # not, we don't need to check the VIF's network's physical networks as
+        # they will not affect the VIF to port mapping.
+        physnets = set()
+        if any(port.physical_network is not None for port in task.ports):
+            try:
+                physnets = neutron.get_physnets_by_port_uuid(client, vif_id)
+            except (exception.InvalidParameterValue, exception.NetworkError):
+                # TODO(mgoddard): Remove this except clause and handle errors
+                # properly. We can do this once a strategy has been determined
+                # for handling the tempest VIF tests in an environment that
+                # may not support neutron.
+                # NOTE(sambetts): If a client error occurs this is because
+                # either neutron doesn't exist because we're running in
+                # standalone environment or we can't find a matching neutron
+                # port which means a user might be requesting a non-neutron
+                # port. Assume no physical network information exists in these
+                # cases.
+                pass
+
+            if len(physnets) > 1:
+                # NOTE(mgoddard): Neutron cannot currently handle hosts which
+                # are mapped to multiple segments in the same routed network.
+                node_physnets = network.get_physnets_for_node(task)
+                if len(node_physnets.intersection(physnets)) > 1:
+                    reason = _("Node has ports which map to multiple segments "
+                               "of the routed network to which the VIF is "
+                               "attached. Currently neutron only supports "
+                               "hosts which map to one segment of a routed "
+                               "network")
+                    raise exception.VifInvalidForAttach(
+                        node=task.node.uuid, vif=vif_id, reason=reason)
+
+        port_like_obj = get_free_port_like_object(task, vif_id, physnets)
+
         # Address is optional for portgroups
         if port_like_obj.address:
             # Check if the requested vif_id is a neutron port. If it is
@@ -369,6 +564,10 @@ class VIFPortIDMixin(object):
             try:
                 client.show_port(vif_id)
             except neutron_exceptions.NeutronClientException:
+                # TODO(mgoddard): Remove this except clause and handle errors
+                # properly. We can do this once a strategy has been determined
+                # for handling the tempest VIF tests in an environment that
+                # may not support neutron.
                 # NOTE(sambetts): If a client error occurs this is because
                 # either neutron doesn't exist because we're running in
                 # standalone environment or we can't find a matching neutron
@@ -387,10 +586,8 @@ class VIFPortIDMixin(object):
                             'vif': vif_id, 'port': vif_id,
                             'mac': port_like_obj.address})
 
-        int_info = port_like_obj.internal_info
-        int_info[TENANT_VIF_KEY] = vif_id
-        port_like_obj.internal_info = int_info
-        port_like_obj.save()
+        self._save_vif_to_port_like_obj(port_like_obj, vif_id)
+
         # NOTE(vsaienko) allow to attach VIF to active instance.
         if task.node.provision_state == states.ACTIVE:
             plug_port_to_tenant_network(task, port_like_obj, client=client)
@@ -403,44 +600,12 @@ class VIFPortIDMixin(object):
         :raises: VifNotAttached if VIF not attached.
         :raises: NetworkError: if unbind Neutron port failed.
         """
+        # NOTE(mgoddard): Lookup the port first to check that the VIF is
+        # attached, and fail if not.
+        port_like_obj = self._get_port_like_obj_by_vif_id(task, vif_id)
 
-        # NOTE(vsaienko) We picking object to attach on vif-attach side.
-        # Here we should only detach VIF and shouldn't duplicate/follow
-        # attach rules, just walk over all objects and detach VIF.
-        for port_like_obj in task.portgroups + task.ports:
-            # FIXME(sambetts) Remove this when we no longer support a nova
-            # driver that uses port.extra
-            vif_port_id = port_like_obj.internal_info.get(
-                TENANT_VIF_KEY, port_like_obj.extra.get("vif_port_id"))
-            if vif_port_id == vif_id:
-                int_info = port_like_obj.internal_info
-                extra = port_like_obj.extra
-                int_info.pop(TENANT_VIF_KEY, None)
-                extra.pop('vif_port_id', None)
-                port_like_obj.extra = extra
-                port_like_obj.internal_info = int_info
-                port_like_obj.save()
-                # NOTE(vsaienko) allow to unplug VIFs from ACTIVE instance.
-                if task.node.provision_state == states.ACTIVE:
-                    neutron.unbind_neutron_port(vif_port_id)
-                break
-        else:
-            raise exception.VifNotAttached(vif=vif_id, node=task.node.uuid)
+        self._clear_vif_from_port_like_obj(port_like_obj)
 
-    def get_current_vif(self, task, p_obj):
-        """Returns the currently used VIF associated with port or portgroup
-
-        We are booting the node only in one network at a time, and presence of
-        cleaning_vif_port_id means we're doing cleaning, of
-        provisioning_vif_port_id - provisioning.
-        Otherwise it's a tenant network
-
-        :param task: A TaskManager instance.
-        :param p_obj: Ironic port or portgroup object.
-        :returns: VIF ID associated with p_obj or None.
-        """
-
-        return (p_obj.internal_info.get('cleaning_vif_port_id') or
-                p_obj.internal_info.get('provisioning_vif_port_id') or
-                p_obj.internal_info.get(TENANT_VIF_KEY) or
-                p_obj.extra.get('vif_port_id') or None)
+        # NOTE(vsaienko) allow to unplug VIFs from ACTIVE instance.
+        if task.node.provision_state == states.ACTIVE:
+            neutron.unbind_neutron_port(vif_id)
