@@ -1,3 +1,4 @@
+# Copyright 2017 Hewlett Packard Enterprise Development Company LP.
 # Copyright 2016 Hewlett Packard Enterprise Development LP.
 # Copyright 2016 Universidade Federal de Campina Grande
 # All Rights Reserved.
@@ -19,17 +20,12 @@ import abc
 from futurist import periodics
 from ironic_lib import metrics_utils
 from oslo_log import log as logging
-import retrying
 import six
 
 from ironic.common import exception
-from ironic.common.i18n import _, _LE, _LI, _LW
 from ironic.common import states
-from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import agent
-from ironic.drivers.modules import agent_base_vendor
-from ironic.drivers.modules import deploy_utils as ironic_deploy_utils
 from ironic.drivers.modules import iscsi_deploy
 from ironic.drivers.modules.oneview import common
 from ironic.drivers.modules.oneview import deploy_utils
@@ -81,18 +77,18 @@ class OneViewPeriodicTasks(object):
                 # remaining nodes. This node will be checked in
                 # the next periodic call.
 
-                LOG.error(_LE("Error while determining if node "
-                              "%(node_uuid)s is in use by OneView. "
-                              "Error: %(error)s"),
+                LOG.error("Error while determining if node "
+                          "%(node_uuid)s is in use by OneView. "
+                          "Error: %(error)s",
                           {'node_uuid': node.uuid, 'error': e})
 
                 continue
 
             if oneview_using:
-                purpose = (_LI('Updating node %(node_uuid)s in use '
-                               'by OneView from %(provision_state)s state '
-                               'to %(target_state)s state and maintenance '
-                               'mode %(maintenance)s.'),
+                purpose = ('Updating node %(node_uuid)s in use '
+                           'by OneView from %(provision_state)s state '
+                           'to %(target_state)s state and maintenance '
+                           'mode %(maintenance)s.',
                            {'node_uuid': node_uuid,
                             'provision_state': states.AVAILABLE,
                             'target_state': states.MANAGEABLE,
@@ -143,18 +139,18 @@ class OneViewPeriodicTasks(object):
                     # remaining nodes. This node will be checked in
                     # the next periodic call.
 
-                    LOG.error(_LE("Error while determining if node "
-                                  "%(node_uuid)s is in use by OneView. "
-                                  "Error: %(error)s"),
+                    LOG.error("Error while determining if node "
+                              "%(node_uuid)s is in use by OneView. "
+                              "Error: %(error)s",
                               {'node_uuid': node.uuid, 'error': e})
 
                     continue
 
                 if not oneview_using:
-                    purpose = (_LI('Bringing node %(node_uuid)s back from '
-                                   'use by OneView from %(provision_state)s '
-                                   'state to %(target_state)s state and '
-                                   'maintenance mode %(maintenance)s.'),
+                    purpose = ('Bringing node %(node_uuid)s back from '
+                               'use by OneView from %(provision_state)s '
+                               'state to %(target_state)s state and '
+                               'maintenance mode %(maintenance)s.',
                                {'node_uuid': node_uuid,
                                 'provision_state': states.MANAGEABLE,
                                 'target_state': states.AVAILABLE,
@@ -202,10 +198,10 @@ class OneViewPeriodicTasks(object):
 
                 node = objects.Node.get(context, node_uuid)
 
-                purpose = (_LI('Bringing node %(node_uuid)s back from use '
-                               'by OneView from %(provision_state)s state '
-                               'to %(target_state)s state and '
-                               'maintenance mode %(maintenance)s.'),
+                purpose = ('Bringing node %(node_uuid)s back from use '
+                           'by OneView from %(provision_state)s state '
+                           'to %(target_state)s state and '
+                           'maintenance mode %(maintenance)s.',
                            {'node_uuid': node_uuid,
                             'provision_state': states.CLEANFAIL,
                             'target_state': states.MANAGEABLE,
@@ -246,118 +242,27 @@ class OneViewIscsiDeploy(iscsi_deploy.ISCSIDeploy, OneViewPeriodicTasks):
 
     @METRICS.timer('OneViewIscsiDeploy.prepare')
     def prepare(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.prepare(self.oneview_client, task)
+        deploy_utils.prepare(self.oneview_client, task)
         super(OneViewIscsiDeploy, self).prepare(task)
 
     @METRICS.timer('OneViewIscsiDeploy.tear_down')
     def tear_down(self, task):
-        if (common.is_dynamic_allocation_enabled(task.node) and
-           not CONF.conductor.automated_clean):
+        if not CONF.conductor.automated_clean:
             deploy_utils.tear_down(self.oneview_client, task)
         return super(OneViewIscsiDeploy, self).tear_down(task)
 
     @METRICS.timer('OneViewIscsiDeploy.prepare_cleaning')
     def prepare_cleaning(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.prepare_cleaning(self.oneview_client, task)
+        deploy_utils.prepare_cleaning(self.oneview_client, task)
         return super(OneViewIscsiDeploy, self).prepare_cleaning(task)
 
     @METRICS.timer('OneViewIscsiDeploy.tear_down_cleaning')
     def tear_down_cleaning(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.tear_down_cleaning(self.oneview_client, task)
+        deploy_utils.tear_down_cleaning(self.oneview_client, task)
         super(OneViewIscsiDeploy, self).tear_down_cleaning(task)
 
 
-# NOTE (thiagop): We overwrite this interface because we cannot change the boot
-# device of OneView managed blades while they are still powered on. We moved
-# the call of node_set_boot_device from reboot_to_instance to
-# reboot_and_finish_deploy and changed the behavior to shutdown the node before
-# doing it.
-# TODO(thiagop): remove this interface once bug/1503855 is fixed
-class OneViewAgentDeployMixin(object):
-
-    @METRICS.timer('OneViewAgentDeployMixin.reboot_to_instance')
-    def reboot_to_instance(self, task, **kwargs):
-        task.process_event('resume')
-        node = task.node
-        error = self.check_deploy_success(node)
-        if error is not None:
-            # TODO(jimrollenhagen) power off if using neutron dhcp to
-            #                      align with pxe driver?
-            msg = (_('node %(node)s command status errored: %(error)s') %
-                   {'node': node.uuid, 'error': error})
-            LOG.error(msg)
-            ironic_deploy_utils.set_failed_state(task, msg)
-            return
-
-        LOG.info(_LI('Image successfully written to node %s'), node.uuid)
-        LOG.debug('Rebooting node %s to instance', node.uuid)
-
-        self.reboot_and_finish_deploy(task)
-
-        # NOTE(TheJulia): If we deployed a whole disk image, we
-        # should expect a whole disk image and clean-up the tftp files
-        # on-disk incase the node is disregarding the boot preference.
-        # TODO(rameshg87): Not all in-tree drivers using reboot_to_instance
-        # have a boot interface. So include a check for now. Remove this
-        # check once all in-tree drivers have a boot interface.
-        if task.driver.boot:
-            task.driver.boot.clean_up_ramdisk(task)
-
-    @METRICS.timer('OneViewAgentDeployMixin.reboot_and_finish_deploy')
-    def reboot_and_finish_deploy(self, task):
-        """Helper method to trigger reboot on the node and finish deploy.
-
-        This method initiates a reboot on the node. On success, it
-        marks the deploy as complete. On failure, it logs the error
-        and marks deploy as failure.
-
-        :param task: a TaskManager object containing the node
-        :raises: InstanceDeployFailure, if node reboot failed.
-        """
-        wait = CONF.agent.post_deploy_get_power_state_retry_interval * 1000
-        attempts = CONF.agent.post_deploy_get_power_state_retries + 1
-
-        @retrying.retry(
-            stop_max_attempt_number=attempts,
-            retry_on_result=lambda state: state != states.POWER_OFF,
-            wait_fixed=wait
-        )
-        def _wait_until_powered_off(task):
-            return task.driver.power.get_power_state(task)
-
-        node = task.node
-
-        try:
-            try:
-                self._client.power_off(node)
-                _wait_until_powered_off(task)
-            except Exception as e:
-                LOG.warning(
-                    _LW('Failed to soft power off node %(node_uuid)s '
-                        'in at least %(timeout)d seconds. Error: %(error)s'),
-                    {'node_uuid': node.uuid,
-                     'timeout': (wait * (attempts - 1)) / 1000,
-                     'error': e})
-                manager_utils.node_power_action(task, states.POWER_OFF)
-
-            manager_utils.node_set_boot_device(task, 'disk',
-                                               persistent=True)
-            manager_utils.node_power_action(task, states.POWER_ON)
-        except Exception as e:
-            msg = (_('Error rebooting node %(node)s after deploy. '
-                     'Error: %(error)s') %
-                   {'node': node.uuid, 'error': e})
-            agent_base_vendor.log_and_raise_deployment_error(task, msg)
-
-        task.process_event('done')
-        LOG.info(_LI('Deployment to node %s done'), task.node.uuid)
-
-
-class OneViewAgentDeploy(OneViewAgentDeployMixin, agent.AgentDeploy,
-                         OneViewPeriodicTasks):
+class OneViewAgentDeploy(agent.AgentDeploy, OneViewPeriodicTasks):
     """Class for OneView Agent deployment driver."""
 
     oneview_driver = common.AGENT_PXE_ONEVIEW
@@ -381,25 +286,21 @@ class OneViewAgentDeploy(OneViewAgentDeployMixin, agent.AgentDeploy,
 
     @METRICS.timer('OneViewAgentDeploy.prepare')
     def prepare(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.prepare(self.oneview_client, task)
+        deploy_utils.prepare(self.oneview_client, task)
         super(OneViewAgentDeploy, self).prepare(task)
 
     @METRICS.timer('OneViewAgentDeploy.tear_down')
     def tear_down(self, task):
-        if (common.is_dynamic_allocation_enabled(task.node) and
-           not CONF.conductor.automated_clean):
+        if not CONF.conductor.automated_clean:
             deploy_utils.tear_down(self.oneview_client, task)
         return super(OneViewAgentDeploy, self).tear_down(task)
 
     @METRICS.timer('OneViewAgentDeploy.prepare_cleaning')
     def prepare_cleaning(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.prepare_cleaning(self.oneview_client, task)
+        deploy_utils.prepare_cleaning(self.oneview_client, task)
         return super(OneViewAgentDeploy, self).prepare_cleaning(task)
 
     @METRICS.timer('OneViewAgentDeploy.tear_down_cleaning')
     def tear_down_cleaning(self, task):
-        if common.is_dynamic_allocation_enabled(task.node):
-            deploy_utils.tear_down_cleaning(self.oneview_client, task)
+        deploy_utils.tear_down_cleaning(self.oneview_client, task)
         super(OneViewAgentDeploy, self).tear_down_cleaning(task)
