@@ -26,21 +26,21 @@ from ironic.drivers import ipmi
 from ironic.drivers.modules import agent
 from ironic.drivers.modules.cimc import management as cimc_mgmt
 from ironic.drivers.modules.cimc import power as cimc_power
+from ironic.drivers.modules.ilo import boot as ilo_boot
 from ironic.drivers.modules.ilo import console as ilo_console
-from ironic.drivers.modules.ilo import deploy as ilo_deploy
 from ironic.drivers.modules.ilo import inspect as ilo_inspect
 from ironic.drivers.modules.ilo import management as ilo_management
 from ironic.drivers.modules.ilo import power as ilo_power
 from ironic.drivers.modules.ilo import vendor as ilo_vendor
 from ironic.drivers.modules import inspector
 from ironic.drivers.modules import ipmitool
+from ironic.drivers.modules.irmc import boot as irmc_boot
 from ironic.drivers.modules.irmc import inspect as irmc_inspect
 from ironic.drivers.modules.irmc import management as irmc_management
 from ironic.drivers.modules.irmc import power as irmc_power
 from ironic.drivers.modules import iscsi_deploy
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules import snmp
-from ironic.drivers.modules import ssh
 from ironic.drivers.modules.ucs import management as ucs_mgmt
 from ironic.drivers.modules.ucs import power as ucs_power
 
@@ -50,40 +50,16 @@ PXEAndIPMIToolDriver = ipmi.PXEAndIPMIToolDriver
 PXEAndIPMIToolAndSocatDriver = ipmi.PXEAndIPMIToolAndSocatDriver
 
 
-class PXEAndSSHDriver(base.BaseDriver):
-    """PXE + SSH driver.
-
-    NOTE: This driver is meant only for testing environments.
-
-    This driver implements the `core` functionality, combining
-    :class:`ironic.drivers.modules.ssh.SSH` for power on/off and
-    reboot of virtual machines tunneled over SSH, with
-    :class:`ironic.drivers.modules.iscsi_deploy.ISCSIDeploy` for
-    image deployment. Implementations are in those respective
-    classes; this class is merely the glue between them.
-    """
-
-    supported = False
-
-    def __init__(self):
-        self.power = ssh.SSHPower()
-        self.boot = pxe.PXEBoot()
-        self.deploy = iscsi_deploy.ISCSIDeploy()
-        self.management = ssh.SSHManagement()
-        self.inspect = inspector.Inspector.create_if_enabled(
-            'PXEAndSSHDriver')
-        self.raid = agent.AgentRAID()
-        self.console = ssh.ShellinaboxConsole()
-
-
 class PXEAndIloDriver(base.BaseDriver):
     """PXE + Ilo Driver using IloClient interface.
 
     This driver implements the `core` functionality using
     :class:`ironic.drivers.modules.ilo.power.IloPower` for
     power management
-    :class:`ironic.drivers.modules.ilo.deploy.IloPXEDeploy` for image
-    deployment.
+    :class:`ironic.drivers.modules.iscsi_deploy.ISCSIDeploy` for
+    image deployment.
+    :class:`ironic.drivers.modules.ilo.boot.IloPXEBoot` for boot
+    related actions.
     """
     def __init__(self):
         if not importutils.try_import('proliantutils'):
@@ -91,8 +67,8 @@ class PXEAndIloDriver(base.BaseDriver):
                 driver=self.__class__.__name__,
                 reason=_("Unable to import proliantutils library"))
         self.power = ilo_power.IloPower()
-        self.boot = pxe.PXEBoot()
-        self.deploy = ilo_deploy.IloPXEDeploy()
+        self.boot = ilo_boot.IloPXEBoot()
+        self.deploy = iscsi_deploy.ISCSIDeploy()
         self.vendor = ilo_vendor.VendorPassthru()
         self.console = ilo_console.IloConsoleInterface()
         self.management = ilo_management.IloManagement()
@@ -140,7 +116,7 @@ class PXEAndIRMCDriver(base.BaseDriver):
                 reason=_("Unable to import python-scciclient library"))
         self.power = irmc_power.IRMCPower()
         self.console = ipmitool.IPMIShellinaboxConsole()
-        self.boot = pxe.PXEBoot()
+        self.boot = irmc_boot.IRMCPXEBoot()
         self.deploy = iscsi_deploy.ISCSIDeploy()
         self.management = irmc_management.IRMCManagement()
         self.inspect = irmc_inspect.IRMCInspect()
