@@ -127,15 +127,6 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'vendor_interface': object_fields.StringField(nullable=True),
     }
 
-    def as_dict(self, secure=False):
-        d = super(Node, self).as_dict()
-        if secure:
-            d['driver_info'] = strutils.mask_dict_password(
-                d.get('driver_info', {}), "******")
-            d['instance_info'] = strutils.mask_dict_password(
-                d.get('instance_info', {}), "******")
-        return d
-
     def _validate_property_values(self, properties):
         """Check if the input of local_gb, cpus and memory_mb are valid.
 
@@ -171,6 +162,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     def get(cls, context, node_id):
         """Find a node based on its id or uuid and return a Node object.
 
+        :param context: Security context
         :param node_id: the id *or* uuid of a node.
         :returns: a :class:`Node` object.
         """
@@ -187,13 +179,15 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def get_by_id(cls, context, node_id):
-        """Find a node based on its integer id and return a Node object.
+        """Find a node based on its integer ID and return a Node object.
 
-        :param node_id: the id of a node.
+        :param cls: the :class:`Node`
+        :param context: Security context
+        :param node_id: the ID of a node.
         :returns: a :class:`Node` object.
         """
         db_node = cls.dbapi.get_node_by_id(node_id)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -202,13 +196,15 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def get_by_uuid(cls, context, uuid):
-        """Find a node based on uuid and return a Node object.
+        """Find a node based on UUID and return a Node object.
 
-        :param uuid: the uuid of a node.
+        :param cls: the :class:`Node`
+        :param context: Security context
+        :param uuid: the UUID of a node.
         :returns: a :class:`Node` object.
         """
         db_node = cls.dbapi.get_node_by_uuid(uuid)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -219,11 +215,13 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     def get_by_name(cls, context, name):
         """Find a node based on name and return a Node object.
 
+        :param cls: the :class:`Node`
+        :param context: Security context
         :param name: the logical name of a node.
         :returns: a :class:`Node` object.
         """
         db_node = cls.dbapi.get_node_by_name(name)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -232,13 +230,15 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def get_by_instance_uuid(cls, context, instance_uuid):
-        """Find a node based on the instance uuid and return a Node object.
+        """Find a node based on the instance UUID and return a Node object.
 
-        :param uuid: the uuid of the instance.
+        :param cls: the :class:`Node`
+        :param context: Security context
+        :param uuid: the UUID of the instance.
         :returns: a :class:`Node` object.
         """
         db_node = cls.dbapi.get_node_by_instance(instance_uuid)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -250,6 +250,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
              sort_dir=None, filters=None):
         """Return a list of Node objects.
 
+        :param cls: the :class:`Node`
         :param context: Security context.
         :param limit: maximum number of resources to return in a single result.
         :param marker: pagination marker for large data sets.
@@ -275,15 +276,16 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         To prevent other ManagerServices from manipulating the given
         Node while a Task is performed, mark it reserved by this host.
 
+        :param cls: the :class:`Node`
         :param context: Security context.
         :param tag: A string uniquely identifying the reservation holder.
-        :param node_id: A node id or uuid.
+        :param node_id: A node ID or UUID.
         :raises: NodeNotFound if the node is not found.
         :returns: a :class:`Node` object.
 
         """
         db_node = cls.dbapi.reserve_node(tag, node_id)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
@@ -322,10 +324,10 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                         object, e.g.: Node(context)
         :raises: InvalidParameterValue if some property values are invalid.
         """
-        values = self.obj_get_changes()
+        values = self.do_version_changes_for_db()
         self._validate_property_values(values.get('properties'))
         db_node = self.dbapi.create_node(values)
-        self._from_db_object(self, db_node)
+        self._from_db_object(self._context, self, db_node)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -364,20 +366,14 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                         object, e.g.: Node(context)
         :raises: InvalidParameterValue if some property values are invalid.
         """
-        updates = self.obj_get_changes()
+        updates = self.do_version_changes_for_db()
         self._validate_property_values(updates.get('properties'))
         if 'driver' in updates and 'driver_internal_info' not in updates:
             # Clean driver_internal_info when changes driver
             self.driver_internal_info = {}
-            updates = self.obj_get_changes()
+            updates = self.do_version_changes_for_db()
         db_node = self.dbapi.update_node(self.uuid, updates)
-
-        # TODO(galyna): updating specific field not touching others to not
-        # change default behaviour. Otherwise it will break a bunch of tests
-        # This can be updated in other way when more fields like `updated_at`
-        # will appear
-        self.updated_at = db_node['updated_at']
-        self.obj_reset_changes()
+        self._from_db_object(self._context, self, db_node)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -409,13 +405,14 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     def get_by_port_addresses(cls, context, addresses):
         """Get a node by associated port addresses.
 
+        :param cls: the :class:`Node`
         :param context: Security context.
         :param addresses: A list of port addresses.
         :raises: NodeNotFound if the node is not found.
         :returns: a :class:`Node` object.
         """
         db_node = cls.dbapi.get_node_by_port_addresses(addresses)
-        node = cls._from_db_object(cls(context), db_node)
+        node = cls._from_db_object(context, cls(), db_node)
         return node
 
 
@@ -451,6 +448,7 @@ class NodePayload(notification.NotificationPayloadBase):
         'network_interface': ('node', 'network_interface'),
         'power_interface': ('node', 'power_interface'),
         'raid_interface': ('node', 'raid_interface'),
+        'storage_interface': ('node', 'storage_interface'),
         'vendor_interface': ('node', 'vendor_interface'),
         'power_state': ('node', 'power_state'),
         'properties': ('node', 'properties'),
@@ -462,17 +460,14 @@ class NodePayload(notification.NotificationPayloadBase):
         'updated_at': ('node', 'updated_at'),
         'uuid': ('node', 'uuid')
     }
-    # TODO(TheJulia): At a later point in time, once storage_interfaces
-    # are able to be leveraged, we need to add the sotrage_interface
-    # field to payload and increment the object versions for all objects
-    # that inherit the NodePayload object.
 
     # Version 1.0: Initial version, based off of Node version 1.18.
     # Version 1.1: Type of network_interface changed to just nullable string
     #              similar to version 1.20 of Node.
     # Version 1.2: Add nullable to console_enabled and maintenance.
     # Version 1.3: Add dynamic interfaces fields exposed via API.
-    VERSION = '1.3'
+    # Version 1.4: Add storage interface field exposed via API.
+    VERSION = '1.4'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'console_enabled': object_fields.BooleanField(nullable=True),
@@ -493,6 +488,7 @@ class NodePayload(notification.NotificationPayloadBase):
         'network_interface': object_fields.StringField(nullable=True),
         'power_interface': object_fields.StringField(nullable=True),
         'raid_interface': object_fields.StringField(nullable=True),
+        'storage_interface': object_fields.StringField(nullable=True),
         'vendor_interface': object_fields.StringField(nullable=True),
         'name': object_fields.StringField(nullable=True),
         'power_state': object_fields.StringField(nullable=True),
@@ -529,7 +525,8 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.1: Parent NodePayload version 1.1
     # Version 1.2: Parent NodePayload version 1.2
     # Version 1.3: Parent NodePayload version 1.3
-    VERSION = '1.3'
+    # Version 1.4: Parent NodePayload version 1.4
+    VERSION = '1.4'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -573,7 +570,8 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.1: Parent NodePayload version 1.1
     # Version 1.2: Parent NodePayload version 1.2
     # Version 1.3: Parent NodePayload version 1.3
-    VERSION = '1.3'
+    # Version 1.4: Parent NodePayload version 1.4
+    VERSION = '1.4'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -602,7 +600,8 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.1: Parent NodePayload version 1.1
     # Version 1.2: Parent NodePayload version 1.2
     # Version 1.3: Parent NodePayload version 1.3
-    VERSION = '1.3'
+    # Version 1.4: Parent NodePayload version 1.4
+    VERSION = '1.4'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info')})
@@ -637,7 +636,8 @@ class NodeCRUDPayload(NodePayload):
     """Payload schema for when ironic creates, updates or deletes a node."""
     # Version 1.0: Initial version
     # Version 1.1: Parent NodePayload version 1.3
-    VERSION = '1.1'
+    # Version 1.2: Parent NodePayload version 1.4
+    VERSION = '1.2'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
