@@ -21,7 +21,6 @@ from glanceclient import client as glance_client
 from glanceclient import exc as glance_exc
 import mock
 from oslo_config import cfg
-from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 from six.moves.urllib import parse as urlparse
 import testtools
@@ -53,41 +52,35 @@ class TestGlanceSerializer(testtools.TestCase):
                     'foo': 'bar',
                     'properties': {
                         'prop1': 'propvalue1',
-                        'mappings': [
-                            {'virtual': 'aaa',
-                             'device': 'bbb'},
-                            {'virtual': 'xxx',
-                             'device': 'yyy'}],
-                        'block_device_mapping': [
-                            {'virtual_device': 'fake',
-                             'device_name': '/dev/fake'},
-                            {'virtual_device': 'ephemeral0',
-                             'device_name': '/dev/fake0'}]}}
+                        'mappings': '['
+                        '{"virtual":"aaa","device":"bbb"},'
+                        '{"virtual":"xxx","device":"yyy"}]',
+                        'block_device_mapping': '['
+                        '{"virtual_device":"fake","device_name":"/dev/fake"},'
+                        '{"virtual_device":"ephemeral0",'
+                        '"device_name":"/dev/fake0"}]'}}
 
-        converted_expected = {
+        expected = {
             'name': 'image1',
             'is_public': True,
             'foo': 'bar',
-            'properties': {'prop1': 'propvalue1'}
+            'properties': {'prop1': 'propvalue1',
+                           'mappings': [
+                               {'virtual': 'aaa',
+                                'device': 'bbb'},
+                               {'virtual': 'xxx',
+                                'device': 'yyy'},
+                           ],
+                           'block_device_mapping': [
+                               {'virtual_device': 'fake',
+                                'device_name': '/dev/fake'},
+                               {'virtual_device': 'ephemeral0',
+                                'device_name': '/dev/fake0'}
+                           ]
+                           }
         }
-        converted = service_utils._convert(metadata, 'to')
-        self.assertEqual(metadata,
-                         service_utils._convert(converted, 'from'))
-        # Fields that rely on dict ordering can't be compared as text
-        mappings = jsonutils.loads(converted['properties']
-                                   .pop('mappings'))
-        self.assertEqual([{"device": "bbb", "virtual": "aaa"},
-                          {"device": "yyy", "virtual": "xxx"}],
-                         mappings)
-        bd_mapping = jsonutils.loads(converted['properties']
-                                     .pop('block_device_mapping'))
-        self.assertEqual([{"virtual_device": "fake",
-                           "device_name": "/dev/fake"},
-                          {"virtual_device": "ephemeral0",
-                           "device_name": "/dev/fake0"}],
-                         bd_mapping)
-        # Compare the remaining
-        self.assertEqual(converted_expected, converted)
+        converted = service_utils._convert(metadata)
+        self.assertEqual(expected, converted)
 
 
 class TestGlanceImageService(base.TestCase):
@@ -122,7 +115,7 @@ class TestGlanceImageService(base.TestCase):
                    'status': None,
                    'is_public': None}
         fixture.update(kwargs)
-        return fixture
+        return stubs.FakeImage(fixture)
 
     @property
     def endpoint(self):
@@ -141,250 +134,10 @@ class TestGlanceImageService(base.TestCase):
                                   updated_at=self.NOW_GLANCE_FORMAT,
                                   deleted_at=self.NOW_GLANCE_FORMAT)
 
-    def test_create_with_instance_id(self):
-        # Ensure instance_id is persisted as an image-property.
-        fixture = {'name': 'test image',
-                   'is_public': False,
-                   'properties': {'instance_id': '42', 'user_id': 'fake'}}
-        image_id = self.service.create(fixture)['id']
-        image_meta = self.service.show(image_id)
-        expected = {
-            'id': image_id,
-            'name': 'test image',
-            'is_public': False,
-            'size': None,
-            'min_disk': None,
-            'min_ram': None,
-            'disk_format': None,
-            'container_format': None,
-            'checksum': None,
-            'created_at': self.NOW_DATETIME,
-            'updated_at': self.NOW_DATETIME,
-            'deleted_at': None,
-            'deleted': None,
-            'status': None,
-            'properties': {'instance_id': '42', 'user_id': 'fake'},
-            'owner': None,
-        }
-
-        self.assertEqual(expected, image_meta)
-
-        image_metas = self.service.detail()
-        self.assertEqual(expected, image_metas[0])
-
-    def test_create_without_instance_id(self):
-        """Test creating an image without an instance ID.
-
-        Ensure we can create an image without having to specify an
-        instance_id. Public images are an example of an image not tied to an
-        instance.
-        """
-        fixture = {'name': 'test image', 'is_public': False}
-        image_id = self.service.create(fixture)['id']
-
-        expected = {
-            'id': image_id,
-            'name': 'test image',
-            'is_public': False,
-            'size': None,
-            'min_disk': None,
-            'min_ram': None,
-            'disk_format': None,
-            'container_format': None,
-            'checksum': None,
-            'created_at': self.NOW_DATETIME,
-            'updated_at': self.NOW_DATETIME,
-            'deleted_at': None,
-            'deleted': None,
-            'status': None,
-            'properties': {},
-            'owner': None,
-        }
-        actual = self.service.show(image_id)
-        self.assertEqual(expected, actual)
-
-    def test_create(self):
-        fixture = self._make_fixture(name='test image')
-        num_images = len(self.service.detail())
-        image_id = self.service.create(fixture)['id']
-
-        self.assertIsNotNone(image_id)
-        self.assertEqual(
-            num_images + 1, len(self.service.detail()))
-
-    def test_create_and_show_non_existing_image(self):
-        fixture = self._make_fixture(name='test image')
-        image_id = self.service.create(fixture)['id']
-
-        self.assertIsNotNone(image_id)
-        self.assertRaises(exception.ImageNotFound,
-                          self.service.show,
-                          'bad image id')
-
-    def test_detail_private_image(self):
-        fixture = self._make_fixture(name='test image')
-        fixture['is_public'] = False
-        properties = {'owner_id': 'proj1'}
-        fixture['properties'] = properties
-
-        self.service.create(fixture)['id']
-
-        proj = self.context.project_id
-        self.context.project_id = 'proj1'
-
-        image_metas = self.service.detail()
-
-        self.context.project_id = proj
-
-        self.assertEqual(1, len(image_metas))
-        self.assertEqual('test image', image_metas[0]['name'])
-        self.assertFalse(image_metas[0]['is_public'])
-
-    def test_detail_marker(self):
-        fixtures = []
-        ids = []
-        for i in range(10):
-            fixture = self._make_fixture(name='TestImage %d' % (i))
-            fixtures.append(fixture)
-            ids.append(self.service.create(fixture)['id'])
-
-        image_metas = self.service.detail(marker=ids[1])
-        self.assertEqual(8, len(image_metas))
-        i = 2
-        for meta in image_metas:
-            expected = {
-                'id': ids[i],
-                'status': None,
-                'is_public': None,
-                'name': 'TestImage %d' % (i),
-                'properties': {},
-                'size': None,
-                'min_disk': None,
-                'min_ram': None,
-                'disk_format': None,
-                'container_format': None,
-                'checksum': None,
-                'created_at': self.NOW_DATETIME,
-                'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
-                'deleted': None,
-                'owner': None,
-            }
-
-            self.assertEqual(expected, meta)
-            i = i + 1
-
-    def test_detail_limit(self):
-        fixtures = []
-        ids = []
-        for i in range(10):
-            fixture = self._make_fixture(name='TestImage %d' % (i))
-            fixtures.append(fixture)
-            ids.append(self.service.create(fixture)['id'])
-
-        image_metas = self.service.detail(limit=5)
-        self.assertEqual(5, len(image_metas))
-
-    def test_detail_default_limit(self):
-        fixtures = []
-        ids = []
-        for i in range(10):
-            fixture = self._make_fixture(name='TestImage %d' % (i))
-            fixtures.append(fixture)
-            ids.append(self.service.create(fixture)['id'])
-
-        image_metas = self.service.detail()
-        for i, meta in enumerate(image_metas):
-            self.assertEqual(meta['name'], 'TestImage %d' % (i))
-
-    def test_detail_marker_and_limit(self):
-        fixtures = []
-        ids = []
-        for i in range(10):
-            fixture = self._make_fixture(name='TestImage %d' % (i))
-            fixtures.append(fixture)
-            ids.append(self.service.create(fixture)['id'])
-
-        image_metas = self.service.detail(marker=ids[3], limit=5)
-        self.assertEqual(5, len(image_metas))
-        i = 4
-        for meta in image_metas:
-            expected = {
-                'id': ids[i],
-                'status': None,
-                'is_public': None,
-                'name': 'TestImage %d' % (i),
-                'properties': {},
-                'size': None,
-                'min_disk': None,
-                'min_ram': None,
-                'disk_format': None,
-                'container_format': None,
-                'checksum': None,
-                'created_at': self.NOW_DATETIME,
-                'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
-                'deleted': None,
-                'owner': None,
-            }
-            self.assertEqual(expected, meta)
-            i = i + 1
-
-    def test_detail_invalid_marker(self):
-        fixtures = []
-        ids = []
-        for i in range(10):
-            fixture = self._make_fixture(name='TestImage %d' % (i))
-            fixtures.append(fixture)
-            ids.append(self.service.create(fixture)['id'])
-
-        self.assertRaises(exception.Invalid, self.service.detail,
-                          marker='invalidmarker')
-
-    def test_update(self):
-        fixture = self._make_fixture(name='test image')
-        image = self.service.create(fixture)
-        image_id = image['id']
-        fixture['name'] = 'new image name'
-        self.service.update(image_id, fixture)
-
-        new_image_data = self.service.show(image_id)
-        self.assertEqual('new image name', new_image_data['name'])
-
-    def test_delete(self):
-        fixture1 = self._make_fixture(name='test image 1')
-        fixture2 = self._make_fixture(name='test image 2')
-        fixtures = [fixture1, fixture2]
-
-        num_images = len(self.service.detail())
-        self.assertEqual(0, num_images)
-
-        ids = []
-        for fixture in fixtures:
-            new_id = self.service.create(fixture)['id']
-            ids.append(new_id)
-
-        num_images = len(self.service.detail())
-        self.assertEqual(2, num_images)
-
-        self.service.delete(ids[0])
-        # When you delete an image from glance, it sets the status to DELETED
-        # and doesn't actually remove the image.
-
-        # Check the image is still there.
-        num_images = len(self.service.detail())
-        self.assertEqual(2, num_images)
-
-        # Check the image is marked as deleted.
-        num_images = len([x for x in self.service.detail()
-                          if not x['deleted']])
-        self.assertEqual(1, num_images)
-
     def test_show_passes_through_to_client(self):
-        fixture = self._make_fixture(name='image1', is_public=True)
-        image_id = self.service.create(fixture)['id']
-
-        image_meta = self.service.show(image_id)
+        image_id = uuidutils.generate_uuid()
+        image = self._make_fixture(name='image1', is_public=True,
+                                   id=image_id)
         expected = {
             'id': image_id,
             'name': 'image1',
@@ -395,65 +148,33 @@ class TestGlanceImageService(base.TestCase):
             'disk_format': None,
             'container_format': None,
             'checksum': None,
-            'created_at': self.NOW_DATETIME,
-            'updated_at': self.NOW_DATETIME,
+            'created_at': None,
+            'updated_at': None,
             'deleted_at': None,
             'deleted': None,
             'status': None,
             'properties': {},
             'owner': None,
         }
+        with mock.patch.object(self.service, 'call', return_value=image):
+            image_meta = self.service.show(image_id)
+            self.service.call.assert_called_once_with('get', image_id)
         self.assertEqual(expected, image_meta)
 
+    def test_show_makes_datetimes(self):
+        image_id = uuidutils.generate_uuid()
+        image = self._make_datetime_fixture()
+        with mock.patch.object(self.service, 'call', return_value=image):
+            image_meta = self.service.show(image_id)
+            self.service.call.assert_called_once_with('get', image_id)
+        self.assertEqual(self.NOW_DATETIME, image_meta['created_at'])
+        self.assertEqual(self.NOW_DATETIME, image_meta['updated_at'])
+
     def test_show_raises_when_no_authtoken_in_the_context(self):
-        fixture = self._make_fixture(name='image1',
-                                     is_public=False,
-                                     properties={'one': 'two'})
-        image_id = self.service.create(fixture)['id']
         self.context.auth_token = False
         self.assertRaises(exception.ImageNotFound,
                           self.service.show,
-                          image_id)
-
-    def test_detail_passes_through_to_client(self):
-        fixture = self._make_fixture(name='image10', is_public=True)
-        image_id = self.service.create(fixture)['id']
-        image_metas = self.service.detail()
-        expected = [
-            {
-                'id': image_id,
-                'name': 'image10',
-                'is_public': True,
-                'size': None,
-                'min_disk': None,
-                'min_ram': None,
-                'disk_format': None,
-                'container_format': None,
-                'checksum': None,
-                'created_at': self.NOW_DATETIME,
-                'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
-                'deleted': None,
-                'status': None,
-                'properties': {},
-                'owner': None,
-            },
-        ]
-        self.assertEqual(expected, image_metas)
-
-    def test_show_makes_datetimes(self):
-        fixture = self._make_datetime_fixture()
-        image_id = self.service.create(fixture)['id']
-        image_meta = self.service.show(image_id)
-        self.assertEqual(self.NOW_DATETIME, image_meta['created_at'])
-        self.assertEqual(self.NOW_DATETIME, image_meta['updated_at'])
-
-    def test_detail_makes_datetimes(self):
-        fixture = self._make_datetime_fixture()
-        self.service.create(fixture)
-        image_meta = self.service.detail()[0]
-        self.assertEqual(self.NOW_DATETIME, image_meta['created_at'])
-        self.assertEqual(self.NOW_DATETIME, image_meta['updated_at'])
+                          uuidutils.generate_uuid())
 
     @mock.patch.object(time, 'sleep', autospec=True)
     def test_download_with_retries(self, mock_sleep):
@@ -744,7 +465,7 @@ class TestGlanceSwiftTempURL(base.TestCase):
 
     @mock.patch('swiftclient.utils.generate_temp_url', autospec=True)
     def test_swift_temp_url_radosgw(self, tempurl_mock):
-        self.config(temp_url_endpoint_type='radosgw', group='glance')
+        self.config(object_store_endpoint_type='radosgw', group='deploy')
         path = ('/v1'
                 '/glance'
                 '/757274c4-2856-4bd2-bb20-9a4a231e187b')
@@ -769,7 +490,7 @@ class TestGlanceSwiftTempURL(base.TestCase):
     def test_swift_temp_url_radosgw_endpoint_with_swift(self, tempurl_mock):
         self.config(swift_endpoint_url='https://swift.radosgw.com/swift',
                     group='glance')
-        self.config(temp_url_endpoint_type='radosgw', group='glance')
+        self.config(object_store_endpoint_type='radosgw', group='deploy')
         path = ('/v1'
                 '/glance'
                 '/757274c4-2856-4bd2-bb20-9a4a231e187b')
@@ -793,7 +514,7 @@ class TestGlanceSwiftTempURL(base.TestCase):
     def test_swift_temp_url_radosgw_endpoint_invalid(self, tempurl_mock):
         self.config(swift_endpoint_url='https://swift.radosgw.com/eggs/',
                     group='glance')
-        self.config(temp_url_endpoint_type='radosgw', group='glance')
+        self.config(object_store_endpoint_type='radosgw', group='deploy')
         self.service._validate_temp_url_config = mock.Mock()
 
         self.assertRaises(exception.InvalidParameterValue,
@@ -851,7 +572,7 @@ class TestGlanceSwiftTempURL(base.TestCase):
 
     def test__validate_temp_url_no_account_exception_radosgw(self):
         self.config(swift_account=None, group='glance')
-        self.config(temp_url_endpoint_type='radosgw', group='glance')
+        self.config(object_store_endpoint_type='radosgw', group='deploy')
         self.service._validate_temp_url_config()
 
     def test__validate_temp_url_endpoint_less_than_download_delay(self):
@@ -1086,24 +807,6 @@ class TestSwiftTempUrlCache(base.TestCase):
         self.assertNotIn(fake_image['id'], self.glance_service._cache)
 
 
-class TestGlanceUrl(base.TestCase):
-
-    def test_generate_glance_http_url(self):
-        self.config(glance_host="127.0.0.1", group='glance')
-        generated_url = service_utils.generate_glance_url()
-        http_url = "http://%s:%d" % (CONF.glance.glance_host,
-                                     CONF.glance.glance_port)
-        self.assertEqual(http_url, generated_url)
-
-    def test_generate_glance_https_url(self):
-        self.config(glance_protocol="https", group='glance')
-        self.config(glance_host="127.0.0.1", group='glance')
-        generated_url = service_utils.generate_glance_url()
-        https_url = "https://%s:%d" % (CONF.glance.glance_host,
-                                       CONF.glance.glance_port)
-        self.assertEqual(https_url, generated_url)
-
-
 class TestServiceUtils(base.TestCase):
 
     def test_parse_image_ref_no_ssl(self):
@@ -1111,24 +814,14 @@ class TestServiceUtils(base.TestCase):
             u'image_\u00F9\u00FA\u00EE\u0111'
         parsed_href = service_utils.parse_image_ref(image_href)
         self.assertEqual((u'image_\u00F9\u00FA\u00EE\u0111',
-                          '127.0.0.1', 9292, False), parsed_href)
+                          'http://127.0.0.1:9292', False), parsed_href)
 
     def test_parse_image_ref_ssl(self):
         image_href = 'https://127.0.0.1:9292/image_path/'\
                      u'image_\u00F9\u00FA\u00EE\u0111'
         parsed_href = service_utils.parse_image_ref(image_href)
         self.assertEqual((u'image_\u00F9\u00FA\u00EE\u0111',
-                          '127.0.0.1', 9292, True), parsed_href)
-
-    def test_generate_image_url(self):
-        image_href = u'image_\u00F9\u00FA\u00EE\u0111'
-        self.config(glance_host='123.123.123.123', group='glance')
-        self.config(glance_port=1234, group='glance')
-        self.config(glance_protocol='https', group='glance')
-        generated_url = service_utils.generate_image_url(image_href)
-        self.assertEqual('https://123.123.123.123:1234/images/'
-                         u'image_\u00F9\u00FA\u00EE\u0111',
-                         generated_url)
+                          'https://127.0.0.1:9292', True), parsed_href)
 
     def test_is_glance_image(self):
         image_href = u'uui\u0111'
@@ -1163,18 +856,37 @@ class TestGlanceAPIServers(base.TestCase):
         super(TestGlanceAPIServers, self).setUp()
         service_utils._GLANCE_API_SERVER = None
 
-    def test__get_api_servers_default(self):
-        host, port, use_ssl = service_utils._get_api_server()
-        self.assertEqual(CONF.glance.glance_host, host)
-        self.assertEqual(CONF.glance.glance_port, port)
-        self.assertEqual(CONF.glance.glance_protocol == 'https', use_ssl)
+    @mock.patch.object(service_utils.keystone, 'get_service_url',
+                       autospec=True)
+    @mock.patch.object(service_utils.keystone, 'get_session', autospec=True)
+    @mock.patch.object(service_utils.keystone, 'get_auth', autospec=True)
+    def test__get_api_servers_with_keystone(self, mock_auth, mock_session,
+                                            mock_service_url):
+        mock_service_url.return_value = 'https://example.com'
+
+        endpoint = service_utils._get_api_server()
+        self.assertEqual('https://example.com', endpoint)
+
+        mock_auth.assert_called_once_with('glance')
+        mock_session.assert_called_once_with('glance',
+                                             auth=mock_auth.return_value)
+        mock_service_url.assert_called_once_with(mock_session.return_value,
+                                                 service_type='image',
+                                                 endpoint_type='public')
+
+    def test__get_api_servers_with_host_port(self):
+        CONF.set_override('glance_host', 'example.com', 'glance')
+        CONF.set_override('glance_port', 42, 'glance')
+        CONF.set_override('glance_protocol', 'https', 'glance')
+        endpoint = service_utils._get_api_server()
+        self.assertEqual('https://example.com:42', endpoint)
 
     def test__get_api_servers_one(self):
         CONF.set_override('glance_api_servers', ['https://10.0.0.1:9293'],
                           'glance')
         s1 = service_utils._get_api_server()
         s2 = service_utils._get_api_server()
-        self.assertEqual(('10.0.0.1', 9293, True), s1)
+        self.assertEqual('https://10.0.0.1:9293', s1)
 
         # Only one server, should always get the same one
         self.assertEqual(s1, s2)
