@@ -19,7 +19,6 @@ from oslo_utils import uuidutils
 from ironic.common import exception
 from ironic.common import neutron
 from ironic.conductor import task_manager
-# from ironic.conf import auth as ironic_auth
 from ironic.tests import base
 from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
@@ -106,8 +105,7 @@ class TestNeutronClient(base.TestCase):
 
     def test_out_range_auth_strategy(self, mock_client_init, mock_session):
         self.assertRaises(ValueError, cfg.CONF.set_override,
-                          'auth_strategy', 'fake', 'neutron',
-                          enforce_type=True)
+                          'auth_strategy', 'fake', 'neutron')
 
 
 class TestNeutronNetworkActions(db_base.DbTestCase):
@@ -136,8 +134,8 @@ class TestNeutronNetworkActions(db_base.DbTestCase):
         patcher.start()
         self.addCleanup(patcher.stop)
 
-    def _test_add_ports_to_vlan_network(self, is_client_id,
-                                        security_groups=None):
+    def _test_add_ports_to_network(self, is_client_id,
+                                   security_groups=None):
         # Ports will be created only if pxe_enabled is True
         self.node.network_interface = 'neutron'
         self.node.save()
@@ -184,17 +182,17 @@ class TestNeutronNetworkActions(db_base.DbTestCase):
             self.client_mock.create_port.assert_called_once_with(
                 expected_body)
 
-    def test_add_ports_to_vlan_network(self):
-        self._test_add_ports_to_vlan_network(is_client_id=False,
-                                             security_groups=None)
+    def test_add_ports_to_network(self):
+        self._test_add_ports_to_network(is_client_id=False,
+                                        security_groups=None)
 
     @mock.patch.object(neutron, '_verify_security_groups')
-    def test_add_ports_to_vlan_network_with_sg(self, verify_mock):
+    def test_add_ports_to_network_with_sg(self, verify_mock):
         sg_ids = []
         for i in range(2):
             sg_ids.append(uuidutils.generate_uuid())
-        self._test_add_ports_to_vlan_network(is_client_id=False,
-                                             security_groups=sg_ids)
+        self._test_add_ports_to_network(is_client_id=False,
+                                        security_groups=sg_ids)
 
     def test_verify_sec_groups(self):
         sg_ids = []
@@ -270,57 +268,11 @@ class TestNeutronNetworkActions(db_base.DbTestCase):
             neutron._verify_security_groups, sg_ids, client)
         client.list_security_groups.assert_called_once_with()
 
-    def test_add_ports_with_client_id_to_vlan_network(self):
-        self._test_add_ports_to_vlan_network(is_client_id=True)
-
-    def _test_add_ports_to_flat_network(self, is_client_id):
-        self.node.network_interface = 'flat'
-        self.node.save()
-        port = self.ports[0]
-        if is_client_id:
-            extra = port.extra
-            extra['client-id'] = self._CLIENT_ID
-            port.extra = extra
-            port.save()
-        expected_body = {
-            'port': {
-                'network_id': self.network_uuid,
-                'admin_state_up': True,
-                'binding:vnic_type': 'baremetal',
-                'device_owner': 'baremetal:none',
-                'device_id': self.node.uuid,
-                'mac_address': port.address,
-                'binding:profile': {
-                    'local_link_information': [port.local_link_connection]
-                }
-            }
-        }
-        if is_client_id:
-            expected_body['port']['extra_dhcp_opts'] = (
-                [{'opt_name': 'client-id', 'opt_value': self._CLIENT_ID}])
-        # Ensure we can create ports
-        self.client_mock.create_port.return_value = {
-            'port': self.neutron_port}
-        expected = {port.uuid: self.neutron_port['id']}
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            ports = neutron.add_ports_to_network(task, self.network_uuid)
-            self.assertEqual(expected, ports)
-            self.client_mock.create_port.assert_called_once_with(
-                expected_body)
-
-    @mock.patch.object(neutron, 'validate_port_info', autospec=True,
-                       return_value=True)
-    def test_add_ports_to_flat_network(self, vpi_mock):
-        self._test_add_ports_to_flat_network(is_client_id=False)
-        self.assertTrue(vpi_mock.called)
-
-    @mock.patch.object(neutron, 'validate_port_info', autospec=True,
-                       return_value=True)
-    def test_add_ports_with_client_id_to_flat_network(self, vpi_mock):
-        self._test_add_ports_to_flat_network(is_client_id=True)
+    def test_add_ports_with_client_id_to_network(self):
+        self._test_add_ports_to_network(is_client_id=True)
 
     @mock.patch.object(neutron, 'validate_port_info', autospec=True)
-    def test_add_ports_to_vlan_network_instance_uuid(self, vpi_mock):
+    def test_add_ports_to_network_instance_uuid(self, vpi_mock):
         self.node.instance_uuid = uuidutils.generate_uuid()
         self.node.network_interface = 'neutron'
         self.node.save()
@@ -438,18 +390,6 @@ class TestNeutronNetworkActions(db_base.DbTestCase):
         self.client_mock.delete_port.assert_called_once_with(
             self.neutron_port['id'])
 
-    def test_remove_neutron_ports_delete_race(self):
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.client_mock.delete_port.side_effect = \
-                neutron_client_exc.PortNotFoundClient
-            self.client_mock.list_ports.return_value = {
-                'ports': [self.neutron_port]}
-            neutron.remove_neutron_ports(task, {'param': 'value'})
-        self.client_mock.list_ports.assert_called_once_with(
-            **{'param': 'value'})
-        self.client_mock.delete_port.assert_called_once_with(
-            self.neutron_port['id'])
-
     def test_get_node_portmap(self):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             portmap = neutron.get_node_portmap(task)
@@ -458,6 +398,25 @@ class TestNeutronNetworkActions(db_base.DbTestCase):
                 portmap
             )
 
+    def test_get_local_group_information(self):
+        pg = object_utils.create_test_portgroup(
+            self.context, node_id=self.node.id,
+            uuid=uuidutils.generate_uuid(),
+            address='52:54:55:cf:2d:32',
+            mode='802.3ad', properties={'bond_opt1': 'foo',
+                                        'opt2': 'bar'},
+            name='test-pg'
+        )
+        expected = {
+            'id': pg.uuid,
+            'name': pg.name,
+            'bond_mode': pg.mode,
+            'bond_properties': {'bond_opt1': 'foo', 'bond_opt2': 'bar'},
+        }
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            res = neutron.get_local_group_information(task, pg)
+        self.assertEqual(expected, res)
+
     @mock.patch.object(neutron, 'remove_ports_from_network')
     def test_rollback_ports(self, remove_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -716,3 +675,228 @@ class TestUnbindPort(base.TestCase):
                                                                      body)
         mock_log.info.assert_called_once_with('Port %s was not found while '
                                               'unbinding.', port_id)
+
+
+class TestGetNetworkByUUIDOrName(base.TestCase):
+
+    def setUp(self):
+        super(TestGetNetworkByUUIDOrName, self).setUp()
+        self.client = mock.MagicMock()
+
+    def test__get_network_by_uuid_or_name_uuid(self):
+        network_uuid = '9acb0256-2c1b-420a-b9d7-62bee90b6ed7'
+        networks = {
+            'networks': [{
+                'field1': 'value1',
+                'field2': 'value2',
+            }],
+        }
+        fields = ['field1', 'field2']
+        self.client.list_networks.return_value = networks
+        result = neutron._get_network_by_uuid_or_name(
+            self.client, network_uuid, fields=fields)
+        self.client.list_networks.assert_called_once_with(
+            id=network_uuid, fields=fields)
+        self.assertEqual(networks['networks'][0], result)
+
+    def test__get_network_by_uuid_or_name_name(self):
+        network_name = 'test-net'
+        networks = {
+            'networks': [{
+                'field1': 'value1',
+                'field2': 'value2',
+            }],
+        }
+        fields = ['field1', 'field2']
+        self.client.list_networks.return_value = networks
+        result = neutron._get_network_by_uuid_or_name(
+            self.client, network_name, fields=fields)
+        self.client.list_networks.assert_called_once_with(
+            name=network_name, fields=fields)
+        self.assertEqual(networks['networks'][0], result)
+
+    def test__get_network_by_uuid_or_name_failure(self):
+        network_uuid = '9acb0256-2c1b-420a-b9d7-62bee90b6ed7'
+        self.client.list_networks.side_effect = (
+            neutron_client_exc.NeutronClientException())
+        self.assertRaises(exception.NetworkError,
+                          neutron._get_network_by_uuid_or_name,
+                          self.client, network_uuid)
+        self.client.list_networks.assert_called_once_with(id=network_uuid)
+
+    def test__get_network_by_uuid_or_name_missing(self):
+        network_uuid = '9acb0256-2c1b-420a-b9d7-62bee90b6ed7'
+        networks = {
+            'networks': [],
+        }
+        self.client.list_networks.return_value = networks
+        self.assertRaises(exception.InvalidParameterValue,
+                          neutron._get_network_by_uuid_or_name,
+                          self.client, network_uuid)
+        self.client.list_networks.assert_called_once_with(id=network_uuid)
+
+    def test__get_network_by_uuid_or_name_duplicate(self):
+        network_name = 'test-net'
+        networks = {
+            'networks': [
+                {'id': '9acb0256-2c1b-420a-b9d7-62bee90b6ed7'},
+                {'id': '9014b6a7-8291-4676-80b0-ab00988ce3c7'},
+            ],
+        }
+        self.client.list_networks.return_value = networks
+        self.assertRaises(exception.InvalidParameterValue,
+                          neutron._get_network_by_uuid_or_name,
+                          self.client, network_name)
+        self.client.list_networks.assert_called_once_with(name=network_name)
+
+
+@mock.patch.object(neutron, '_get_network_by_uuid_or_name', autospec=True)
+@mock.patch.object(neutron, '_get_port_by_uuid', autospec=True)
+class TestGetPhysnetsByPortUUID(base.TestCase):
+
+    PORT_FIELDS = ['network_id']
+    NETWORK_FIELDS = ['provider:physical_network', 'segments']
+
+    def setUp(self):
+        super(TestGetPhysnetsByPortUUID, self).setUp()
+        self.client = mock.MagicMock()
+
+    def test_get_physnets_by_port_uuid_single_segment(self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        physnet = 'fake-physnet'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.return_value = {
+            'provider:physical_network': physnet,
+        }
+        result = neutron.get_physnets_by_port_uuid(self.client,
+                                                   port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
+        self.assertEqual({physnet}, result)
+
+    def test_get_physnets_by_port_uuid_single_segment_no_physnet(
+            self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.return_value = {
+            'provider:physical_network': None,
+        }
+        result = neutron.get_physnets_by_port_uuid(self.client,
+                                                   port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
+        self.assertEqual(set(), result)
+
+    def test_get_physnets_by_port_uuid_multiple_segments(self, mock_gp,
+                                                         mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        physnet1 = 'fake-physnet-1'
+        physnet2 = 'fake-physnet-2'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.return_value = {
+            'segments': [
+                {
+                    'provider:physical_network': physnet1,
+                },
+                {
+                    'provider:physical_network': physnet2,
+                },
+            ],
+        }
+        result = neutron.get_physnets_by_port_uuid(self.client,
+                                                   port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
+        self.assertEqual({physnet1, physnet2}, result)
+
+    def test_get_physnets_by_port_uuid_multiple_segments_no_physnet(
+            self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.return_value = {
+            'segments': [
+                {
+                    'provider:physical_network': None,
+                },
+                {
+                    'provider:physical_network': None,
+                },
+            ],
+        }
+        result = neutron.get_physnets_by_port_uuid(self.client,
+                                                   port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
+        self.assertEqual(set(), result)
+
+    def test_get_physnets_by_port_uuid_port_missing(self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        mock_gp.side_effect = exception.InvalidParameterValue('error')
+        self.assertRaises(exception.InvalidParameterValue,
+                          neutron.get_physnets_by_port_uuid,
+                          self.client, port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        self.assertFalse(mock_gn.called)
+
+    def test_get_physnets_by_port_uuid_port_failure(self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        mock_gp.side_effect = exception.NetworkError
+        self.assertRaises(exception.NetworkError,
+                          neutron.get_physnets_by_port_uuid,
+                          self.client, port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        self.assertFalse(mock_gn.called)
+
+    def test_get_physnets_by_port_uuid_network_missing(
+            self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.side_effect = exception.InvalidParameterValue('error')
+        self.assertRaises(exception.InvalidParameterValue,
+                          neutron.get_physnets_by_port_uuid,
+                          self.client, port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
+
+    def test_get_physnets_by_port_uuid_network_failure(
+            self, mock_gp, mock_gn):
+        port_uuid = 'fake-port-uuid'
+        network_uuid = 'fake-network-uuid'
+        mock_gp.return_value = {
+            'network_id': network_uuid,
+        }
+        mock_gn.side_effect = exception.NetworkError
+        self.assertRaises(exception.NetworkError,
+                          neutron.get_physnets_by_port_uuid,
+                          self.client, port_uuid)
+        mock_gp.assert_called_once_with(self.client, port_uuid,
+                                        fields=self.PORT_FIELDS)
+        mock_gn.assert_called_once_with(self.client, network_uuid,
+                                        fields=self.NETWORK_FIELDS)
