@@ -64,6 +64,16 @@ class TestPXEUtils(db_base.DbTestCase):
             'ipxe_timeout': 120
         })
 
+        self.ipxe_options_boot_from_volume = self.ipxe_options.copy()
+        self.ipxe_options_boot_from_volume.update({
+            'boot_from_volume': True,
+            'iscsi_boot_url': 'iscsi:fake_host::3260:0:fake_iqn',
+            'iscsi_initiator_iqn': 'fake_iqn',
+            'iscsi_volumes': ['iscsi:fake_host::3260:1:fake_iqn'],
+            'username': 'fake_username',
+            'password': 'fake_password'
+        })
+
         self.node = object_utils.create_test_node(self.context)
 
     def test_default_pxe_config(self):
@@ -93,7 +103,7 @@ class TestPXEUtils(db_base.DbTestCase):
         # NOTE(lucasagomes): iPXE is just an extension of the PXE driver,
         # it doesn't have it's own configuration option for template.
         # More info:
-        # http://docs.openstack.org/developer/ironic/deploy/install-guide.html
+        # https://docs.openstack.org/ironic/latest/install/
         self.config(
             pxe_config_template='ironic/drivers/modules/ipxe_config.template',
             group='pxe'
@@ -115,7 +125,7 @@ class TestPXEUtils(db_base.DbTestCase):
         # NOTE(lucasagomes): iPXE is just an extension of the PXE driver,
         # it doesn't have it's own configuration option for template.
         # More info:
-        # http://docs.openstack.org/developer/ironic/deploy/install-guide.html
+        # https://docs.openstack.org/ironic/latest/install/
         self.config(
             pxe_config_template='ironic/drivers/modules/ipxe_config.template',
             group='pxe'
@@ -133,6 +143,49 @@ class TestPXEUtils(db_base.DbTestCase):
 
         self.assertEqual(six.text_type(expected_template), rendered_template)
 
+    def test_default_ipxe_boot_from_volume_config(self):
+        self.config(
+            pxe_config_template='ironic/drivers/modules/ipxe_config.template',
+            group='pxe'
+        )
+        self.config(http_url='http://1.2.3.4:1234', group='deploy')
+        rendered_template = utils.render_template(
+            CONF.pxe.pxe_config_template,
+            {'pxe_options': self.ipxe_options_boot_from_volume,
+             'ROOT': '{{ ROOT }}',
+             'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'})
+
+        templ_file = 'ironic/tests/unit/drivers/' \
+                     'ipxe_config_boot_from_volume.template'
+        with open(templ_file) as f:
+            expected_template = f.read().rstrip()
+
+        self.assertEqual(six.text_type(expected_template), rendered_template)
+
+    def test_default_ipxe_boot_from_volume_config_no_volumes(self):
+        self.config(
+            pxe_config_template='ironic/drivers/modules/ipxe_config.template',
+            group='pxe'
+        )
+        self.config(http_url='http://1.2.3.4:1234', group='deploy')
+
+        pxe_options = self.ipxe_options_boot_from_volume
+        pxe_options['iscsi_volumes'] = []
+
+        rendered_template = utils.render_template(
+            CONF.pxe.pxe_config_template,
+            {'pxe_options': pxe_options,
+             'ROOT': '{{ ROOT }}',
+             'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'})
+
+        templ_file = 'ironic/tests/unit/drivers/' \
+                     'ipxe_config_boot_from_volume_no_volumes.template'
+        with open(templ_file) as f:
+            expected_template = f.read().rstrip()
+        self.assertEqual(six.text_type(expected_template), rendered_template)
+
+    # NOTE(TheJulia): Remove elilo support after the deprecation period,
+    # in the Queens release.
     def test_default_elilo_config(self):
         pxe_opts = self.pxe_options
         pxe_opts['boot_mode'] = 'uefi'
@@ -277,11 +330,12 @@ class TestPXEUtils(db_base.DbTestCase):
         unlink_mock.assert_called_once_with('/tftpboot/10.10.0.1.conf')
         create_link_mock.assert_has_calls(create_link_calls)
 
+    @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch('ironic.common.utils.write_to_file', autospec=True)
     @mock.patch('ironic.common.utils.render_template', autospec=True)
     @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
     def test_create_pxe_config(self, ensure_tree_mock, render_mock,
-                               write_mock):
+                               write_mock, chmod_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             pxe_utils.create_pxe_config(task, self.pxe_options,
                                         CONF.pxe.pxe_config_template)
@@ -291,23 +345,86 @@ class TestPXEUtils(db_base.DbTestCase):
                  'ROOT': '{{ ROOT }}',
                  'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'}
             )
+        node_dir = os.path.join(CONF.pxe.tftp_root, self.node.uuid)
+        pxe_dir = os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg')
         ensure_calls = [
-            mock.call(os.path.join(CONF.pxe.tftp_root, self.node.uuid)),
-            mock.call(os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg'))
+            mock.call(node_dir), mock.call(pxe_dir),
         ]
         ensure_tree_mock.assert_has_calls(ensure_calls)
+        chmod_mock.assert_not_called()
 
         pxe_cfg_file_path = pxe_utils.get_pxe_config_file_path(self.node.uuid)
         write_mock.assert_called_with(pxe_cfg_file_path,
                                       render_mock.return_value)
 
+    @mock.patch.object(os, 'chmod', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
+    def test_create_pxe_config_set_dir_permission(self, ensure_tree_mock,
+                                                  render_mock,
+                                                  write_mock, chmod_mock):
+        self.config(dir_permission=0o755, group='pxe')
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            pxe_utils.create_pxe_config(task, self.pxe_options,
+                                        CONF.pxe.pxe_config_template)
+            render_mock.assert_called_with(
+                CONF.pxe.pxe_config_template,
+                {'pxe_options': self.pxe_options,
+                 'ROOT': '{{ ROOT }}',
+                 'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'}
+            )
+        node_dir = os.path.join(CONF.pxe.tftp_root, self.node.uuid)
+        pxe_dir = os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg')
+        ensure_calls = [
+            mock.call(node_dir), mock.call(pxe_dir),
+        ]
+        ensure_tree_mock.assert_has_calls(ensure_calls)
+        chmod_calls = [mock.call(node_dir, 0o755), mock.call(pxe_dir, 0o755)]
+        chmod_mock.assert_has_calls(chmod_calls)
+
+        pxe_cfg_file_path = pxe_utils.get_pxe_config_file_path(self.node.uuid)
+        write_mock.assert_called_with(pxe_cfg_file_path,
+                                      render_mock.return_value)
+
+    @mock.patch.object(os.path, 'isdir', autospec=True)
+    @mock.patch.object(os, 'chmod', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
+    def test_create_pxe_config_existing_dirs(self, ensure_tree_mock,
+                                             render_mock,
+                                             write_mock, chmod_mock,
+                                             isdir_mock):
+        self.config(dir_permission=0o755, group='pxe')
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            isdir_mock.return_value = True
+            pxe_utils.create_pxe_config(task, self.pxe_options,
+                                        CONF.pxe.pxe_config_template)
+            render_mock.assert_called_with(
+                CONF.pxe.pxe_config_template,
+                {'pxe_options': self.pxe_options,
+                 'ROOT': '{{ ROOT }}',
+                 'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'}
+            )
+        ensure_tree_mock.assert_has_calls([])
+        chmod_mock.assert_not_called()
+        isdir_mock.assert_has_calls([])
+        pxe_cfg_file_path = pxe_utils.get_pxe_config_file_path(self.node.uuid)
+        write_mock.assert_called_with(pxe_cfg_file_path,
+                                      render_mock.return_value)
+
+    # NOTE(TheJulia): Remove elilo support after the deprecation period,
+    # in the Queens release.
+    @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch('ironic.common.pxe_utils._link_ip_address_pxe_configs',
                 autospec=True)
     @mock.patch('ironic.common.utils.write_to_file', autospec=True)
     @mock.patch('ironic.common.utils.render_template', autospec=True)
     @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
     def test_create_pxe_config_uefi_elilo(self, ensure_tree_mock, render_mock,
-                                          write_mock, link_ip_configs_mock):
+                                          write_mock, link_ip_configs_mock,
+                                          chmod_mock):
         self.config(
             uefi_pxe_config_template=('ironic/drivers/modules/'
                                       'elilo_efi_pxe_config.template'),
@@ -320,9 +437,10 @@ class TestPXEUtils(db_base.DbTestCase):
 
             ensure_calls = [
                 mock.call(os.path.join(CONF.pxe.tftp_root, self.node.uuid)),
-                mock.call(os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg'))
+                mock.call(os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg')),
             ]
             ensure_tree_mock.assert_has_calls(ensure_calls)
+            chmod_mock.assert_not_called()
             render_mock.assert_called_with(
                 CONF.pxe.uefi_pxe_config_template,
                 {'pxe_options': self.pxe_options,
@@ -334,13 +452,15 @@ class TestPXEUtils(db_base.DbTestCase):
         write_mock.assert_called_with(pxe_cfg_file_path,
                                       render_mock.return_value)
 
+    @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch('ironic.common.pxe_utils._link_ip_address_pxe_configs',
                 autospec=True)
     @mock.patch('ironic.common.utils.write_to_file', autospec=True)
     @mock.patch('ironic.common.utils.render_template', autospec=True)
     @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
     def test_create_pxe_config_uefi_grub(self, ensure_tree_mock, render_mock,
-                                         write_mock, link_ip_configs_mock):
+                                         write_mock, link_ip_configs_mock,
+                                         chmod_mock):
         grub_tmplte = "ironic/drivers/modules/pxe_grub_config.template"
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.properties['capabilities'] = 'boot_mode:uefi'
@@ -349,9 +469,10 @@ class TestPXEUtils(db_base.DbTestCase):
 
             ensure_calls = [
                 mock.call(os.path.join(CONF.pxe.tftp_root, self.node.uuid)),
-                mock.call(os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg'))
+                mock.call(os.path.join(CONF.pxe.tftp_root, 'pxelinux.cfg')),
             ]
             ensure_tree_mock.assert_has_calls(ensure_calls)
+            chmod_mock.assert_not_called()
             render_mock.assert_called_with(
                 grub_tmplte,
                 {'pxe_options': self.pxe_options,
@@ -363,13 +484,15 @@ class TestPXEUtils(db_base.DbTestCase):
         write_mock.assert_called_with(pxe_cfg_file_path,
                                       render_mock.return_value)
 
+    @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch('ironic.common.pxe_utils._link_mac_pxe_configs',
                 autospec=True)
     @mock.patch('ironic.common.utils.write_to_file', autospec=True)
     @mock.patch('ironic.common.utils.render_template', autospec=True)
     @mock.patch('oslo_utils.fileutils.ensure_tree', autospec=True)
     def test_create_pxe_config_uefi_ipxe(self, ensure_tree_mock, render_mock,
-                                         write_mock, link_mac_pxe_mock):
+                                         write_mock, link_mac_pxe_mock,
+                                         chmod_mock):
         self.config(ipxe_enabled=True, group='pxe')
         ipxe_template = "ironic/drivers/modules/ipxe_config.template"
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -379,9 +502,10 @@ class TestPXEUtils(db_base.DbTestCase):
 
             ensure_calls = [
                 mock.call(os.path.join(CONF.deploy.http_root, self.node.uuid)),
-                mock.call(os.path.join(CONF.deploy.http_root, 'pxelinux.cfg'))
+                mock.call(os.path.join(CONF.deploy.http_root, 'pxelinux.cfg')),
             ]
             ensure_tree_mock.assert_has_calls(ensure_calls)
+            chmod_mock.assert_not_called()
             render_mock.assert_called_with(
                 ipxe_template,
                 {'pxe_options': self.ipxe_options,
@@ -408,6 +532,55 @@ class TestPXEUtils(db_base.DbTestCase):
         rmtree_mock.assert_called_once_with(
             os.path.join(CONF.pxe.tftp_root, self.node.uuid))
 
+    @mock.patch.object(os.path, 'isfile', lambda path: False)
+    @mock.patch('ironic.common.utils.file_has_content', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    def test_create_ipxe_boot_script(self, render_mock, write_mock,
+                                     file_has_content_mock):
+        render_mock.return_value = 'foo'
+        pxe_utils.create_ipxe_boot_script()
+        self.assertFalse(file_has_content_mock.called)
+        write_mock.assert_called_once_with(
+            os.path.join(CONF.deploy.http_root,
+                         os.path.basename(CONF.pxe.ipxe_boot_script)),
+            'foo')
+        render_mock.assert_called_once_with(
+            CONF.pxe.ipxe_boot_script,
+            {'ipxe_for_mac_uri': 'pxelinux.cfg/'})
+
+    @mock.patch.object(os.path, 'isfile', lambda path: True)
+    @mock.patch('ironic.common.utils.file_has_content', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    def test_create_ipxe_boot_script_copy_file_different(
+            self, render_mock, write_mock, file_has_content_mock):
+        file_has_content_mock.return_value = False
+        render_mock.return_value = 'foo'
+        pxe_utils.create_ipxe_boot_script()
+        file_has_content_mock.assert_called_once_with(
+            os.path.join(CONF.deploy.http_root,
+                         os.path.basename(CONF.pxe.ipxe_boot_script)),
+            'foo')
+        write_mock.assert_called_once_with(
+            os.path.join(CONF.deploy.http_root,
+                         os.path.basename(CONF.pxe.ipxe_boot_script)),
+            'foo')
+        render_mock.assert_called_once_with(
+            CONF.pxe.ipxe_boot_script,
+            {'ipxe_for_mac_uri': 'pxelinux.cfg/'})
+
+    @mock.patch.object(os.path, 'isfile', lambda path: True)
+    @mock.patch('ironic.common.utils.file_has_content', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    def test_create_ipxe_boot_script_already_exists(self, render_mock,
+                                                    write_mock,
+                                                    file_has_content_mock):
+        file_has_content_mock.return_value = True
+        pxe_utils.create_ipxe_boot_script()
+        self.assertFalse(write_mock.called)
+
     def test__get_pxe_mac_path(self):
         mac = '00:11:22:33:44:55:66'
         self.assertEqual('/tftpboot/pxelinux.cfg/01-00-11-22-33-44-55-66',
