@@ -34,7 +34,7 @@ from ironic import objects
 from ironic.objects import fields
 from ironic.tests import base as tests_base
 from ironic.tests.unit.conductor import mgr_utils
-from ironic.tests.unit.db import base as tests_db_base
+from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as obj_utils
 
 
@@ -42,7 +42,7 @@ CONF = cfg.CONF
 
 
 @mgr_utils.mock_record_keepalive
-class StartStopTestCase(mgr_utils.ServiceSetUpMixin, tests_db_base.DbTestCase):
+class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test_start_registers_conductor(self):
         self.assertRaises(exception.ConductorNotFound,
                           objects.Conductor.get_by_hostname,
@@ -250,9 +250,36 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, tests_db_base.DbTestCase):
         self.service.del_host()
         self.assertTrue(wait_mock.called)
 
+    def test_start_fails_on_missing_config_for_configdrive(self):
+        """Check to fail conductor on missing config options"""
 
-class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
-                              tests_db_base.DbTestCase):
+        missing_parameters_error = ("Parameters missing to make a "
+                                    "connection with radosgw")
+        CONF.set_override('configdrive_use_object_store', True,
+                          group='deploy')
+        CONF.set_override('object_store_endpoint_type', 'radosgw',
+                          group='deploy')
+        params = {'auth_url': 'http://1.2.3.4',
+                  'username': 'foo', 'password': 'foo_pass'}
+        CONF.register_opts((cfg.StrOpt(x) for x in params),
+                           group='swift')
+        for key, value in params.items():
+            test_params = params.copy()
+            test_params[key] = None
+            for test_key, test_value in test_params.items():
+                CONF.set_override(key, test_value, group='swift')
+            with self.assertRaisesRegex(exception.ConfigInvalid,
+                                        missing_parameters_error):
+                self._start_service()
+
+    def test_conductor_shutdown_flag(self):
+        self._start_service()
+        self.assertFalse(self.service._shutdown)
+        self.service.del_host()
+        self.assertTrue(self.service._shutdown)
+
+
+class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__check_enabled_interfaces_success(self):
         base_manager._check_enabled_interfaces()
 
@@ -268,7 +295,7 @@ class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         base_manager._check_enabled_interfaces()
 
 
-class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, tests_db_base.DbTestCase):
+class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__conductor_service_record_keepalive(self):
         self._start_service()
         # avoid wasting time at the event.wait()
@@ -323,7 +350,7 @@ class ManagerSpawnWorkerTestCase(tests_base.TestCase):
                    autospec=True)
 @mgr_utils.mock_record_keepalive
 class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
-                                 tests_db_base.DbTestCase):
+                                 db_base.DbTestCase):
     def setUp(self):
         super(RegisterInterfacesTestCase, self).setUp()
         self._start_service()
@@ -395,8 +422,7 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertFalse(reg_mock.called)
 
 
-class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin,
-                            tests_db_base.DbTestCase):
+class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     @mock.patch.object(notification_utils, 'emit_console_notification')
     def test__start_consoles(self, mock_notify):
         obj_utils.create_test_node(self.context,
