@@ -50,7 +50,6 @@ from oslo_utils import uuidutils
 import sqlalchemy
 import sqlalchemy.exc
 
-from ironic.common.i18n import _LE
 from ironic.conf import CONF
 from ironic.db.sqlalchemy import migration
 from ironic.db.sqlalchemy import models
@@ -83,7 +82,7 @@ def _get_connect_string(backend, user, passwd, database):
 
 @contextlib.contextmanager
 def patch_with_engine(engine):
-    with mock.patch.object(enginefacade.get_legacy_facade(),
+    with mock.patch.object(enginefacade.writer,
                            'get_engine') as patch_engine:
         patch_engine.return_value = engine
         yield
@@ -133,8 +132,8 @@ class WalkVersionsMixin(object):
                 if check:
                     check(engine, data)
         except Exception:
-            LOG.error(_LE("Failed to migrate to version %(version)s on engine "
-                          "%(engine)s"),
+            LOG.error("Failed to migrate to version %(version)s on engine "
+                      "%(engine)s",
                       {'version': version, 'engine': engine})
             raise
 
@@ -635,6 +634,23 @@ class MigrationCheckersMixin(object):
                               (sqlalchemy.types.Boolean,
                                sqlalchemy.types.Integer))
 
+    def _check_3d86a077a3f2(self, engine, data):
+        ports = db_utils.get_table(engine, 'ports')
+        col_names = [column.name for column in ports.c]
+        self.assertIn('physical_network', col_names)
+        self.assertIsInstance(ports.c.physical_network.type,
+                              sqlalchemy.types.String)
+
+    def _check_868cb606a74a(self, engine, data):
+        for table in ['chassis', 'conductors', 'node_tags', 'nodes',
+                      'portgroups', 'ports', 'volume_connectors',
+                      'volume_targets', 'conductor_hardware_interfaces']:
+            table = db_utils.get_table(engine, table)
+            col_names = [column.name for column in table.c]
+            self.assertIn('version', col_names)
+            self.assertIsInstance(table.c.version.type,
+                                  sqlalchemy.types.String)
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('head')
