@@ -15,7 +15,6 @@
 """Test class for Management Interface used by iLO modules."""
 
 import mock
-from oslo_config import cfg
 from oslo_utils import importutils
 
 from ironic.common import boot_devices
@@ -32,7 +31,6 @@ from ironic.tests.unit.objects import utils as obj_utils
 ilo_error = importutils.try_import('proliantutils.exception')
 
 INFO_DICT = db_utils.get_test_ilo_info()
-CONF = cfg.CONF
 
 
 class IloManagementTestCase(db_base.DbTestCase):
@@ -254,10 +252,9 @@ class IloManagementTestCase(db_base.DbTestCase):
             task.driver.management.reset_ilo_credential(task)
             clean_step_mock.assert_called_once_with(
                 task.node, 'reset_ilo_credential', 'fake-password')
-            self.assertIsNone(
-                task.node.driver_info.get('ilo_change_password'))
-            self.assertEqual(task.node.driver_info['ilo_password'],
-                             'fake-password')
+            self.assertNotIn('ilo_change_password', task.node.driver_info)
+            self.assertEqual('fake-password',
+                             task.node.driver_info['ilo_password'])
 
     @mock.patch.object(ilo_management, 'LOG', spec_set=True, autospec=True)
     @mock.patch.object(ilo_management, '_execute_ilo_clean_step',
@@ -309,11 +306,10 @@ class IloManagementTestCase(db_base.DbTestCase):
             clean_step_mock.assert_called_once_with(
                 task.node, 'activate_license', 'XXXXX-YYYYY-ZZZZZ-XYZZZ-XXYYZ')
 
-    @mock.patch.object(ilo_management, 'LOG', spec_set=True, autospec=True)
     @mock.patch.object(ilo_management, '_execute_ilo_clean_step',
                        spec_set=True, autospec=True)
     def test_activate_license_no_or_invalid_format_license_key(
-            self, clean_step_mock, log_mock):
+            self, clean_step_mock):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             for license_key_value in (None, [], {}):
@@ -394,7 +390,7 @@ class IloManagementTestCase(db_base.DbTestCase):
                                'fw_location_for_another_filepath', 'chassis'),
                      ]
             clean_step_mock.assert_has_calls(calls)
-            self.assertTrue(clean_step_mock.call_count == 5)
+            self.assertEqual(5, clean_step_mock.call_count)
 
     def test_update_firmware_throws_if_invalid_update_mode_provided(self):
         with task_manager.acquire(self.context, self.node.uuid,
