@@ -65,7 +65,6 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_info = task.node.driver_info
-            driver_info['dynamic_allocation'] = True
             task.node.driver_info = driver_info
             task.node.provision_state = states.DEPLOYING
             self.assertRaises(
@@ -111,12 +110,12 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
                 '/rest/server-profiles/1234556789'
             task.node.driver_info = driver_info
 
-            self.assertTrue(
-                'applied_server_profile_uri' in task.node.driver_info
+            self.assertIn(
+                'applied_server_profile_uri', task.node.driver_info
             )
             deploy_utils.tear_down(ov_client, task)
-            self.assertFalse(
-                'applied_server_profile_uri' in task.node.driver_info
+            self.assertNotIn(
+                'applied_server_profile_uri', task.node.driver_info
             )
         self.assertTrue(
             ov_client.delete_server_profile.called
@@ -174,7 +173,6 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_info = task.node.driver_info
-            driver_info['dynamic_allocation'] = True
             task.node.driver_info = driver_info
             task.node.provision_state = states.DEPLOYING
             self.assertRaises(
@@ -220,7 +218,6 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_info = task.node.driver_info
-            driver_info['dynamic_allocation'] = True
             task.node.driver_info = driver_info
             self.assertTrue(
                 deploy_utils.is_node_in_use_by_oneview(oneview_client,
@@ -278,7 +275,6 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_info = task.node.driver_info
-            driver_info['dynamic_allocation'] = True
             driver_info['applied_server_profile_uri'] = 'same/applied_sp_uri/'
             task.node.driver_info = driver_info
             self.assertTrue(
@@ -407,8 +403,8 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
                 oneview_client, task.node
             )
             self.assertTrue(oneview_client.delete_server_profile.called)
-            self.assertTrue(
-                'applied_server_profile_uri' not in task.node.driver_info
+            self.assertNotIn(
+                'applied_server_profile_uri', task.node.driver_info
             )
 
     @mock.patch.object(objects.Node, 'save')
@@ -443,6 +439,6 @@ class OneViewDeployUtilsTestCase(db_base.DbTestCase):
                 task.node
             )
             self.assertTrue(ov_client.delete_server_profile.called)
-            self.assertTrue(
-                'applied_server_profile_uri' in task.node.driver_info
+            self.assertIn(
+                'applied_server_profile_uri', task.node.driver_info
             )
