@@ -29,7 +29,9 @@ from ironic.drivers.modules import agent_base_vendor
 from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import fake
+from ironic.drivers.modules.network import flat as flat_network
 from ironic.drivers.modules import pxe
+from ironic.drivers.modules.storage import noop as noop_storage
 from ironic.drivers import utils as driver_utils
 from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
@@ -51,17 +53,6 @@ class TestAgentMethods(db_base.DbTestCase):
                                                   driver='fake_agent')
         dhcp_factory.DHCPFactory._dhcp_provider = None
 
-    @mock.patch.object(agent.LOG, 'warning', autospec=True)
-    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy',
-                       autospec=True)
-    def test_build_instance_info_for_deploy_warn(self, build_mock, warn_mock):
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
-        with task_manager.acquire(
-                self.context, self.node.uuid, shared=False) as task:
-            agent.build_instance_info_for_deploy(task)
-            build_mock.assert_called_once_with(task)
-        self.assertTrue(warn_mock.called)
-
     @mock.patch.object(images, 'image_show', autospec=True)
     def test_check_image_size(self, show_mock):
         show_mock.return_value = {
@@ -155,11 +146,16 @@ class TestAgentDeploy(db_base.DbTestCase):
         super(TestAgentDeploy, self).setUp()
         mgr_utils.mock_the_extension_manager(driver='fake_agent')
         self.driver = agent.AgentDeploy()
+        # NOTE(TheJulia): We explicitly set the noop storage interface as the
+        # default below for deployment tests in order to raise any change
+        # in the default which could be a breaking behavior change
+        # as the storage interface is explicitly an "opt-in" interface.
         n = {
             'driver': 'fake_agent',
             'instance_info': INSTANCE_INFO,
             'driver_info': DRIVER_INFO,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
+            'storage_interface': 'noop',
         }
         self.node = object_utils.create_test_node(self.context, **n)
         self.ports = [
@@ -261,18 +257,59 @@ class TestAgentDeploy(db_base.DbTestCase):
                 task.driver.boot, task)
             show_mock.assert_called_once_with(self.context, 'fake-image')
 
+    @mock.patch.object(pxe.PXEBoot, 'validate', autospec=True)
+    @mock.patch.object(deploy_utils, 'check_for_missing_params',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'validate_capabilities', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    def test_validate_storage_should_write_image_false(self, mock_write,
+                                                       mock_capabilities,
+                                                       mock_params,
+                                                       mock_pxe_validate):
+        mock_write.return_value = False
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            self.driver.validate(task)
+            mock_capabilities.assert_called_once_with(task.node)
+            self.assertFalse(mock_params.called)
+
+    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     @mock.patch('ironic.conductor.utils.node_power_action', autospec=True)
-    def test_deploy(self, power_mock):
+    def test_deploy(self, power_mock, mock_pxe_instance):
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             driver_return = self.driver.deploy(task)
             self.assertEqual(driver_return, states.DEPLOYWAIT)
             power_mock.assert_called_once_with(task, states.REBOOT)
+            self.assertFalse(mock_pxe_instance.called)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'unconfigure_tenant_networks', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    def test_deploy_storage_should_write_image_false(self, mock_write,
+                                                     mock_pxe_instance):
+        mock_write.return_value = False
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            driver_return = self.driver.deploy(task)
+            self.assertEqual(driver_return, states.DEPLOYDONE)
+            self.assertTrue(mock_pxe_instance.called)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'detach_volumes',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
     @mock.patch('ironic.conductor.utils.node_power_action', autospec=True)
-    def test_tear_down(self, power_mock, unconfigure_tenant_nets_mock):
+    def test_tear_down(self, power_mock,
+                       unconfigure_tenant_nets_mock,
+                       storage_detach_volumes_mock):
+        object_utils.create_test_volume_target(
+            self.context, node_id=self.node.id)
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             driver_return = self.driver.tear_down(task)
@@ -280,18 +317,29 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertEqual(driver_return, states.DELETED)
             unconfigure_tenant_nets_mock.assert_called_once_with(mock.ANY,
                                                                  task)
+            storage_detach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+        # Verify no volumes exist for new task instances.
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            self.assertEqual(0, len(task.volume_targets))
 
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
     @mock.patch.object(deploy_utils, 'build_agent_options')
     @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', autospec=True)
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'unconfigure_tenant_networks', spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
     def test_prepare(
             self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
             build_instance_info_mock, build_options_mock,
-            pxe_prepare_ramdisk_mock):
+            pxe_prepare_ramdisk_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             task.node.provision_state = states.DEPLOYING
@@ -306,6 +354,9 @@ class TestAgentDeploy(db_base.DbTestCase):
                 task, {'a': 'b'})
             add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
             unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
 
         self.node.refresh()
         self.assertEqual('bar', self.node.instance_info['foo'])
@@ -331,8 +382,11 @@ class TestAgentDeploy(db_base.DbTestCase):
         self.node.refresh()
         self.assertEqual('bar', self.node.instance_info['foo'])
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info')
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance')
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
     @mock.patch.object(deploy_utils, 'build_agent_options')
@@ -340,7 +394,8 @@ class TestAgentDeploy(db_base.DbTestCase):
     def test_prepare_active(
             self, build_instance_info_mock, build_options_mock,
             pxe_prepare_ramdisk_mock, pxe_prepare_instance_mock,
-            add_provisioning_net_mock):
+            add_provisioning_net_mock, storage_driver_info_mock,
+            storage_attach_volumes_mock):
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             task.node.provision_state = states.ACTIVE
@@ -352,9 +407,49 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertFalse(pxe_prepare_ramdisk_mock.called)
             self.assertTrue(pxe_prepare_instance_mock.called)
             self.assertFalse(add_provisioning_net_mock.called)
+            self.assertTrue(storage_driver_info_mock.called)
+            self.assertFalse(storage_attach_volumes_mock.called)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy',
+                       autospec=True)
+    def test_prepare_storage_write_false(
+            self, build_instance_info_mock, build_options_mock,
+            pxe_prepare_ramdisk_mock, pxe_prepare_instance_mock,
+            remove_tenant_net_mock, add_provisioning_net_mock,
+            storage_driver_info_mock, storage_attach_volumes_mock,
+            should_write_image_mock):
+        should_write_image_mock.return_value = False
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+
+            self.driver.prepare(task)
+
+            self.assertFalse(build_instance_info_mock.called)
+            self.assertFalse(build_options_mock.called)
+            self.assertFalse(pxe_prepare_ramdisk_mock.called)
+            self.assertFalse(pxe_prepare_instance_mock.called)
+            self.assertFalse(add_provisioning_net_mock.called)
+            self.assertTrue(storage_driver_info_mock.called)
+            self.assertTrue(storage_attach_volumes_mock.called)
+            self.assertEqual(2, should_write_image_mock.call_count)
+
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
     @mock.patch.object(deploy_utils, 'build_agent_options')
     @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
@@ -372,6 +467,30 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertFalse(pxe_prepare_ramdisk_mock.called)
             self.assertFalse(add_provisioning_net_mock.called)
 
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(deploy_utils, 'build_agent_options')
+    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy')
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    def test_prepare_boot_from_volume(self, mock_write,
+                                      build_instance_info_mock,
+                                      build_options_mock,
+                                      pxe_prepare_ramdisk_mock,
+                                      add_provisioning_net_mock):
+        mock_write.return_value = False
+        with task_manager.acquire(
+                self.context, self.node['uuid'], shared=False) as task:
+            task.node.provision_state = states.DEPLOYING
+            build_instance_info_mock.return_value = {'foo': 'bar'}
+            build_options_mock.return_value = {'a': 'b'}
+
+            self.driver.prepare(task)
+            build_instance_info_mock.assert_not_called()
+            build_options_mock.assert_not_called()
+            pxe_prepare_ramdisk_mock.assert_not_called()
+
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory._set_dhcp_provider')
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory.clean_dhcp')
     @mock.patch.object(pxe.PXEBoot, 'clean_up_instance')
@@ -474,12 +593,6 @@ class TestAgentDeploy(db_base.DbTestCase):
             tear_down_cleaning_mock.assert_called_once_with(
                 task, manage_boot=False)
 
-    def test_heartbeat(self):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            self.driver.heartbeat(task, 'url')
-            self.assertFalse(task.shared)
-
     def _test_continue_deploy(self, additional_driver_info=None,
                               additional_expected_image_info=None):
         self.node.provision_state = states.DEPLOYWAIT
@@ -670,7 +783,9 @@ class TestAgentDeploy(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             get_power_state_mock.return_value = states.POWER_OFF
-            task.node.driver_internal_info['is_whole_disk_image'] = False
+            driver_internal_info = task.node.driver_internal_info
+            driver_internal_info['is_whole_disk_image'] = False
+            task.node.driver_internal_info = driver_internal_info
 
             task.driver.deploy.reboot_to_instance(task)
 
@@ -715,7 +830,9 @@ class TestAgentDeploy(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             get_power_state_mock.return_value = states.POWER_OFF
-            task.node.driver_internal_info['is_whole_disk_image'] = True
+            driver_internal_info = task.node.driver_internal_info
+            driver_internal_info['is_whole_disk_image'] = True
+            task.node.driver_internal_info = driver_internal_info
             task.driver.boot = None
 
             task.driver.deploy.reboot_to_instance(task)
@@ -803,7 +920,9 @@ class TestAgentDeploy(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             get_power_state_mock.return_value = states.POWER_OFF
-            task.node.driver_internal_info['is_whole_disk_image'] = False
+            driver_internal_info = task.node.driver_internal_info
+            driver_internal_info['is_whole_disk_image'] = False
+            task.node.driver_internal_info = driver_internal_info
             boot_option = {'capabilities': '{"boot_option": "local"}'}
             task.node.instance_info = boot_option
             task.driver.deploy.reboot_to_instance(task)
