@@ -38,6 +38,8 @@ def touch(filename):
     open(filename, 'w').close()
 
 
+@mock.patch('ironic.common.glance_service.service_utils.parse_image_ref',
+            lambda image: (image, 'example.com', True))
 class TestImageCacheFetch(base.TestCase):
 
     def setUp(self):
