@@ -36,7 +36,9 @@ from ironic.drivers.modules import agent_base_vendor
 from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import iscsi_deploy
+from ironic.drivers.modules.network import flat as flat_network
 from ironic.drivers.modules import pxe
+from ironic.drivers.modules.storage import noop as noop_storage
 from ironic.drivers import utils as driver_utils
 from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
@@ -533,11 +535,16 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
         super(ISCSIDeployTestCase, self).setUp()
         mgr_utils.mock_the_extension_manager(driver="fake_pxe")
         self.driver = driver_factory.get_driver("fake_pxe")
+        # NOTE(TheJulia): We explicitly set the noop storage interface as the
+        # default below for deployment tests in order to raise any change
+        # in the default which could be a breaking behavior change
+        # as the storage interface is explicitly an "opt-in" interface.
         self.node = obj_utils.create_test_node(
             self.context, driver='fake_pxe',
             instance_info=INST_INFO_DICT,
             driver_info=DRV_INFO_DICT,
             driver_internal_info=DRV_INTERNAL_INFO_DICT,
+            storage_interface='noop',
         )
         self.node.driver_internal_info['agent_url'] = 'http://1.2.3.4:1234'
         dhcp_factory.DHCPFactory._dhcp_provider = None
@@ -562,11 +569,38 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             validate_capabilities_mock.assert_called_once_with(task.node)
             validate_mock.assert_called_once_with(task)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(iscsi_deploy, 'validate', autospec=True)
+    @mock.patch.object(deploy_utils, 'validate_capabilities', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'validate', autospec=True)
+    def test_validate_storage_should_write_image_false(
+            self, pxe_validate_mock,
+            validate_capabilities_mock, validate_mock,
+            should_write_image_mock):
+        should_write_image_mock.return_value = False
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            task.driver.deploy.validate(task)
+
+            pxe_validate_mock.assert_called_once_with(task.driver.boot, task)
+            validate_capabilities_mock.assert_called_once_with(task.node)
+            self.assertFalse(validate_mock.called)
+            should_write_image_mock.assert_called_once_with(
+                task.driver.storage, task)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_node_active(self, prepare_instance_mock,
-                                 add_provisioning_net_mock):
+                                 add_provisioning_net_mock,
+                                 storage_driver_info_mock,
+                                 storage_attach_volumes_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.provision_state = states.ACTIVE
 
@@ -575,9 +609,11 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             prepare_instance_mock.assert_called_once_with(
                 task.driver.boot, task)
             self.assertEqual(0, add_provisioning_net_mock.call_count)
+            storage_driver_info_mock.assert_called_once_with(task)
+        self.assertFalse(storage_attach_volumes_mock.called)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_node_adopting(self, prepare_instance_mock,
                                    add_provisioning_net_mock):
@@ -590,15 +626,21 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                 task.driver.boot, task)
             self.assertEqual(0, add_provisioning_net_mock.call_count)
 
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'add_provisioning_network', spec_set=True, autospec=True)
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'unconfigure_tenant_networks', spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
     def test_prepare_node_deploying(
             self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
-            mock_prepare_ramdisk, mock_agent_options):
+            mock_prepare_ramdisk, mock_agent_options,
+            storage_driver_info_mock, storage_attach_volumes_mock):
         mock_agent_options.return_value = {'c': 'd'}
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.provision_state = states.DEPLOYING
@@ -610,6 +652,44 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                 task.driver.boot, task, {'c': 'd'})
             add_provisioning_net_mock.assert_called_once_with(mock.ANY, task)
             unconfigure_tenant_net_mock.assert_called_once_with(mock.ANY, task)
+            storage_driver_info_mock.assert_called_once_with(task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'attach_volumes',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'populate_storage_driver_internal_info',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
+    def test_prepare_node_deploying_storage_should_write_false(
+            self, unconfigure_tenant_net_mock, add_provisioning_net_mock,
+            mock_prepare_ramdisk, mock_agent_options,
+            storage_driver_info_mock, storage_attach_volumes_mock,
+            storage_should_write_mock):
+        storage_should_write_mock.return_value = False
+        mock_agent_options.return_value = {'c': 'd'}
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.provision_state = states.DEPLOYING
+
+            task.driver.deploy.prepare(task)
+
+            mock_agent_options.assert_called_once_with(task.node)
+            mock_prepare_ramdisk.assert_called_once_with(
+                task.driver.boot, task, {'c': 'd'})
+            self.assertFalse(add_provisioning_net_mock.called)
+            self.assertFalse(unconfigure_tenant_net_mock.called)
+            storage_driver_info_mock.assert_called_once_with(task)
+            storage_attach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+            self.assertEqual(1, storage_should_write_mock.call_count)
 
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     @mock.patch.object(iscsi_deploy, 'check_image_size', autospec=True)
@@ -625,11 +705,48 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             mock_check_image_size.assert_called_once_with(task)
             mock_node_power_action.assert_called_once_with(task, states.REBOOT)
 
-    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
-                'unconfigure_tenant_networks', autospec=True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'configure_tenant_networks',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'remove_provisioning_network',
+                       spec_set=True, autospec=True)
+    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
+    @mock.patch.object(iscsi_deploy, 'check_image_size', autospec=True)
+    @mock.patch.object(iscsi_deploy, 'cache_instance_image', autospec=True)
+    def test_deploy_storage_check_write_image_false(self,
+                                                    mock_cache_instance_image,
+                                                    mock_check_image_size,
+                                                    mock_node_power_action,
+                                                    mock_remove_network,
+                                                    mock_tenant_network,
+                                                    mock_write):
+        mock_write.return_value = False
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+        with task_manager.acquire(self.context,
+                                  self.node.uuid, shared=False) as task:
+            state = task.driver.deploy.deploy(task)
+            self.assertEqual(state, states.DEPLOYDONE)
+            self.assertFalse(mock_cache_instance_image.called)
+            self.assertFalse(mock_check_image_size.called)
+            mock_remove_network.assert_called_once_with(mock.ANY, task)
+            mock_tenant_network.assert_called_once_with(mock.ANY, task)
+            self.assertEqual(2, mock_node_power_action.call_count)
+
+    @mock.patch.object(noop_storage.NoopStorage, 'detach_volumes',
+                       autospec=True)
+    @mock.patch.object(flat_network.FlatNetwork,
+                       'unconfigure_tenant_networks',
+                       spec_set=True, autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     def test_tear_down(self, node_power_action_mock,
-                       unconfigure_tenant_nets_mock):
+                       unconfigure_tenant_nets_mock,
+                       storage_detach_volumes_mock):
+        obj_utils.create_test_volume_target(
+            self.context, node_id=self.node.id)
         with task_manager.acquire(self.context,
                                   self.node.uuid, shared=False) as task:
             state = task.driver.deploy.tear_down(task)
@@ -638,6 +755,12 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                                                            states.POWER_OFF)
             unconfigure_tenant_nets_mock.assert_called_once_with(mock.ANY,
                                                                  task)
+            storage_detach_volumes_mock.assert_called_once_with(
+                task.driver.storage, task)
+        # Verify no volumes exist for new task instances.
+        with task_manager.acquire(self.context,
+                                  self.node.uuid, shared=False) as task:
+            self.assertEqual(0, len(task.volume_targets))
 
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory._set_dhcp_provider')
     @mock.patch('ironic.common.dhcp_factory.DHCPFactory.clean_dhcp')
@@ -703,12 +826,6 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             agent_execute_clean_step_mock.assert_called_once_with(
                 task, {'some-step': 'step-info'})
 
-    def test_heartbeat(self):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            self.driver.deploy.heartbeat(task, 'url')
-            self.assertFalse(task.shared)
-
     @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'reboot_and_finish_deploy', autospec=True)
     @mock.patch.object(iscsi_deploy, 'do_agent_iscsi_deploy', autospec=True)
