@@ -91,7 +91,9 @@ SCCICLIENT_IRMC_SCCI_SPEC = (
     'UNMOUNT_CD',
     'MOUNT_FD',
     'UNMOUNT_FD',
+    'SCCIError',
     'SCCIClientError',
+    'SCCIError',
     'SCCIInvalidInputError',
     'get_share_type',
     'get_client',
@@ -101,6 +103,25 @@ SCCICLIENT_IRMC_SCCI_SPEC = (
     'get_virtual_fd_set_params_cmd',
     'get_essential_properties',
 )
+SCCICLIENT_IRMC_ELCM_SPEC = (
+    'backup_bios_config',
+    'restore_bios_config',
+    'set_secure_boot_mode',
+)
+
+SCCICLIENT_VIOM_SPEC = (
+    'validate_physical_port_id',
+    'VIOMConfiguration',
+)
+
+SCCICLIENT_VIOM_CONF_SPEC = (
+    'set_lan_port',
+    'set_iscsi_volume',
+    'set_fc_volume',
+    'apply',
+    'dump_json',
+    'terminate',
+)
 
 ONEVIEWCLIENT_SPEC = (
     'client',
@@ -121,3 +142,22 @@ ONEVIEWCLIENT_STATES_SPEC = (
     'ONEVIEW_RESETTING',
     'ONEVIEW_ERROR',
 )
+
+SUSHY_CONSTANTS_SPEC = (
+    'BOOT_SOURCE_TARGET_PXE',
+    'BOOT_SOURCE_TARGET_HDD',
+    'BOOT_SOURCE_TARGET_CD',
+    'BOOT_SOURCE_TARGET_BIOS_SETUP',
+    'SYSTEM_POWER_STATE_ON',
+    'SYSTEM_POWER_STATE_POWERING_ON',
+    'SYSTEM_POWER_STATE_OFF',
+    'SYSTEM_POWER_STATE_POWERING_OFF',
+    'RESET_ON',
+    'RESET_FORCE_OFF',
+    'RESET_GRACEFUL_SHUTDOWN',
+    'RESET_GRACEFUL_RESTART',
+    'RESET_FORCE_RESTART',
+    'RESET_NMI',
+    'BOOT_SOURCE_ENABLED_CONTINUOUS',
+    'BOOT_SOURCE_ENABLED_ONCE',
+)
