@@ -21,17 +21,17 @@ from testtools import matchers
 from ironic.common import context
 from ironic.common import exception
 from ironic import objects
-from ironic.tests.unit.db import base
-from ironic.tests.unit.db import utils
+from ironic.tests.unit.db import base as db_base
+from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
 
 
-class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
+class TestNodeObject(db_base.DbTestCase, obj_utils.SchemasTestMixIn):
 
     def setUp(self):
         super(TestNodeObject, self).setUp()
         self.ctxt = context.get_admin_context()
-        self.fake_node = utils.get_test_node()
+        self.fake_node = db_utils.get_test_node()
         self.node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
 
     def test_get_by_id(self):
@@ -73,12 +73,15 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
 
     def test_save(self):
         uuid = self.fake_node['uuid']
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
         with mock.patch.object(self.dbapi, 'get_node_by_uuid',
                                autospec=True) as mock_get_node:
             mock_get_node.return_value = self.fake_node
             with mock.patch.object(self.dbapi, 'update_node',
                                    autospec=True) as mock_update_node:
-                mock_update_node.return_value = utils.get_test_node()
+                mock_update_node.return_value = db_utils.get_test_node(
+                    properties={"fake": "property"}, driver='fake-driver',
+                    driver_internal_info={}, updated_at=test_time)
                 n = objects.Node.get(self.context, uuid)
                 self.assertEqual({"private_state": "secret value"},
                                  n.driver_internal_info)
@@ -90,8 +93,11 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
                 mock_update_node.assert_called_once_with(
                     uuid, {'properties': {"fake": "property"},
                            'driver': 'fake-driver',
-                           'driver_internal_info': {}})
+                           'driver_internal_info': {},
+                           'version': objects.Node.VERSION})
                 self.assertEqual(self.context, n._context)
+                res_updated_at = (n.updated_at).replace(tzinfo=None)
+                self.assertEqual(test_time, res_updated_at)
                 self.assertEqual({}, n.driver_internal_info)
 
     def test_save_updated_at_field(self):
@@ -104,7 +110,7 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
             with mock.patch.object(self.dbapi, 'update_node',
                                    autospec=True) as mock_update_node:
                 mock_update_node.return_value = (
-                    utils.get_test_node(extra=extra, updated_at=test_time))
+                    db_utils.get_test_node(extra=extra, updated_at=test_time))
                 n = objects.Node.get(self.context, uuid)
                 self.assertEqual({"private_state": "secret value"},
                                  n.driver_internal_info)
@@ -119,7 +125,8 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
                     uuid, {'properties': {"fake": "property"},
                            'driver': 'fake-driver',
                            'driver_internal_info': {},
-                           'extra': {'test': 123}})
+                           'extra': {'test': 123},
+                           'version': objects.Node.VERSION})
                 self.assertEqual(self.context, n._context)
                 res_updated_at = n.updated_at.replace(tzinfo=None)
                 self.assertEqual(test_time, res_updated_at)
@@ -141,7 +148,7 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
 
     def test_save_after_refresh(self):
         # Ensure that it's possible to do object.save() after object.refresh()
-        db_node = utils.create_test_node()
+        db_node = db_utils.create_test_node()
         n = objects.Node.get_by_uuid(self.context, db_node.uuid)
         n_copy = objects.Node.get_by_uuid(self.context, db_node.uuid)
         n.name = 'b240'
@@ -209,7 +216,14 @@ class TestNodeObject(base.DbTestCase, obj_utils.SchemasTestMixIn):
 
     def test_create(self):
         node = objects.Node(self.context, **self.fake_node)
-        node.create()
+        with mock.patch.object(self.dbapi, 'create_node',
+                               autospec=True) as mock_create_node:
+            mock_create_node.return_value = db_utils.get_test_node()
+
+            node.create()
+
+            args, _kwargs = mock_create_node.call_args
+            self.assertEqual(objects.Node.VERSION, args[0]['version'])
 
     def test_create_with_invalid_properties(self):
         node = objects.Node(self.context, **self.fake_node)
