@@ -17,6 +17,7 @@ import datetime
 
 import iso8601
 import mock
+from oslo_utils import timeutils
 from oslo_versionedobjects import base as object_base
 from oslo_versionedobjects import exception as object_exception
 from oslo_versionedobjects import fixture as object_fixture
@@ -37,15 +38,21 @@ class MyObj(base.IronicObject, object_base.VersionedObjectDictCompat):
     fields = {'foo': fields.IntegerField(),
               'bar': fields.StringField(),
               'missing': fields.StringField(),
+              # nested object added as string for simplicity
+              'nested_object': fields.StringField(),
               }
 
-    def obj_make_compatible(self, primitive, target_version):
-        super(MyObj, self).obj_make_compatible(primitive, target_version)
-        if target_version == '1.4' and 'missing' in primitive:
-            del primitive['missing']
+    def _set_from_db_object(self, context, db_object, fields=None):
+        fields = set(fields or self.fields) - {'nested_object'}
+        super(MyObj, self)._set_from_db_object(context, db_object, fields)
+        # some special manipulation with nested_object here
+        self['nested_object'] = db_object.get('nested_object', '') + 'test'
 
     def obj_load_attr(self, attrname):
-        setattr(self, attrname, 'loaded!')
+        if attrname == 'version':
+            setattr(self, attrname, None)
+        else:
+            setattr(self, attrname, 'loaded!')
 
     @object_base.remotable_classmethod
     def query(cls, context):
@@ -68,6 +75,7 @@ class MyObj(base.IronicObject, object_base.VersionedObjectDictCompat):
 
     @object_base.remotable
     def save(self, context=None):
+        self.do_version_changes_for_db()
         self.obj_reset_changes()
 
     @object_base.remotable
@@ -82,6 +90,16 @@ class MyObj(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.save()
         self.foo = 42
 
+    def _convert_to_version(self, target_version,
+                            remove_unavailable_fields=True):
+        if target_version == '1.5':
+            self.missing = 'foo'
+        elif self.missing:
+            if remove_unavailable_fields:
+                delattr(self, 'missing')
+            else:
+                self.missing = ''
+
 
 class MyObj2(object):
     @classmethod
@@ -326,7 +344,7 @@ class _TestObject(object):
 
     def test_object_inheritance(self):
         base_fields = list(base.IronicObject.fields)
-        myobj_fields = ['foo', 'bar', 'missing'] + base_fields
+        myobj_fields = ['foo', 'bar', 'missing', 'nested_object'] + base_fields
         myobj3_fields = ['new_field']
         self.assertTrue(issubclass(TestSubclassedObject, MyObj))
         self.assertEqual(len(myobj_fields), len(MyObj.fields))
@@ -336,16 +354,270 @@ class _TestObject(object):
         self.assertEqual(set(myobj_fields) | set(myobj3_fields),
                          set(TestSubclassedObject.fields.keys()))
 
-    def test_get_changes(self):
+    def _test_get_changes(self, target_version='1.5'):
         obj = MyObj(self.context)
+        self.assertEqual('1.5', obj.VERSION)
+        self.assertEqual(target_version, obj.get_target_version())
         self.assertEqual({}, obj.obj_get_changes())
         obj.foo = 123
         self.assertEqual({'foo': 123}, obj.obj_get_changes())
         obj.bar = 'test'
-        self.assertEqual({'foo': 123, 'bar': 'test'}, obj.obj_get_changes())
+        obj.missing = 'test'  # field which is missing in v1.4
+        self.assertEqual({'foo': 123, 'bar': 'test', 'missing': 'test'},
+                         obj.obj_get_changes())
         obj.obj_reset_changes()
         self.assertEqual({}, obj.obj_get_changes())
 
+    def test_get_changes(self):
+        self._test_get_changes()
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test_get_changes_pinned(self, mock_release_mapping):
+        # obj_get_changes() is not affected by pinning
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[-1])
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.4'],
+            }
+        }
+        self._test_get_changes(target_version='1.4')
+
+    def test_convert_to_version_same(self):
+        # no changes
+        obj = MyObj(self.context)
+        self.assertEqual('1.5', obj.VERSION)
+        obj.convert_to_version('1.5', remove_unavailable_fields=False)
+        self.assertEqual('1.5', obj.VERSION)
+        self.assertEqual(obj.__class__.VERSION, obj.VERSION)
+        self.assertEqual({}, obj.obj_get_changes())
+
+    def test_convert_to_version_new(self):
+        obj = MyObj(self.context)
+        obj.VERSION = '1.4'
+        obj.convert_to_version('1.5', remove_unavailable_fields=False)
+        self.assertEqual('1.5', obj.VERSION)
+        self.assertEqual(obj.__class__.VERSION, obj.VERSION)
+        self.assertEqual({'missing': 'foo'}, obj.obj_get_changes())
+
+    def test_convert_to_version_old(self):
+        obj = MyObj(self.context)
+        obj.missing = 'something'
+        obj.obj_reset_changes()
+        obj.convert_to_version('1.4', remove_unavailable_fields=True)
+        self.assertEqual('1.4', obj.VERSION)
+        self.assertEqual({}, obj.obj_get_changes())
+
+    def test_convert_to_version_old_keep(self):
+        obj = MyObj(self.context)
+        obj.missing = 'something'
+        obj.obj_reset_changes()
+        obj.convert_to_version('1.4', remove_unavailable_fields=False)
+        self.assertEqual('1.4', obj.VERSION)
+        self.assertEqual({'missing': ''}, obj.obj_get_changes())
+
+    @mock.patch.object(MyObj, 'convert_to_version', autospec=True)
+    def test_do_version_changes_for_db(self, mock_convert):
+        # no object conversion
+        obj = MyObj(self.context)
+        self.assertEqual('1.5', obj.VERSION)
+        self.assertEqual('1.5', obj.get_target_version())
+        self.assertEqual({}, obj.obj_get_changes())
+        obj.foo = 123
+        obj.bar = 'test'
+        obj.missing = 'test'  # field which is missing in v1.4
+        self.assertEqual({'foo': 123, 'bar': 'test', 'missing': 'test'},
+                         obj.obj_get_changes())
+        changes = obj.do_version_changes_for_db()
+        self.assertEqual({'foo': 123, 'bar': 'test', 'missing': 'test',
+                          'version': '1.5'}, changes)
+        self.assertEqual('1.5', obj.VERSION)
+        self.assertFalse(mock_convert.called)
+
+    @mock.patch.object(MyObj, 'convert_to_version', autospec=True)
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_do_version_changes_for_db_pinned(self, mock_target_version,
+                                              mock_convert):
+        # obj is same version as pinned, no conversion done
+        mock_target_version.return_value = '1.4'
+
+        obj = MyObj(self.context)
+        obj.VERSION = '1.4'
+        self.assertEqual('1.4', obj.get_target_version())
+        obj.foo = 123
+        obj.bar = 'test'
+        self.assertEqual({'foo': 123, 'bar': 'test'}, obj.obj_get_changes())
+        self.assertEqual('1.4', obj.VERSION)
+        changes = obj.do_version_changes_for_db()
+        self.assertEqual({'foo': 123, 'bar': 'test', 'version': '1.4'},
+                         changes)
+        self.assertEqual('1.4', obj.VERSION)
+        mock_target_version.assert_called_with()
+        self.assertFalse(mock_convert.called)
+
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_do_version_changes_for_db_downgrade(self, mock_target_version):
+        # obj is 1.5; convert to 1.4
+        mock_target_version.return_value = '1.4'
+
+        obj = MyObj(self.context)
+        obj.foo = 123
+        obj.bar = 'test'
+        obj.missing = 'something'
+        self.assertEqual({'foo': 123, 'bar': 'test', 'missing': 'something'},
+                         obj.obj_get_changes())
+        self.assertEqual('1.5', obj.VERSION)
+        changes = obj.do_version_changes_for_db()
+        self.assertEqual({'foo': 123, 'bar': 'test', 'missing': '',
+                          'version': '1.4'}, changes)
+        self.assertEqual('1.4', obj.VERSION)
+        mock_target_version.assert_called_with()
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def _test__from_db_object(self, version, mock_release_mapping):
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.4'],
+            }
+        }
+        missing = ''
+        if version == '1.5':
+            missing = 'foo'
+        obj = MyObj(self.context)
+        dbobj = {'created_at': timeutils.utcnow(),
+                 'updated_at': timeutils.utcnow(),
+                 'version': version,
+                 'foo': 123, 'bar': 'test', 'missing': missing}
+        MyObj._from_db_object(self.context, obj, dbobj)
+        self.assertEqual(obj.__class__.VERSION, obj.VERSION)
+        self.assertEqual(123, obj.foo)
+        self.assertEqual('test', obj.bar)
+        self.assertEqual('foo', obj.missing)
+        self.assertFalse(mock_release_mapping.called)
+
+    def test__from_db_object(self):
+        self._test__from_db_object('1.5')
+
+    def test__from_db_object_old(self):
+        self._test__from_db_object('1.4')
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test__from_db_object_no_version(self, mock_release_mapping):
+        # DB doesn't have version; get it from mapping
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.4'],
+            }
+        }
+        obj = MyObj(self.context)
+        dbobj = {'created_at': timeutils.utcnow(),
+                 'updated_at': timeutils.utcnow(),
+                 'version': None,
+                 'foo': 123, 'bar': 'test', 'missing': ''}
+        MyObj._from_db_object(self.context, obj, dbobj)
+        self.assertEqual(obj.__class__.VERSION, obj.VERSION)
+        self.assertEqual(123, obj.foo)
+        self.assertEqual('test', obj.bar)
+        self.assertEqual('foo', obj.missing)
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test__from_db_object_no_version_subset_of_fields(self,
+                                                         mock_release_mapping):
+        # DB doesn't have version; get it from mapping
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.5'],
+            }
+        }
+        obj = MyObj(self.context)
+        dbobj = {'created_at': timeutils.utcnow(),
+                 'updated_at': timeutils.utcnow(),
+                 'version': None,
+                 'foo': 123, 'bar': 'test', 'missing': '',
+                 'nested_object': 'test'}
+        # Mock obj_load_attr as this is what is called if an attribute that we
+        # try to access is not yet set. For all ironic objects it's a noop,
+        # we've implemented it in MyObj purely for testing
+        with mock.patch.object(obj, 'obj_load_attr'):
+            MyObj._from_db_object(self.context, obj, dbobj,
+                                  fields=['foo', 'bar'])
+            self.assertEqual(obj.__class__.VERSION, obj.VERSION)
+            self.assertEqual(123, obj.foo)
+            self.assertEqual('test', obj.bar)
+            self.assertRaises(AttributeError, getattr, obj, 'missing')
+            self.assertEqual('testtest', obj.nested_object)
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test__from_db_object_map_version_bad(self, mock_release_mapping):
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.6'],
+            }
+        }
+        obj = MyObj(self.context)
+        dbobj = {'created_at': timeutils.utcnow(),
+                 'updated_at': timeutils.utcnow(),
+                 'version': None,
+                 'foo': 123, 'bar': 'test', 'missing': ''}
+        self.assertRaises(object_exception.IncompatibleObjectVersion,
+                          MyObj._from_db_object, self.context, obj, dbobj)
+
+    def test_get_target_version_no_pin(self):
+        obj = MyObj(self.context)
+        self.assertEqual('1.5', obj.get_target_version())
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test_get_target_version_pinned(self, mock_release_mapping):
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[-1])
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.4'],
+            }
+        }
+        obj = MyObj(self.context)
+        self.assertEqual('1.4', obj.get_target_version())
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test_get_target_version_pinned_no_myobj(self, mock_release_mapping):
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[-1])
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'NotMyObj': ['1.4'],
+            }
+        }
+        obj = MyObj(self.context)
+        self.assertEqual('1.5', obj.get_target_version())
+
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test_get_target_version_pinned_bad(self, mock_release_mapping):
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[-1])
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyObj': ['1.6'],
+            }
+        }
+        obj = MyObj(self.context)
+        self.assertRaises(object_exception.IncompatibleObjectVersion,
+                          obj.get_target_version)
+
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_supports_version(self, mock_target_version):
+        mock_target_version.return_value = "1.5"
+        obj = MyObj(self.context)
+        self.assertTrue(obj.supports_version((1, 5)))
+        self.assertFalse(obj.supports_version((1, 6)))
+
     def test_obj_fields(self):
         @base.IronicObjectRegistry.register_if(False)
         class TestObj(base.IronicObject,
@@ -409,34 +681,38 @@ class TestObject(_LocalTest, _TestObject):
 # The fingerprint values should only be changed if there is a version bump.
 expected_object_fingerprints = {
     'Node': '1.21-52674c214141cf3e09f8688bfed54577',
-    'MyObj': '1.5-4f5efe8f0fcaf182bbe1c7fe3ba858db',
+    'MyObj': '1.5-9459d30d6954bffc7a9afd347a807ca6',
     'Chassis': '1.3-d656e039fd8ae9f34efc232ab3980905',
-    'Port': '1.6-609504503d68982a10f495659990084b',
+    'Port': '1.7-898a47921f4a1f53fcdddd4eeb179e0b',
     'Portgroup': '1.3-71923a81a86743b313b190f5c675e258',
     'Conductor': '1.2-5091f249719d4a465062a1b3dc7f860d',
     'EventType': '1.1-aa2ba1afd38553e3880c267404e8d370',
     'NotificationPublisher': '1.0-51a09397d6c0687771fb5be9a999605d',
-    'NodePayload': '1.3-e54d6506953ad0aa0b965615b0aa38a0',
+    'NodePayload': '1.4-1ab0efe090ee3b2bd48d280a5acec1d4',
     'NodeSetPowerStateNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetPowerStatePayload': '1.3-c9657ea90d565993219bdc34e7a64231',
+    'NodeSetPowerStatePayload': '1.4-9ab5116c2c59dfb9e56dcc7ef660a431',
     'NodeCorrectedPowerStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCorrectedPowerStatePayload': '1.3-db79d69db4212e3eaf51a8b50c1d171e',
+    'NodeCorrectedPowerStatePayload': '1.4-d624154b8fb4e6cdda1554a8bf3f4cf5',
     'NodeSetProvisionStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetProvisionStatePayload': '1.3-96e85e927b10d96c79c27f5fb6727f86',
+    'NodeSetProvisionStatePayload': '1.4-605c17b779aeedf0925937b2785d9f43',
     'VolumeConnector': '1.0-3e0252c0ab6e6b9d158d09238a577d97',
     'VolumeTarget': '1.0-0b10d663d8dae675900b2c7548f76f5e',
     'ChassisCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'ChassisCRUDPayload': '1.0-dce63895d8186279a7dd577cffccb202',
     'NodeCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCRUDPayload': '1.1-35c16dd49d75812763e4e99bfebc3191',
+    'NodeCRUDPayload': '1.2-b7a265a5e2fe47adada3c7c20c68e465',
     'PortCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'PortCRUDPayload': '1.1-1ecf2d63b68014c52cb52d0227f8b5b8',
+    'PortCRUDPayload': '1.2-233d259df442eb15cc584fae1fe81504',
     'NodeMaintenanceNotification': '1.0-59acc533c11d306f149846f922739c15',
     'NodeConsoleNotification': '1.0-59acc533c11d306f149846f922739c15',
     'PortgroupCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'PortgroupCRUDPayload': '1.0-b73c1fecf0cef3aa56bbe3c7e2275018',
+    'VolumeConnectorCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
+    'VolumeConnectorCRUDPayload': '1.0-5e8dbb41e05b6149d8f7bfd4daff9339',
+    'VolumeTargetCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
+    'VolumeTargetCRUDPayload': '1.0-30dcc4735512c104a3a36a2ae1e2aeb2',
 }
 
 
@@ -485,8 +761,9 @@ class TestObjectSerializer(test_base.TestCase):
                                        mock_indirection_api,
                                        my_version='1.6'):
         ser = base.IronicObjectSerializer()
+        backported_obj = MyObj()
         mock_indirection_api.object_backport_versions.return_value \
-            = 'backported'
+            = backported_obj
 
         @base.IronicObjectRegistry.register
         class MyTestObj(MyObj):
@@ -500,7 +777,7 @@ class TestObjectSerializer(test_base.TestCase):
             self.assertFalse(
                 mock_indirection_api.object_backport_versions.called)
         else:
-            self.assertEqual('backported', result)
+            self.assertEqual(backported_obj, result)
             versions = object_base.obj_tree_get_versions('MyTestObj')
             mock_indirection_api.object_backport_versions.assert_called_with(
                 self.context, primitive, versions)
@@ -525,10 +802,36 @@ class TestObjectSerializer(test_base.TestCase):
         "Test object with unsupported (newer) version and revision"
         self._test_deserialize_entity_newer('1.7', '1.6.1', my_version='1.6.1')
 
-    @mock.patch.object(MyObj, 'obj_make_compatible')
-    def test_serialize_entity_no_backport(self, make_compatible_mock):
-        """Test single element serializer with no backport."""
-        serializer = base.IronicObjectSerializer()
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    def test_deserialize_entity_pin_ignored(self, mock_release_mapping):
+        # Deserializing doesn't look at pinning
+        CONF.set_override('pin_release_version',
+                          release_mappings.RELEASE_VERSIONS[-1])
+        mock_release_mapping.__getitem__.return_value = {
+            'objects': {
+                'MyTestObj': ['1.0'],
+            }
+        }
+        ser = base.IronicObjectSerializer()
+
+        @base.IronicObjectRegistry.register
+        class MyTestObj(MyObj):
+            VERSION = '1.1'
+
+        obj = MyTestObj(self.context)
+        primitive = obj.obj_to_primitive()
+        result = ser.deserialize_entity(self.context, primitive)
+        self.assertEqual('1.1', result.VERSION)
+        self.assertEqual('1.0', result.get_target_version())
+        self.assertFalse(mock_release_mapping.called)
+
+    @mock.patch.object(base.IronicObject, 'convert_to_version', autospec=True)
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_serialize_entity_unpinned_api(self, mock_version, mock_convert):
+        """Test single element serializer with no backport, unpinned."""
+        mock_version.return_value = MyObj.VERSION
+        serializer = base.IronicObjectSerializer(is_server=False)
         obj = MyObj(self.context)
         obj.foo = 1
         obj.bar = 'text'
@@ -541,73 +844,122 @@ class TestObjectSerializer(test_base.TestCase):
         self.assertEqual('textt', data['missing'])
         changes = primitive['ironic_object.changes']
         self.assertEqual(set(['foo', 'bar', 'missing']), set(changes))
-        make_compatible_mock.assert_not_called()
+        self.assertFalse(mock_version.called)
+        self.assertFalse(mock_convert.called)
 
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING')
-    def test_serialize_entity_backport(self, mock_release_mapping):
-        """Test single element serializer with backport."""
-        CONF.set_override('pin_release_version',
-                          release_mappings.RELEASE_VERSIONS[-1],
-                          enforce_type=True)
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {
-                'MyObj': '1.4',
-            }
-        }
-        serializer = base.IronicObjectSerializer()
+    @mock.patch.object(base.IronicObject, 'convert_to_version', autospec=True)
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_serialize_entity_unpinned_conductor(self, mock_version,
+                                                 mock_convert):
+        """Test single element serializer with no backport, unpinned."""
+        mock_version.return_value = MyObj.VERSION
+        serializer = base.IronicObjectSerializer(is_server=True)
         obj = MyObj(self.context)
         obj.foo = 1
         obj.bar = 'text'
         obj.missing = 'textt'
         primitive = serializer.serialize_entity(self.context, obj)
+        self.assertEqual('1.5', primitive['ironic_object.version'])
+        data = primitive['ironic_object.data']
+        self.assertEqual(1, data['foo'])
+        self.assertEqual('text', data['bar'])
+        self.assertEqual('textt', data['missing'])
+        changes = primitive['ironic_object.changes']
+        self.assertEqual(set(['foo', 'bar', 'missing']), set(changes))
+        mock_version.assert_called_once_with()
+        self.assertFalse(mock_convert.called)
+
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_serialize_entity_pinned_api(self, mock_version):
+        """Test single element serializer with backport to pinned version."""
+        mock_version.return_value = '1.4'
+
+        serializer = base.IronicObjectSerializer(is_server=False)
+        obj = MyObj(self.context)
+        obj.foo = 1
+        obj.bar = 'text'
+        obj.missing = 'miss'
+        self.assertEqual('1.5', obj.VERSION)
+        primitive = serializer.serialize_entity(self.context, obj)
+        self.assertEqual('1.5', primitive['ironic_object.version'])
+        data = primitive['ironic_object.data']
+        self.assertEqual(1, data['foo'])
+        self.assertEqual('text', data['bar'])
+        self.assertEqual('miss', data['missing'])
+        self.assertFalse(mock_version.called)
+
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_serialize_entity_pinned_conductor(self, mock_version):
+        """Test single element serializer with backport to pinned version."""
+        mock_version.return_value = '1.4'
+
+        serializer = base.IronicObjectSerializer(is_server=True)
+        obj = MyObj(self.context)
+        obj.foo = 1
+        obj.bar = 'text'
+        obj.missing = 'miss'
+        self.assertEqual('1.5', obj.VERSION)
+        primitive = serializer.serialize_entity(self.context, obj)
         self.assertEqual('1.4', primitive['ironic_object.version'])
         data = primitive['ironic_object.data']
         self.assertEqual(1, data['foo'])
         self.assertEqual('text', data['bar'])
         self.assertNotIn('missing', data)
-        changes = primitive['ironic_object.changes']
-        self.assertEqual(set(['foo', 'bar']), set(changes))
+        self.assertNotIn('ironic_object.changes', primitive)
+        mock_version.assert_called_once_with()
 
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING')
-    def test_serialize_entity_invalid_pin(self, mock_release_mapping):
-        CONF.set_override('pin_release_version',
-                          release_mappings.RELEASE_VERSIONS[-1],
-                          enforce_type=True)
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {
-                'MyObj': '1.6',
-            }
-        }
-        serializer = base.IronicObjectSerializer()
+    @mock.patch.object(base.IronicObject, 'get_target_version')
+    def test_serialize_entity_invalid_pin(self, mock_version):
+        mock_version.side_effect = object_exception.InvalidTargetVersion(
+            version='1.6')
+
+        serializer = base.IronicObjectSerializer(is_server=True)
         obj = MyObj(self.context)
         self.assertRaises(object_exception.InvalidTargetVersion,
                           serializer.serialize_entity, self.context, obj)
+        mock_version.assert_called_once_with()
 
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING')
-    def test_serialize_entity_no_pin(self, mock_release_mapping):
-        CONF.set_override('pin_release_version',
-                          release_mappings.RELEASE_VERSIONS[-1],
-                          enforce_type=True)
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {}
-        }
-        serializer = base.IronicObjectSerializer()
+    @mock.patch.object(base.IronicObject, 'convert_to_version', autospec=True)
+    def _test__process_object(self, mock_convert, is_server=True):
         obj = MyObj(self.context)
-        primitive = serializer.serialize_entity(self.context, obj)
-        self.assertEqual('1.5', primitive['ironic_object.version'])
+        obj.foo = 1
+        obj.bar = 'text'
+        obj.missing = 'miss'
+        primitive = obj.obj_to_primitive()
 
-    @mock.patch('ironic.objects.base.IronicObject._get_target_version')
-    @mock.patch('ironic.objects.base.LOG.warning')
-    def test_serialize_entity_unknown_entity(self, mock_warn, mock_version):
-        class Foo(object):
-            fields = {'foobar': fields.IntegerField()}
+        serializer = base.IronicObjectSerializer(is_server=is_server)
+        obj2 = serializer._process_object(self.context, primitive)
+        self.assertEqual(obj.foo, obj2.foo)
+        self.assertEqual(obj.bar, obj2.bar)
+        self.assertEqual(obj.missing, obj2.missing)
+        self.assertEqual(obj.VERSION, obj2.VERSION)
+        self.assertFalse(mock_convert.called)
 
-        serializer = base.IronicObjectSerializer()
-        obj = Foo()
-        primitive = serializer.serialize_entity(self.context, obj)
-        self.assertEqual(obj, primitive)
-        self.assertTrue(mock_warn.called)
-        mock_version.assert_not_called()
+    def test__process_object_api(self):
+        self._test__process_object(is_server=False)
+
+    def test__process_object_conductor(self):
+        self._test__process_object(is_server=True)
+
+    @mock.patch.object(base.IronicObject, 'convert_to_version', autospec=True)
+    def _test__process_object_convert(self, is_server, mock_convert):
+        obj = MyObj(self.context)
+        obj.foo = 1
+        obj.bar = 'text'
+        obj.missing = ''
+        obj.VERSION = '1.4'
+        primitive = obj.obj_to_primitive()
+
+        serializer = base.IronicObjectSerializer(is_server=is_server)
+        serializer._process_object(self.context, primitive)
+        mock_convert.assert_called_once_with(
+            mock.ANY, '1.5', remove_unavailable_fields=not is_server)
+
+    def test__process_object_convert_api(self):
+        self._test__process_object_convert(False)
+
+    def test__process_object_convert_conductor(self):
+        self._test__process_object_convert(True)
 
 
 class TestRegistry(test_base.TestCase):
